@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can addresstribute it and/or modify
@@ -25,7 +25,6 @@
 #include "ntop_includes.h"
 
 class AddressResolution {
-  AddressList localNetworks;
   int num_resolvers;
   u_int32_t num_resolved_addresses, num_resolved_fails;
   pthread_t *resolveThreadLoop;
@@ -36,15 +35,8 @@ class AddressResolution {
   ~AddressResolution();
 
   void startResolveAddressLoop();
-  void resolveHostName(char *numeric_ip, char *rsp = NULL, u_int rsp_len = 0);
-
-  inline u_int8_t getNumLocalNetworks()       { return localNetworks.getNumAddresses();    };
-  inline char *get_local_network(u_int8_t id) { return localNetworks.getAddressString(id); };
-  bool setLocalNetworks(char *rule);
-  int16_t findAddress(int family, void *addr, u_int8_t *network_mask_bits = NULL);
-  void setLocalNetwork(char *net)             { localNetworks.addAddresses(net);           };
-  void getLocalNetworks(lua_State* vm);
-  inline void dump()                          { localNetworks.dump(); }
+  void resolveHostName(const char *numeric_ip, char *rsp = NULL, u_int rsp_len = 0);
+  bool resolveHost(const char *host, char *rsp, u_int rsp_len, bool v4);
 };
 
 #endif /* _ADDRESS_RESOLUTION_H_ */
