@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,7 +31,7 @@ class AutonomousSystemHash : public GenericHash {
  public:
   AutonomousSystemHash(NetworkInterface *iface, u_int _num_hashes, u_int _max_hash_size);
 
-  AutonomousSystem* get(IpAddress *ipa);
+  AutonomousSystem* get(IpAddress *ipa, bool is_inline_call);
 
 #ifdef AS_DEBUG
   void printHash();
