@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,7 +31,7 @@ class CountriesHash : public GenericHash {
  public:
   CountriesHash(NetworkInterface *iface, u_int _num_hashes, u_int _max_hash_size);
 
-  Country* get(const char *country_name);
+  Country* get(const char *country_name, bool is_inline_call);
 };
 
 #endif /* _COUNTRIES_HASH_H_ */
