@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,32 +31,38 @@ struct queries_breakdown {
 };
 
 struct dns_stats {
-  u_int32_t num_queries, num_replies_ok, num_replies_error;
+  MonitoredCounter<u_int32_t> num_queries, num_replies_ok, num_replies_error;
   struct queries_breakdown breakdown;
 };
 
 class DnsStats {
  private:
-  struct dns_stats sent, rcvd;
+  struct dns_stats sent_stats, rcvd_stats;
 
-  void incQueryBreakdown(struct queries_breakdown *bd, u_int16_t query_type);
   void deserializeStats(json_object *o, struct dns_stats *stats);
   json_object* getStatsJSONObject(struct dns_stats *stats);
-  void luaStats(lua_State *vm, struct dns_stats *stats, const char *label);
-  void incNumDNSQueries(u_int16_t query_type, struct dns_stats *s);
+  void luaStats(lua_State *vm, struct dns_stats *stats, const char *label, bool verbose);
 
  public:
   DnsStats();
 
-  inline void incNumDNSQueriesSent(u_int16_t query_type) { incNumDNSQueries(query_type, &sent); };
-  inline void incNumDNSQueriesRcvd(u_int16_t query_type) { incNumDNSQueries(query_type, &rcvd); };
-  inline void incNumDNSResponsesSent(u_int8_t ret_code)  { if(ret_code == 0) sent.num_replies_ok++; else sent.num_replies_error++; };
-  inline void incNumDNSResponsesRcvd(u_int8_t ret_code)  { if(ret_code == 0) rcvd.num_replies_ok++; else rcvd.num_replies_error++; };
+  void incStats(bool as_client, const FlowDNSStats *fts);
+  void updateStats(const struct timeval * const tv);
+
+  u_int32_t getSentNumQueries()       { return sent_stats.num_queries.get(); }
+  u_int32_t getSentNumRepliesOk()     { return sent_stats.num_replies_ok.get(); }
+  u_int32_t getSentNumRepliesError()  { return sent_stats.num_replies_error.get(); }
+
+  u_int32_t getRcvdNumQueries()       { return rcvd_stats.num_queries.get(); }
+  u_int32_t getRcvdNumRepliesOk()     { return rcvd_stats.num_replies_ok.get(); }
+  u_int32_t getRcvdNumRepliesError()  { return rcvd_stats.num_replies_error.get(); }
 
   char* serialize();
   void deserialize(json_object *o);
   json_object* getJSONObject();
-  void lua(lua_State *vm);
+  void lua(lua_State *vm, bool verbose);
+  bool hasAnomalies(time_t when);
+  void luaAnomalies(lua_State* vm, time_t when);
 };
 
 #endif /* _STATS_H_ */
