@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,6 +24,8 @@
 
 #include "ntop_includes.h"
 
+class GenericHashEntry;
+
 /** @defgroup MonitoringData Monitoring Data
  * This is the group that contains all classes and datastructures that handle monitoring data.
  */
@@ -37,17 +39,30 @@
  */
 class GenericHash {
  protected:
-  GenericHashEntry **table; /**< Entry table. It is used for maintain an update history.*/
+  GenericHashEntry **table; /**< Entry table. It is used for maintain an update history */
   char *name;
-  u_int32_t num_hashes; /**< Number of hash.*/
-  u_int32_t current_size; /**< Current size of hash (including idle or ready-to-purge elements).*/
-  u_int32_t max_hash_size; /**< Max size of hash.*/
-  Mutex **locks, purgeLock;
-  NetworkInterface *iface; /**< Pointer of network interface for this generic hash.*/
-  u_int last_purged_hash; /**< Index of last purged hash.*/
+  u_int32_t num_hashes; /**< Number of hash */
+  u_int32_t current_size; /**< Current size of hash (including idle or ready-to-purge elements) */
+  u_int32_t max_hash_size; /**< Max size of hash */
+  u_int32_t upper_num_visited_entries; /**< Max number of entries to purge per run */
+  u_int32_t hash_mask;
+  RwLock **locks;
+  NetworkInterface *iface; /**< Pointer of network interface for this generic hash */
+  u_int32_t last_purged_hash; /**< Index of last purged hash */
+  u_int32_t last_entry_id; /**< An uniue identifier assigned to each entry in the hash table */
   u_int purge_step;
+  u_int walk_idle_start_hash_id; /**< The id of the hash bucket from which to start walkIdle hash table walk */
+  struct {
+    u_int64_t num_idle_transitions;
+    u_int64_t num_purged;
+  } entry_state_transition_counters;
+
+  vector<GenericHashEntry*> *idle_entries_in_use;   /**< Vector used by the offline thread in charge to hold idle entries but still in use */
+  vector<GenericHashEntry*> *idle_entries;          /**< Vector used by the offline thread in charge of deleting hash table entries */
+  vector<GenericHashEntry*> *idle_entries_shadow;   /**< Vector prepared by the purgeIdle and periodically swapped to idle_entries */
   
  public:
+
   /**
    * @brief A Constructor
    * @details Creating a new GenericHash.
@@ -65,6 +80,7 @@ class GenericHash {
    * @brief A Destructor
    */
   virtual ~GenericHash();
+
   /**
    * @brief Get number of entries.
    * @details Inline method.
@@ -72,29 +88,32 @@ class GenericHash {
    * @return Current size of hash.
    */
   inline u_int32_t getNumEntries() { return(current_size); };
+
+  /**
+   * @brief Get number of idle entries, that is, entries no longer in the hash table but still to be purged.
+   * @details Inline method.
+   *
+   * @return The number of idle entries.
+   */
+  u_int32_t getNumIdleEntries() const;
+
   /**
    * @brief Add new entry to generic hash.
    * @details If current_size < max_hash_size, this method calculate a new hash key for the new entry, add it and update the current_size value.
    *
    * @param h Pointer of new entry to add.
+   * @param h whether the bucket should be locked before addin the entry to the linked list.
    * @return True if the entry has been added successfully,
    *         False otherwise.
    *
    */
-  bool add(GenericHashEntry *h);
+  bool add(GenericHashEntry *h, bool do_lock);
+
   /**
-   * @brief Remove entry from generic hash
-   * @details Check if the entry is present inside the hash, remove it and update the hash.
-   *
-   * @param h Pointer of entry to remove.
-   * @return True if the entry has been remove successfully,
-   *         False otherwise.
-   * @warning GenericHashEntry* memory is NOT freed.
-   */
-  bool remove(GenericHashEntry *h);
-  /**
-   * @brief generic walker for the hash.
-   * @details This method uses the walker function to compare each elements of the hash with the user data.
+   * @brief Generic hash table walker
+   * @details This method traverses all the non-idle entries of the hash table, calling
+   *          the walker function on each of them. Function idle() is called for each entry
+   *          to evaluate its state, determine if the entry is idle, and possibly call the walker.
    *
    * @param begin_slot begin hash slot. Use 0 to walk all slots
    * @param walk_all true = walk all hash, false, walk only one (non NULL) slot
@@ -104,12 +123,23 @@ class GenericHash {
   bool walk(u_int32_t *begin_slot, bool walk_all,
 	    bool (*walker)(GenericHashEntry *h, void *user_data, bool *entryMatched), void *user_data);
 
+  /**
+   * @brief Purge idle entries that have been previous idled by purgeIdle() via periodic calls
+   * @return The number of purged entries
+   */
+  u_int64_t purgeQueuedIdleEntries();
+  
   /**
    * @brief Purge idle hash entries.
    *
+   * @param tv Timestamp of the current purge
+   * @param force_idle Forcefully marks all hash_entry_state_active entries to
+   * hash_entry_state_idle
+   * @param full_scan Force a full scan to purge all idle entries in one shot
+   *
    * @return Numbers of purged entry, 0 otherwise.
    */
-  u_int purgeIdle();
+  u_int purgeIdle(const struct timeval * tv, bool force_idle, bool full_scan);
 
   /**
    * @brief Purge all hash entries.
@@ -124,24 +154,32 @@ class GenericHash {
    * @return Pointer of network interface instance.
    */
   inline NetworkInterface* getInterface() { return(iface); };
+
   /**
-   * @brief Find an entry by key value.
+   * @brief Return the name associated with the hash.
+   * @details Inline method.
    *
-   * @param key Key value to be found in the hash.
-   * @return Pointer of entry that matches with the key parameter, NULL if there isn't entry with the key parameter or if the hash is empty.
+   * @return Pointer to the name
    */
-  GenericHashEntry* findByKey(u_int32_t key);
+  inline const char* getName() const { return name; };
 
   /**
    * @brief Check whether the hash has empty space
    *
    * @return true if there is space left, or false if the hash is full
    */
-  inline bool hasEmptyRoom() { return((current_size < max_hash_size) ? true : false); };
-  inline u_int32_t getCurrentSize() { return current_size;}
+  bool hasEmptyRoom();
+
+  /**
+   * @brief Populates a lua table with hash table stats, including
+   * the state transitions
+   *
+   * @param vm A lua VM
+   *
+   * @return Current size of hash.
+   */
+  void lua(lua_State* vm);
 
-  inline void disablePurge() { /* purgeLock.lock(__FILE__, __LINE__);   */ }
-  inline void enablePurge()  { /* purgeLock.unlock(__FILE__, __LINE__); */ }
 };
 
 #endif /* _GENERIC_HASH_H_ */
