@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2014-18 - ntop.org
+ * (C) 2014-22 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -60,14 +60,11 @@ class HTTPstats {
   struct http_response_stats last_response_sample[2];
   struct timeval last_update_time;
 
-  Mutex m;
   HostHash *h;
+  Host *host;
   bool warning_shown;
   VirtualHostHash *virtualHosts;
 
-  void incRequest(struct http_query_stats *q, const char *method);
-  void incResponse(struct http_response_stats *r, const char *method);
-
   void getRequests(const struct http_query_stats *q,
 		   u_int32_t *num_get, u_int32_t *num_post, u_int32_t *num_head,
 		   u_int32_t *num_put, u_int32_t *num_other);
@@ -99,25 +96,27 @@ class HTTPstats {
   inline u_int16_t makeRate(u_int16_t v, float tdiff) { return((u_int16_t)((((float)v* 1000)/tdiff) + .5f)); }
 
  public:
-  HTTPstats(HostHash *_h);
+  HTTPstats(Host *host);
   ~HTTPstats();
 
   inline u_int32_t get_num_virtual_hosts() { return(virtualHosts ? virtualHosts->getNumEntries() : 0); }
 
-  inline void incRequestAsReceiver(const char *method)       { incRequest(&query[AS_RECEIVER], method);        };
-  inline void incRequestAsSender(const char *method)         { incRequest(&query[AS_SENDER], method);          };
-  inline void incResponseAsSender(const char *return_code)   { incResponse(&response[AS_SENDER], return_code); };
-  inline void incResponseAsReceiver(const char *return_code) { incResponse(&response[AS_RECEIVER], return_code);};
-
+  void incStats(bool as_client, const FlowHTTPStats *fts);
   char* serialize();
   void deserialize(json_object *o);
   json_object* getJSONObject();
 
+  u_int32_t getSentNumQueries();
+  u_int32_t getSentNumResponses();
+  u_int32_t getRcvdNumQueries();
+  u_int32_t getRcvdNumResponses();
+
   void lua(lua_State *vm);
   u_int32_t luaVirtualHosts(lua_State *vm, char *virtual_host, Host *h);
 
-  void updateStats(struct timeval *tv);
-  bool updateHTTPHostRequest(char *virtual_host_name,
+  void updateStats(const struct timeval *tv);
+  bool updateHTTPHostRequest(time_t t,
+			     char *virtual_host_name,
 			     u_int32_t num_requests,
 			     u_int32_t bytes_sent,
 			     u_int32_t bytes_rcvd);
