@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -33,7 +33,7 @@ class HostHash : public GenericHash {
   HostHash(NetworkInterface *iface, u_int _num_hashes, u_int _max_hash_size);
 
   /* Search for an host by IP and VLAN */
-  Host* get(u_int16_t vlanId, IpAddress *key);
+  Host* get(VLANid vlanId, IpAddress *key, bool is_inline_call, u_int16_t observation_point_id);
 
   void incNumHTTPEntries();  
   void decNumHTTPEntries();
