@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,20 +24,39 @@
 
 #include "ntop_includes.h"
 
-class Mac : public GenericHashEntry, public GenericTrafficElement {
+class Mac : public GenericHashEntry, public SerializableElement {
  private:
+  Mutex m;
   u_int8_t mac[6];
+  u_int16_t host_pool_id;
   u_int32_t bridge_seen_iface_id; /* != 0 for bridge interfaces only */
-  char *fingerprint;
-  const char *manuf, *model, *ssid;
-  OperatingSystem os;
-  bool source_mac, special_mac, dhcpHost, lockDeviceTypeChanges;
-  ArpStats arp_stats;
+  bool special_mac, lockDeviceTypeChanges, broadcast_mac;
+  bool stats_reset_requested, data_delete_requested;
+  const char *manuf;
+  MacStats *stats, *stats_shadow;
+  time_t last_stats_reset;
+
+  struct {
+    char * dhcp; /* Extracted from DHCP dissection */
+  } names;
+
+  char * fingerprint;
+  char * model;
+  char * ssid;
+  
+  bool source_mac;
   DeviceType device_type;
 #ifdef NTOPNG_PRO
   time_t captive_portal_notified;
 #endif
+  /* END Mac data: */
+
   void checkDeviceTypeFromManufacturer();
+  void readDHCPCache();
+  void freeMacData();
+  void deleteMacData();
+  bool statsResetRequested();
+  void checkStatsReset();
 
  public:
   Mac(NetworkInterface *_iface, u_int8_t _mac[6]);
@@ -54,8 +73,6 @@ class Mac : public GenericHashEntry, public GenericTrafficElement {
   }
   inline void decUses()            { GenericHashEntry::decUses(); }
   inline bool isSpecialMac()       { return(special_mac);         }
-  inline bool isDhcpHost()         { return(dhcpHost);            }
-  inline void setDhcpHost()        { dhcpHost = true;             }
   inline bool isSourceMac()        { return(source_mac);          }
   inline void setSourceMac() {
     if(!source_mac && !special_mac) {
@@ -64,37 +81,21 @@ class Mac : public GenericHashEntry, public GenericTrafficElement {
     }
   }
 
+#ifdef HAVE_NEDGE
   MacLocation locate();
+#endif
+
   inline u_int32_t key()                       { return(Utils::macHash(mac)); }
-  inline u_int8_t* get_mac()                   { return(mac);                 }
-  inline const char * const get_manufacturer() { return manuf ? manuf : NULL; }
+  inline const u_int8_t* const get_mac() const { return(mac);                 }
+
+  static u_int64_t to64(u_int8_t mac[6]);
+  u_int64_t get_mac64();
+
+  inline const char * get_manufacturer() { return manuf ? manuf : NULL; }
   bool isNull() const;
 
   bool equal(const u_int8_t _mac[6]);
-  inline void incSentStats(u_int64_t num_pkts, u_int64_t num_bytes)  {
-    sent.incStats(num_pkts, num_bytes);
-    if(first_seen == 0) first_seen = iface->getTimeLastPktRcvd();
-    last_seen = iface->getTimeLastPktRcvd();
-  }
-  inline void incRcvdStats(u_int64_t num_pkts, u_int64_t num_bytes) {
-    rcvd.incStats(num_pkts, num_bytes);
-  }
-  inline void incnDPIStats(u_int32_t when, u_int16_t protocol,
-	    u_int64_t sent_packets, u_int64_t sent_bytes, u_int64_t sent_goodput_bytes,
-	    u_int64_t rcvd_packets, u_int64_t rcvd_bytes, u_int64_t rcvd_goodput_bytes) {
-    if(ndpiStats || (ndpiStats = new nDPIStats())) {
-      //ndpiStats->incStats(when, protocol.master_proto, sent_packets, sent_bytes, rcvd_packets, rcvd_bytes);
-      //ndpiStats->incStats(when, protocol.app_proto, sent_packets, sent_bytes, rcvd_packets, rcvd_bytes);
-      ndpiStats->incCategoryStats(when,
-				  getInterface()->get_ndpi_proto_category(protocol),
-				  sent_bytes, rcvd_bytes);
-    }
-  }
 
-  inline void incSentArpRequests()   { arp_stats.sent_requests++;         }
-  inline void incSentArpReplies()    { arp_stats.sent_replies++;          }
-  inline void incRcvdArpRequests()   { arp_stats.rcvd_requests++;         }
-  inline void incRcvdArpReplies()    { arp_stats.rcvd_replies++;          }
 #ifdef NTOPNG_PRO
   inline time_t getNotifiedTime()    { return captive_portal_notified;       };
   inline void   setNotifiedTime()    { captive_portal_notified = time(NULL); };
@@ -117,27 +118,60 @@ class Mac : public GenericHashEntry, public GenericTrafficElement {
     if(!lockDeviceTypeChanges) lockDeviceTypeChanges = true;
   }
   inline DeviceType getDeviceType()        { return (device_type); }
-  inline u_int64_t  getNumSentArp()   { return (u_int64_t)arp_stats.sent_requests + arp_stats.sent_replies; }
-  inline u_int64_t  getNumRcvdArp()   { return (u_int64_t)arp_stats.rcvd_requests + arp_stats.rcvd_replies; }
-
-  bool idle();
+  char * getDHCPName(char * const buf, ssize_t buf_size);
+  void set_hash_entry_state_idle();
   void lua(lua_State* vm, bool show_details, bool asListElement);
-  inline char* get_string_key(char *buf, u_int buf_len) { return(Utils::formatMac(mac, buf, buf_len)); };
+  virtual char* get_string_key(char *buf, u_int buf_len) const { return(Utils::formatMac(mac, buf, buf_len)); };
   inline int16_t findAddress(AddressTree *ptree)        { return ptree ? ptree->findMac(mac) : -1;     };
   inline char* print(char *str, u_int str_len)          { return(Utils::formatMac(mac, str, str_len)); };
-  char* serialize();
-  void deserialize(char *key, char *json_str);
-  json_object* getJSONObject();
-  void updateFingerprint();
-  void updateHostPool(bool isInlineCall);
-  inline void setOperatingSystem(OperatingSystem _os) { os = _os;   }
-  inline OperatingSystem getOperatingSystem()         { return(os); }
-  inline char* getFingerprint()                       { return(fingerprint); }
-  inline void setFingerprint(char *f) { if(f) { if(fingerprint) free(fingerprint); fingerprint = strdup(f); updateFingerprint(); } }
-  void setModel(char* m);
-  inline char* getModel() { return((char*)model); }
-  void setSSID(char* s);
-  inline char* getSSID()  { return((char*)ssid);  }
+  void updateHostPool(bool isInlineCall, bool firstUpdate = false);
+  void inlineSetModel(const char * m);
+  bool inlineSetFingerprint(const char * f);
+  void inlineSetSSID(const char * s);
+  void inlineSetDHCPName(const char * dhcp_name);
+  inline u_int16_t get_host_pool() { return(host_pool_id); }
+  inline const char* getFingerprint() { return(fingerprint); }
+
+  inline void requestStatsReset()                        { stats_reset_requested = true; };
+  inline void requestDataReset()                         { data_delete_requested = true; requestStatsReset(); };
+  void checkDataReset();
+
+  inline void incSentStats(time_t t, u_int64_t num_pkts, u_int64_t num_bytes)  {
+    if(first_seen == 0) first_seen = t;
+    stats->incSentStats(t, num_pkts, num_bytes), last_seen = t;
+  }
+  
+  inline void incnDPIStats(time_t when, ndpi_protocol_category_t ndpi_category,
+	    u_int64_t sent_packets, u_int64_t sent_bytes, u_int64_t sent_goodput_bytes,
+	    u_int64_t rcvd_packets, u_int64_t rcvd_bytes, u_int64_t rcvd_goodput_bytes) {
+    stats->incnDPIStats(when, ndpi_category, sent_packets, sent_bytes, sent_goodput_bytes,
+			rcvd_packets, rcvd_bytes, rcvd_goodput_bytes);
+  }
+  
+  inline void incRcvdStats(time_t t,u_int64_t num_pkts, u_int64_t num_bytes) {
+    stats->incRcvdStats(t, num_pkts, num_bytes);
+  }
+
+  void deserialize(json_object *obj);
+  void serialize(json_object *obj, DetailsLevel details_level);
+  char* getSerializationKey(char *buf, uint bufsize);
+
+  inline u_int64_t  getNumSentArp()  { return(stats->getNumSentArp());      }
+  inline u_int64_t  getNumRcvdArp()  { return(stats->getNumRcvdArp());      }
+  inline void incNumDroppedFlows()   { stats->incNumDroppedFlows(); }
+  inline void incSentArpRequests()   { stats->incSentArpRequests(); }
+  inline void incSentArpReplies()    { stats->incSentArpReplies();  }
+  inline void incRcvdArpRequests()   { stats->incRcvdArpRequests(); }
+  inline void incRcvdArpReplies()    { stats->incRcvdArpReplies();  }
+  void periodic_stats_update(const struct timeval *tv);
+  inline u_int64_t getNumBytes()              { return(stats->getNumBytes());      }
+  inline float getBytesThpt()                 { return(stats->getBytesThpt());     }
+  inline bool  isMulticast() { return Utils::isMulticastMac(mac); }
+
+  inline bool isBroadcast() { return(broadcast_mac); }
+
+  inline void incNumDHCPRequests() { stats->incNumDHCPRequests(); }
+  inline void incNumDHCPReplies()  { stats->incNumDHCPReplies();  }
 };
 
 #endif /* _MAC_H_ */
