@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,7 +31,7 @@ class MacHash : public GenericHash {
  public:
   MacHash(NetworkInterface *iface, u_int _num_hashes, u_int _max_hash_size);
 
-  Mac* get(const u_int8_t mac[6]);
+  Mac* get(const u_int8_t mac[6], bool is_inline_call);
 };
 
 #endif /* _MAC_HASH_H_ */
