@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,55 +24,52 @@
 
 #include "ntop_includes.h"
 
+#ifdef HAVE_MYSQL
+
 class MySQLDB : public DB {
  protected:
   MYSQL mysql;
   bool db_operational;
-  struct timeval lastUpdateTime;
-  u_int32_t mysqlDroppedFlows, mysqlConsumerDroppedFlows;
+  FILE *log_fd;
   u_int32_t mysqlEnqueuedFlows;
-  u_int64_t mysqlExportedFlows, mysqlLastExportedFlows;
-  float mysqlExportRate;
-
-  u_int64_t checkpointDroppedFlows, checkpointExportedFlows; /* Those will hold counters at checkpoints */
-
-  static volatile bool db_created;
+  Mutex m;
+  
+  volatile bool db_created;
   pthread_t queryThreadLoop;
 
   bool connectToDB(MYSQL *conn, bool select_db);
+  void open_log();
   char* get_last_db_error(MYSQL *conn) { return((char*)mysql_error(conn)); }
   int exec_sql_query(MYSQL *conn, const char *sql, bool doReconnect = true,
 		     bool ignoreErrors = false, bool doLock = true);
-
+  void try_exec_sql_query(MYSQL *conn, char *sql);
+  virtual bool createDBSchema();
+  bool createNprobeDBView();
+  MYSQL* mysql_try_connect(MYSQL *conn, const char *dbname);
+  int exec_quick_sql_query(char *sql, char *out, u_int out_len);
+  void mysql_result_to_lua(lua_State *vm, MYSQL_RES *result,
+			   int num_fields, bool limitRows);
+  const char *getEngineName();
+  void printFailure(const char *query, int status);
+    
  public:
   MySQLDB(NetworkInterface *_iface);
   virtual ~MySQLDB();
 
   virtual void* queryLoop();
-  virtual bool createDBSchema(bool set_db_created = true);
-  virtual bool createNprobeDBView();
-  void disconnectFromDB(MYSQL *conn);
-  static volatile bool isDbCreated() { return db_created; };
-  void checkPointCounters(bool drops_only) {
-    if(!drops_only)
-      checkpointExportedFlows = mysqlExportedFlows;
-    checkpointDroppedFlows = mysqlDroppedFlows + mysqlConsumerDroppedFlows;
-  };
-  inline u_int32_t numDroppedFlows() const { return mysqlDroppedFlows + mysqlConsumerDroppedFlows; };
-  inline float exportRate()          const { return mysqlExportRate; };
-  static char *escapeAphostrophes(const char *unescaped);
-  int flow2InsertValues(Flow *f, char *json, char *values_buf, size_t values_buf_len) const;
   virtual bool dumpFlow(time_t when, Flow *f, char *json);
-  virtual void flush() { ; };
-  int exec_sql_query(lua_State *vm, char *sql, bool limitRows, bool wait_for_db_created = true);
-  void startDBLoop();
-  void shutdown();
-  void updateStats(const struct timeval *tv);
-  void lua(lua_State* vm, bool since_last_checkpoint) const;
-#ifdef NTOPNG_PRO
-  bool dumpAggregatedFlow(time_t when, AggregatedFlow *f) { return(false); };
-#endif
 
+  void disconnectFromDB(MYSQL *conn);
+  virtual bool isDbCreated() { return db_created; };
+  char *escapeAphostrophes(const char *unescaped);
+  int flow2InsertValues(Flow *f, char *json, char *values_buf, size_t values_buf_len);
+  int exec_sql_query(lua_State *vm, char *sql, bool limitRows, bool wait_for_db_created);
+  virtual bool startQueryLoop();
+  void shutdown();
+  int exec_single_query(lua_State *vm, char *sql);
+  int select_database(char *dbname);
 };
 
+#endif
+
 #endif /* _MYSQL_DB_CLASS_H_ */
