@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,19 +31,31 @@ class NetworkDiscovery {
   int udp_sock;
   NetworkInterface *iface;
   pcap_t *pd;
-
+  lua_State* mdns_vm;
+  Mutex m;
+  struct bpf_program fcode;
+  bool has_bpf_filter;
+    
   u_int32_t wrapsum(u_int32_t sum);
   u_int16_t in_cksum(u_int8_t *buf, u_int16_t buf_len, u_int32_t sum);
   u_int16_t buildMDNSDiscoveryDatagram(const char *query, u_int32_t sender_ip, u_int8_t *sender_mac,
 				       char *datagram, u_int datagram_len);
   void dissectMDNS(u_char *buf, u_int buf_len, char *out, u_int out_len);
-    
+  inline void setMDNSvm(lua_State* vm) {
+    m.lock(__FILE__, __LINE__);
+    mdns_vm = vm;
+    m.unlock(__FILE__, __LINE__);
+  }
+  void discoverDHCP(u_char *mac);
+  
 public:
   NetworkDiscovery(NetworkInterface *_iface);
   ~NetworkDiscovery();
 
   void discover(lua_State* vm, u_int timeout);
   void arpScan(lua_State* vm);
+  void queueMDNSResponse(u_int32_t src_ip_nw_byte_order, u_char *buf, u_int buf_len);
+  void sendArpNetwork(void *data, u_int32_t netp, u_int32_t maskp, u_int32_t sender_ip);
 };
 
 #endif /* _NETWORK_DISCOVERY_H_ */
