@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,7 +27,6 @@
 class NtopGlobals {
   bool is_shutdown, shutdown_requested, do_decode_tunnels;
   time_t start_time;
-  u_int ifMTU, snaplen;
   Trace *trace;
   u_int16_t file_id;
 
@@ -35,16 +34,16 @@ class NtopGlobals {
   NtopGlobals();
   ~NtopGlobals();
 
-  inline u_int getUptime()             { return((u_int)(time(NULL)-start_time+1)); };
-  inline u_int getIfMTU()              { return(ifMTU);              };
-  inline u_int getSnaplen()            { return(snaplen);            };
-  inline Trace *getTrace()             { return(trace);              }; 
-  inline bool  decode_tunnels()        { return(do_decode_tunnels);  };
-  inline bool  isShutdown()            { return(is_shutdown);        };
-  inline void  shutdown()              { is_shutdown = true;         };
-  inline bool  isShutdownRequested()   { return(shutdown_requested); };
-  inline void  requestShutdown()       { shutdown_requested = true;  };
-  
+  inline u_int getUptime()                     const { return((u_int)(time(NULL)-start_time+1)); };
+  inline Trace *getTrace()                     const { return(trace);                   }; 
+  inline bool  decode_tunnels()                const { return(do_decode_tunnels);       };
+  inline bool  isShutdown()                    const { return(is_shutdown);             };
+  inline bool  isShutdownRequested()           const { return(shutdown_requested);      };
+  inline void  shutdown()                            { is_shutdown = true;              };
+  inline void  requestShutdown()                     { shutdown_requested = true;       };
+  inline u_int getSnaplen(const char * ifname) const {
+    return(Utils::getIfMTU(ifname) + 14 /* Ethernet */ + 4 /* VLAN */ + 4 /* QinQ */);
+  };
   char* get_temp_filename(char *buf, u_int buf_len);
 };
 
