@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,13 +28,21 @@ class PeriodicActivities {
  private:
   ThreadedActivity *activities[CONST_MAX_NUM_THREADED_ACTIVITIES];
   u_int16_t num_activities;
-
+  ThreadPool *th_pool;
+  pthread_t pthreadLoop;
+  bool      thread_running;
+  
+  u_int8_t getNumThreadsPerPool(const char* path, std::vector<char*> *iface_scripts_list, std::vector<char*> *system_scripts_list);
+  
  public:
   PeriodicActivities();
   ~PeriodicActivities();
 
   void startPeriodicActivitiesLoop();
-  void sendShutdownSignal();
+  void lua(NetworkInterface *iface, lua_State *vm);
+  void run();
+
+  inline bool isRunning() { return(thread_running); }
 };
 
 #endif /* _PERIODIC_ACTIVITIES_H_ */
