@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-22 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -43,9 +43,11 @@ class Prefs {
   char *http_binding_address1, *http_binding_address2;
   char *https_binding_address1, *https_binding_address2;
   bool enable_client_x509_auth, reproduce_at_original_speed;
-  char *lan_interface, *wan_interface, *zmq_publish_events_url;
+  char *zmq_publish_events_url;
+  const char *clickhouse_client, *clickhouse_cluster_name;
   Ntop *ntop;
-  bool enable_dns_resolution, sniff_dns_responses, pcap_file_purge_hosts_flows,
+  bool enable_dns_resolution, sniff_dns_responses, sniff_name_responses,
+    sniff_local_name_responses, pcap_file_purge_hosts_flows,
     categorization_enabled, resolve_all_host_ip, change_user, daemonize,
     enable_auto_logout, enable_auto_logout_at_runtime, use_promiscuous_mode,
     enable_ixia_timestamps, enable_vss_apcon_timestamps, enable_interface_name_only,
@@ -53,13 +55,13 @@ class Prefs {
     service_license_check, enable_sql_log, enable_access_log, log_to_file,
     enable_mac_ndpi_stats, enable_activities_debug, enable_behaviour_analysis,
     enable_asn_behaviour_analysis, enable_network_behaviour_analysis, enable_iface_l7_behaviour_analysis,
-    emit_flow_alerts, emit_host_alerts, use_clickhouse;
+    emit_flow_alerts, emit_host_alerts, dump_flows_on_clickhouse;
   u_int32_t behaviour_analysis_learning_period;
-  u_int32_t iec60870_learning_period;
+  u_int32_t iec60870_learning_period, devices_learning_period;
   ServiceAcceptance behaviour_analysis_learning_status_during_learning,
     behaviour_analysis_learning_status_post_learning;
   TsDriver timeseries_driver;
-  u_int64_t iec104_allowed_typeids[2];  
+  u_int64_t iec104_allowed_typeids[2];
   u_int32_t auth_session_duration;
   bool auth_session_midnight_expiration;
 
@@ -71,7 +73,7 @@ class Prefs {
   u_int32_t housekeeping_frequency;
   bool disable_alerts, enable_top_talkers, enable_idle_local_hosts_cache,
     enable_active_local_hosts_cache;
-  bool enable_flow_device_port_rrd_creation, enable_observation_points_rrd_creation;
+  bool enable_flow_device_port_rrd_creation, enable_observation_points_rrd_creation, enable_intranet_traffic_rrd_creation;
   bool enable_tiny_flows_export;
   bool enable_captive_portal, enable_informative_captive_portal, mac_based_captive_portal;
   bool override_dst_with_post_nat_dst, override_src_with_post_nat_src;
@@ -81,10 +83,13 @@ class Prefs {
   bool enable_zmq_encryption;
   bool flow_table_time, flow_table_probe_order;
   bool enable_broadcast_domain_too_large;
+#ifdef NTOPNG_PRO
+  bool create_labels_logfile;
+#endif
   u_int32_t max_num_secs_before_delete_alert, alert_page_refresh_rate;
   int32_t max_entity_alerts;
   u_int32_t safe_search_dns_ip, global_primary_dns_ip, global_secondary_dns_ip;
-  u_int32_t max_num_packets_per_tiny_flow, max_num_bytes_per_tiny_flow;
+  u_int32_t max_num_packets_per_tiny_flow, max_num_bytes_per_tiny_flow, dump_frequency;
   u_int32_t max_extracted_pcap_bytes;
   u_int32_t max_ui_strlen;
   u_int8_t default_l7policy;
@@ -97,12 +102,12 @@ class Prefs {
   u_int http_port, https_port;
   u_int8_t num_interfaces;
   u_int16_t auto_assigned_pool_id;
-  bool dump_flows_on_es, dump_flows_on_mysql, dump_flows_on_syslog, dump_flows_on_nindex,
-    dump_json_flows_on_disk, load_json_flows_from_disk_to_nindex, dump_ext_json;
+  bool dump_flows_on_es, dump_flows_on_mysql, dump_flows_on_syslog,
+    dump_json_flows_on_disk, dump_ext_json;
 #ifdef NTOPNG_PRO
   bool dump_flows_direct;
   bool is_geo_map_score_enabled, is_geo_map_asname_enabled, is_geo_map_alerted_flows_enabled,
-    is_geo_map_blacklisted_flows_enabled, is_geo_map_host_name_enabled, 
+    is_geo_map_blacklisted_flows_enabled, is_geo_map_host_name_enabled,
     is_geo_map_rxtx_data_enabled, is_geo_map_num_flows_enabled;
 #endif
   bool enable_runtime_flows_dump; /**< runtime preference to enable/disable flows dump from the UI */
@@ -117,6 +122,9 @@ class Prefs {
     , *pro_callbacks_dir
 #endif
     ;
+#if defined(HAVE_KAFKA) && defined(NTOPNG_PRO)
+  char *kafka_brokers_list, *kafka_topic, *kafka_options;
+#endif
   char *categorization_key;
   char *zmq_encryption_pwd;
   char *zmq_encryption_priv_key;
@@ -140,12 +148,13 @@ class Prefs {
   bool json_labels_string_format;
   char *es_type, *es_index, *es_url, *es_user, *es_pwd, *es_host;
   char *mysql_host, *mysql_dbname, *mysql_user, *mysql_pw;
-#ifndef WIN32
+#if !defined(WIN32) && !defined(__APPLE__)
   int flows_syslog_facility;
 #endif
   int mysql_port;
+  int clickhouse_tcp_port;
   char *ls_host,*ls_port,*ls_proto;
-  bool has_cmdl_trace_lvl; /**< Indicate whether a verbose level 
+  bool has_cmdl_trace_lvl; /**< Indicate whether a verbose level
 			      has been provided on the command line.*/
 #ifndef HAVE_NEDGE
   bool appliance;
@@ -156,6 +165,7 @@ class Prefs {
 #endif
 
   char *test_pre_script_path;
+  char *test_runtime_script_path;
   char *test_post_script_path;
 
 #ifdef NTOPNG_PRO
@@ -163,6 +173,8 @@ class Prefs {
 #endif
   bool print_version, print_version_json;
 
+  std::set<std::string> lan_interfaces, wan_interfaces;
+
   inline void help()      { usage();     }
   inline void nDPIhelp()  { nDPIusage(); }
   void setCommandLineString(int optkey, const char * optarg);
@@ -173,7 +185,7 @@ class Prefs {
   void parseHTTPPort(char *arg);
   char *parseLocalNetworks(char *arg);
 
-  static inline void set_binding_address(char ** const dest, const char * const addr) {
+  static inline void set_binding_address(char ** const dest, const char * addr) {
     if(dest && addr && addr[0] != '\0') {
       if(*dest) free(*dest);
       *dest = strdup(addr);
@@ -181,7 +193,7 @@ class Prefs {
   };
   bool getDefaultBoolPrefsValue(const char *pref_key, const bool default_value);
   void refreshBehaviourAnalysis();
-  
+
  public:
   Prefs(Ntop *_ntop);
   virtual ~Prefs();
@@ -189,9 +201,10 @@ class Prefs {
   bool is_pro_edition();
   bool is_enterprise_m_edition();
   bool is_enterprise_l_edition();
+  bool is_enterprise_xl_edition();
   bool is_nedge_edition();
   bool is_nedge_enterprise_edition();
-  
+
   inline bool is_embedded_edition() {
 #ifdef NTOPNG_EMBEDDED_EDITION
     return(true);
@@ -208,8 +221,12 @@ class Prefs {
   inline bool is_users_login_enabled()                  { return(enable_users_login);     };
   inline bool is_localhost_users_login_disabled()       { return(disable_localhost_login);};
   inline bool is_log_to_file_enabled()                  { return(log_to_file);            };
-  inline void disable_dns_responses_decoding()          { sniff_dns_responses = false;    };  
-  inline bool decode_dns_responses()                    { return(sniff_dns_responses);    };
+  inline void disable_dns_responses_decoding()          { sniff_dns_responses = false;    };
+  inline void disable_localhost_name_decoding()         { sniff_local_name_responses = false; };
+  inline void disable_all_name_decoding()               { sniff_name_responses = false;   };
+  inline bool is_dns_decoding_enabled() /* DNS only */  { return(sniff_dns_responses);    };
+  inline bool is_name_decoding_enabled() /* Any */      { return(sniff_name_responses);   };
+  inline bool is_localhost_name_decoding_enabled()      { return(sniff_local_name_responses);   };
   inline void enable_categorization()                   { categorization_enabled = true;  };
   inline bool is_categorization_enabled()               { return(categorization_enabled); };
   inline bool do_change_user()                          { return(change_user);            };
@@ -226,19 +243,29 @@ class Prefs {
   inline u_int8_t get_num_user_specified_interfaces()   { return(num_interfaces);         };
   inline bool  do_dump_flows_on_es()                    { return(dump_flows_on_es);       };
   inline bool  do_dump_flows_on_mysql()                 { return(dump_flows_on_mysql);    };
+  inline bool  do_dump_flows_on_clickhouse()            { return(is_enterprise_m_edition() && dump_flows_on_clickhouse); };
+  inline bool  do_dump_alerts_on_clickhouse()           { return(do_dump_flows_on_clickhouse()); };
   inline bool  do_dump_flows_on_syslog()                { return(dump_flows_on_syslog);   };
-  inline bool  do_dump_flows_on_nindex()                { return(dump_flows_on_nindex);   };
+#if defined(HAVE_KAFKA) && defined(NTOPNG_PRO)
+  inline bool  do_dump_flows_on_kafka()                 { return((kafka_brokers_list && kafka_topic) ? true : false);   };
+#endif
   inline bool  do_dump_extended_json()                  { return(dump_ext_json);          };
   inline bool  do_dump_json_flows_on_disk()             { return(dump_json_flows_on_disk);};
-  inline bool  do_load_json_flows_from_disk_to_nindex() { return(load_json_flows_from_disk_to_nindex); };
-  inline bool  do_dump_flows() const                    { return(dump_flows_on_es || dump_flows_on_mysql || dump_flows_on_syslog || dump_flows_on_nindex); };
+  inline bool  do_dump_flows()                          { return(do_dump_flows_on_es()
+								 || do_dump_flows_on_mysql()
+								 || do_dump_flows_on_clickhouse()
+								 || do_dump_flows_on_syslog()
+#if defined(HAVE_KAFKA) && defined(NTOPNG_PRO)
+								 || do_dump_flows_on_kafka()
+#endif
+								 ); };
 
 #ifdef NTOPNG_PRO
   inline void  toggle_dump_flows_direct(bool enable)    { dump_flows_direct = enable; };
   inline bool  do_dump_flows_direct()                   { return(dump_flows_direct); };
 #endif
   inline bool is_runtime_flows_dump_enabled()     const { return(enable_runtime_flows_dump); };
-  inline bool is_flows_dump_enabled()             const { return(do_dump_flows() && is_runtime_flows_dump_enabled()); };
+  inline bool is_flows_dump_enabled()                   { return(do_dump_flows() && is_runtime_flows_dump_enabled()); };
     
   int32_t getDefaultPrefsValue(const char *pref_key, int32_t default_value);
   void getDefaultStringPrefsValue(const char *pref_key, char **buffer, const char *default_value);
@@ -246,15 +273,18 @@ class Prefs {
   char* get_if_descr(int id);
   inline const char* get_config_file_path()                   { return(config_file_path); };
   inline const char* get_ndpi_proto_file_path()               { return(ndpi_proto_path); };
+  void set_data_dir(char* path)                          { data_dir = path; }
   inline char* get_data_dir()                                 { return(data_dir);       };
   inline char* get_docs_dir()                                 { return(docs_dir);       }; // HTTP docs
   inline const char* get_scripts_dir()                        { return(scripts_dir);    };
   inline const char* get_callbacks_dir()                      { return(callbacks_dir);  };
+  void set_callback_dir(char *path)                     { callbacks_dir = path;   };
   inline const char* get_pcap_dir()                           { return(pcap_dir);       };
 #ifdef NTOPNG_PRO
   inline const char* get_pro_callbacks_dir()                  { return(pro_callbacks_dir); };
 #endif
   inline const char* get_test_pre_script_path()               { return(test_pre_script_path); };
+  inline const char* get_test_runtime_script_path()           { return(test_runtime_script_path); };
   inline const char* get_test_post_script_path()              { return(test_post_script_path); };
   inline char* get_export_endpoint()                    { return(export_endpoint);};
   inline char* get_export_zmq_encryption_key()          { return(export_zmq_encryption_key); };
@@ -314,8 +344,8 @@ class Prefs {
   /* Runtime database dump prefs. Allows the user to toggle flows dump from the UI at runtime. */
   void refreshDbDumpPrefs();
 
-  void bind_http_to_address(const char * const addr1, const char * const addr2);
-  void bind_https_to_address(const char * const addr1, const char * const addr2);
+  void bind_http_to_address(const char * addr1, const char * addr2);
+  void bind_https_to_address(const char * addr1, const char * addr2);
   void bind_http_to_loopback()  { bind_http_to_address((char*)CONST_LOOPBACK_ADDRESS, (char*)CONST_LOOPBACK_ADDRESS);  };
   inline void bind_https_to_loopback() { bind_https_to_address((char*)CONST_LOOPBACK_ADDRESS, (char*)CONST_LOOPBACK_ADDRESS); };
   inline void get_http_binding_addresses(const char** addr1, const char** addr2) { *addr1=http_binding_address1; *addr2=http_binding_address2; };
@@ -334,11 +364,12 @@ class Prefs {
   inline bool use_promiscuous()         { return(use_promiscuous_mode);  };
   inline char* get_mysql_host()         { return(mysql_host);            };
   inline int get_mysql_port()           { return(mysql_port);            };
+  inline int get_clickhouse_tcp_port()  { return(clickhouse_tcp_port);   };
   inline char* get_mysql_dbname()       { return(mysql_dbname);          };
   inline char* get_mysql_tablename()    { return((char*)"flows");        };
   inline char* get_mysql_user()         { return(mysql_user);            };
   inline char* get_mysql_pw()           { return(mysql_pw);              };
-#ifndef WIN32
+#if !defined(WIN32) && !defined(__APPLE__)
   inline int get_flows_syslog_facility() { return(flows_syslog_facility); };
 #endif
   inline char* get_ls_host()            { return(ls_host);               };
@@ -348,10 +379,14 @@ class Prefs {
   inline char* get_zmq_encryption_priv_key() { return(zmq_encryption_priv_key); };
   inline bool  is_zmq_encryption_enabled() { return(enable_zmq_encryption); };
   inline char* get_command_line()       { return(cli ? cli : (char*)""); };
-  inline char* get_lan_interface()      { return(lan_interface ? lan_interface : (char*)""); };
-  inline char* get_wan_interface()      { return(wan_interface ? wan_interface : (char*)""); };
-  inline void set_lan_interface(char *iface) { if(lan_interface) free(lan_interface); lan_interface = strdup(iface); };
-  inline void set_wan_interface(char *iface) { if(wan_interface) free(wan_interface); wan_interface = strdup(iface); };
+  inline const char* get_first_lan_interface() { return(get_num_lan_interfaces() ? lan_interfaces.begin()->c_str() : (char*)""); };
+  inline const char* get_first_wan_interface() { return(get_num_wan_interfaces() ? wan_interfaces.begin()->c_str() : (char*)""); };
+  inline void add_lan_interface(char *iface) { lan_interfaces.insert(iface); };
+  inline void add_wan_interface(char *iface) { wan_interfaces.insert(iface); };
+  inline bool is_lan_interface(char *iface) { return lan_interfaces.find(iface) != lan_interfaces.end(); }
+  inline bool is_wan_interface(char *iface) { return wan_interfaces.find(iface) != wan_interfaces.end(); }
+  inline int get_num_lan_interfaces() { return lan_interfaces.size(); }
+  inline int get_num_wan_interfaces() { return wan_interfaces.size(); }
   inline bool areMacNdpiStatsEnabled()  { return(enable_mac_ndpi_stats); };
   inline pcap_direction_t getCaptureDirection() { return(captureDirection); };
   inline void setCaptureDirection(pcap_direction_t dir) { captureDirection = dir; };
@@ -368,7 +403,6 @@ class Prefs {
   inline u_int32_t get_alerted_host_max_idle()       const { return(local_host_max_idle); /* Treat all hosts as local */         };
   inline u_int32_t get_local_host_cache_duration()      { return(local_host_cache_duration);   };
   inline u_int32_t get_pkt_ifaces_flow_max_idle()       { return(pkt_ifaces_flow_max_idle);    };
-  inline bool  are_alerts_disabled()                    { return(disable_alerts);              };
   inline bool  are_top_talkers_enabled()                { return(enable_top_talkers);     };
   inline bool  flow_table_duration_or_last_seen()       { return(flow_table_time);     };
   inline bool  is_idle_local_host_cache_enabled()       { return(enable_idle_local_hosts_cache);    };
@@ -377,6 +411,7 @@ class Prefs {
   inline bool is_tiny_flows_export_enabled()             { return(enable_tiny_flows_export);            };
   inline bool is_flow_device_port_rrd_creation_enabled() { return(enable_flow_device_port_rrd_creation);};
   inline bool is_observation_points_rrd_creation_enabled() { return(enable_observation_points_rrd_creation);};
+  inline bool is_intranet_traffic_rrd_creation_enabled()   { return(enable_intranet_traffic_rrd_creation);};
   inline bool is_arp_matrix_generation_enabled()         { return(enable_arp_matrix_generation);        };
 
   inline bool do_override_dst_with_post_nat_dst()     const { return(override_dst_with_post_nat_dst);     };
@@ -388,11 +423,12 @@ class Prefs {
   inline bool isInformativeCaptivePortalEnabled()     const { return(enable_informative_captive_portal && !enable_vlan_trunk_bridge); }
   inline bool isMacBasedCaptivePortal()               const { return(mac_based_captive_portal);  }
   inline bool enableActivitiesDebug()                 const { return(enable_activities_debug);   }
-  const char * const getCaptivePortalUrl();
+  const char * getCaptivePortalUrl();
   const TsDriver getTimeseriesDriver()                const { return(timeseries_driver);         }
 
   inline u_int8_t  getDefaultl7Policy()                  { return(default_l7policy);  }
 
+  inline u_int32_t get_dump_frequency()                      const { return(dump_frequency);                      };
   inline u_int32_t get_max_num_packets_per_tiny_flow()       const { return(max_num_packets_per_tiny_flow);       };
   inline u_int32_t get_max_num_bytes_per_tiny_flow()         const { return(max_num_bytes_per_tiny_flow);         };
 
@@ -415,7 +451,7 @@ class Prefs {
   inline void      enableBehaviourAnalysis()     { enable_behaviour_analysis = true;                    };
   inline bool      isBehavourAnalysisEnabled()   { return(enable_behaviour_analysis);                   };
   inline u_int32_t behaviourAnalysisLearningPeriod() { return behaviour_analysis_learning_period;       };
-  
+
   inline bool      isBroadcastDomainTooLargeEnabled()     { return(enable_broadcast_domain_too_large);  };
 
   inline bool      isASNBehavourAnalysisEnabled()     { return(enable_asn_behaviour_analysis);               };
@@ -424,15 +460,26 @@ class Prefs {
 
   inline ServiceAcceptance behaviourAnalysisStatusDuringLearning() { return behaviour_analysis_learning_status_during_learning; };
   inline ServiceAcceptance behaviourAnalysisStatusPostLearning()   { return behaviour_analysis_learning_status_post_learning;   };
-  inline u_int64_t* getIEC104AllowedTypeIDs()    { return(iec104_allowed_typeids);                      };
-  inline u_int32_t getIEC60870LearingPeriod()    { return(iec60870_learning_period);                    };
-  inline bool      dontEmitFlowAlerts()          { return(!emit_flow_alerts);                           };
-  inline bool      dontEmitHostAlerts()          { return(!emit_host_alerts);                           };
-  inline bool      useClickHouse()               { return(use_clickhouse);                              };
-  inline char*     getZMQPublishEventsURL()      { return(zmq_publish_events_url);                      };
-  void setIEC104AllowedTypeIDs(const char * const protos);
+  inline u_int64_t*  getIEC104AllowedTypeIDs()   { return(iec104_allowed_typeids);                      };
+  inline u_int32_t   getIEC60870LearingPeriod()  { return(iec60870_learning_period);                    };
+  inline u_int32_t   devicesLearingPeriod()      { return(devices_learning_period);                     };
+  inline bool        are_alerts_disabled()       { return(disable_alerts);                              };
+  inline bool        dontEmitFlowAlerts()        { return(disable_alerts || !emit_flow_alerts);         };
+  inline bool        dontEmitHostAlerts()        { return(disable_alerts || !emit_host_alerts);         };
+  inline void        dontUseClickHouse()         { dump_flows_on_clickhouse = dump_flows_on_mysql = false; };
+  inline char*       getZMQPublishEventsURL()    { return(zmq_publish_events_url);                      };
+  inline const char* getClickHouseClientPath()   { return(clickhouse_client);                           };
+  inline const char* getClickHouseClusterName()  { return(clickhouse_cluster_name);                     };
+#ifdef NTOPNG_PRO
+  inline bool        isLabelDumpEnabled()        { return(create_labels_logfile);                       };
+#endif
+  void setIEC104AllowedTypeIDs(const char * protos);
   void validate();
+#if defined(HAVE_KAFKA) && defined(NTOPNG_PRO)
+  char* getKakfaBrokersList() { return(kafka_brokers_list); }
+  char* getKafkaTopic()       { return(kafka_topic);        }
+  char* getKafkaOptions()     { return(kafka_options);      }
+#endif
 };
 
 #endif /* _PREFS_H_ */
-
