@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,18 +26,17 @@
 
 class RemoteHost : public Host {
  private:
-  bool remote_to_remote_alerts;
   void initialize();
 
  public:
-  RemoteHost(NetworkInterface *_iface, Mac *_mac, u_int16_t _vlanId, IpAddress *_ip);
-  RemoteHost(NetworkInterface *_iface, char *ipAddress, u_int16_t _vlanId);
+  RemoteHost(NetworkInterface *_iface, Mac *_mac, VLANid _vlanId, u_int16_t _observation_point_id, IpAddress *_ip);
+  RemoteHost(NetworkInterface *_iface, char *ipAddress, VLANid _vlanId, u_int16_t _observation_point_id);
   virtual ~RemoteHost();
 
-  virtual bool setRemoteToRemoteAlerts();
-  virtual int16_t get_local_network_id() { return(-1);                };
-  virtual bool isLocalHost()             { return(false);             };
-  virtual bool isSystemHost()            { return(false);             };
+  void set_hash_entry_state_idle();
+  virtual int16_t get_local_network_id() const { return(-1);                };
+  virtual bool isLocalHost()  const            { return(false);             };
+  virtual bool isSystemHost() const            { return(false);             };
 };
 
 #endif /* _REMOTE_HOST_H_ */
