@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -40,7 +40,7 @@ class TcpFlowStats {
   void deserialize(json_object *o);
   json_object* getJSONObject();
   void lua(lua_State* vm, const char *label);
-  inline void sum(TcpFlowStats *s) {
+  inline void sum(TcpFlowStats *s) const {
     s->numSynFlows += numSynFlows, s->numEstablishedFlows += numEstablishedFlows,
       s->numResetFlows += numResetFlows, s->numFinFlows += numFinFlows;
   };
