@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -35,6 +35,7 @@ typedef struct {
 typedef struct {
   TrafficCounter packets, bytes;
   u_int32_t duration /* sec */, last_epoch_update; /* useful to avoid multiple updates */
+  u_int32_t total_flows;
 } ProtoCounter;
 
 typedef struct {
@@ -43,20 +44,28 @@ typedef struct {
 } CategoryCounter;
 
 class NetworkInterface;
+class ThroughputStats;
 
 /* *************************************** */
 
 class nDPIStats {
  private:
+#ifdef NTOPNG_PRO
+  time_t nextMinPeriodicUpdate;
+  BehaviorAnalysis **behavior_bytes_traffic;
+#endif
   ProtoCounter *counters[MAX_NDPI_PROTOS];
+  ThroughputStats **bytes_thpt;
   /* NOTE: category counters are not dumped to redis right now, they are only used internally */
   CategoryCounter cat_counters[NDPI_PROTOCOL_NUM_CATEGORIES];
 
  public:
-  nDPIStats();
+  nDPIStats(bool enable_throughput_stats = false, bool enable_behavior_stats = false);
   nDPIStats(const nDPIStats &stats);
   ~nDPIStats();
 
+  void updateStats(const struct timeval *tv);
+
   void incStats(u_int32_t when, u_int16_t proto_id,
 		u_int64_t sent_packets, u_int64_t sent_bytes,
 		u_int64_t rcvd_packets, u_int64_t rcvd_bytes);
@@ -64,13 +73,14 @@ class nDPIStats {
   void incCategoryStats(u_int32_t when, ndpi_protocol_category_t category_id,
           u_int64_t sent_bytes, u_int64_t rcvd_bytes);
 
+  void incFlowsStats(u_int16_t proto_id);
+
   void print(NetworkInterface *iface);
-  void lua(NetworkInterface *iface, lua_State* vm, bool with_categories = false, bool tsLua = false);
+  void lua(NetworkInterface *iface, lua_State* vm, bool with_categories = false, bool tsLua = false, bool diff = false);
   char* serialize(NetworkInterface *iface);
   json_object* getJSONObject(NetworkInterface *iface);
-  json_object* getJSONObjectForCheckpoint(NetworkInterface *iface);
   void deserialize(NetworkInterface *iface, json_object *o);
-  void sum(nDPIStats *s);
+  void sum(nDPIStats *s) const;
 
   inline u_int64_t getProtoBytes(u_int16_t proto_id) { 
     if((proto_id < MAX_NDPI_PROTOS) && counters[proto_id]) {
