@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,6 +28,8 @@
 #ifndef _NTOP_FLOW_H_
 #define _NTOP_FLOW_H_
 
+#define UNKNOWN_FLOW_ELEMENT (u_int32_t)-1
+#define UNKNOWN_PEN          (u_int32_t)-1
 #define NTOP_PEN     35632
 #define NTOP_BASE_ID 57472
 
@@ -115,6 +117,8 @@
 #define PACKET_SECTION_OFFSET   102
 #define SAMPLED_PACKET_SIZE   103
 #define SAMPLED_PACKET_ID   104
+#define BGP_NEXT_ADJACENT_ASN 128
+#define BGP_PREV_ADJACENT_ASN 129
 #define EXPORTER_IPV4_ADDRESS   130
 #define EXPORTER_IPV6_ADDRESS   131
 #define FLOW_ID   148
@@ -151,16 +155,19 @@
 #define PACKETS_OBSERVED   318
 #define PACKETS_SELECTED   319
 #define SELECTOR_NAME   335
-#define FRAGMENTS   NTOP_BASE_ID+80
+#define SRC_FRAGMENTS   NTOP_BASE_ID+80
+#define DST_FRAGMENTS   NTOP_BASE_ID+81
 #define SRC_AS_MAP  NTOP_BASE_ID+443
 #define DST_AS_MAP  NTOP_BASE_ID+444
 #define NPROBE_IPV4_ADDRESS  NTOP_BASE_ID+471 /* Exported only via ZMQ */
 /* NTOP_BASE_ID+82 ->87  Available (deprecated) */
-#define NUM_PKTS_UP_TO_128_BYTES   NTOP_BASE_ID+88
+#define CLIENT_TCP_FLAGS            NTOP_BASE_ID+78
+#define SERVER_TCP_FLAGS            NTOP_BASE_ID+79
+#define NUM_PKTS_UP_TO_128_BYTES    NTOP_BASE_ID+88
 #define NUM_PKTS_128_TO_256_BYTES   NTOP_BASE_ID+89
 #define NUM_PKTS_256_TO_512_BYTES   NTOP_BASE_ID+90
-#define NUM_PKTS_512_TO_1024_BYTES   NTOP_BASE_ID+91
-#define NUM_PKTS_1024_TO_1514_BYTES   NTOP_BASE_ID+92
+#define NUM_PKTS_512_TO_1024_BYTES  NTOP_BASE_ID+91
+#define NUM_PKTS_1024_TO_1514_BYTES NTOP_BASE_ID+92
 #define NUM_PKTS_OVER_1514_BYTES   NTOP_BASE_ID+93
 #define CUMULATIVE_ICMP_TYPE NTOP_BASE_ID+98
 #define SRC_IP_COUNTRY   NTOP_BASE_ID+101
@@ -180,16 +187,18 @@
 #define UNTUNNELED_L4_SRC_PORT   NTOP_BASE_ID+115
 #define UNTUNNELED_IPV4_DST_ADDR NTOP_BASE_ID+116
 #define UNTUNNELED_L4_DST_PORT   NTOP_BASE_ID+117
-#define L7_PROTO              NTOP_BASE_ID+118
-#define L7_PROTO_NAME         NTOP_BASE_ID+119
-#define DOWNSTREAM_TUNNEL_ID  NTOP_BASE_ID+120
-#define FLOW_USER_NAME        NTOP_BASE_ID+121
-#define FLOW_SERVER_NAME      NTOP_BASE_ID+122
-#define CLIENT_NW_LATENCY_MS    NTOP_BASE_ID+123
-#define SERVER_NW_LATENCY_MS    NTOP_BASE_ID+124
-#define APPL_LATENCY_MS       NTOP_BASE_ID+125
-#define PLUGIN_NAME           NTOP_BASE_ID+126
-#define RETRANSMITTED_IN_BYTES  NTOP_BASE_ID+127
+#define L7_PROTO                 NTOP_BASE_ID+118
+#define L7_PROTO_NAME            NTOP_BASE_ID+119
+#define L7_INFO                  NTOP_BASE_ID+539
+#define L7_CONFIDENCE            NTOP_BASE_ID+560
+#define DOWNSTREAM_TUNNEL_ID     NTOP_BASE_ID+120
+#define FLOW_USER_NAME           NTOP_BASE_ID+121
+#define FLOW_SERVER_NAME         NTOP_BASE_ID+122
+#define CLIENT_NW_LATENCY_MS     NTOP_BASE_ID+123
+#define SERVER_NW_LATENCY_MS     NTOP_BASE_ID+124
+#define APPL_LATENCY_MS          NTOP_BASE_ID+125
+#define PLUGIN_NAME              NTOP_BASE_ID+126
+#define RETRANSMITTED_IN_BYTES   NTOP_BASE_ID+127
 #define RETRANSMITTED_OUT_BYTES NTOP_BASE_ID+128
 #define SRC_IP_LONG           NTOP_BASE_ID+448
 #define SRC_IP_LAT            NTOP_BASE_ID+449
@@ -224,6 +233,53 @@
 #define DOWNSTREAM_SESSION_ID NTOP_BASE_ID+447
 #define SSL_SERVER_NAME       NTOP_BASE_ID+188
 #define BITTORRENT_HASH       NTOP_BASE_ID+189
+#define JA3C_HASH             NTOP_BASE_ID+489
+#define JA3S_HASH             NTOP_BASE_ID+490
+#define TLS_CIPHER            NTOP_BASE_ID+493
+#define SSL_UNSAFE_CIPHER     NTOP_BASE_ID+494
+#define L7_PROTO_RISK         NTOP_BASE_ID+509
+#define HASSHC_HASH           NTOP_BASE_ID+525
+#define HASSHS_HASH           NTOP_BASE_ID+526
+#define FLOW_VERDICT          NTOP_BASE_ID+522
+#define L7_ERROR_CODE         NTOP_BASE_ID+554
+#define L7_RISK_INFO          NTOP_BASE_ID+555
+
+/* eBPF / Process */
+#define SRC_PROC_PID              NTOP_BASE_ID+168
+#define SRC_PROC_NAME             NTOP_BASE_ID+169
+#define SRC_PROC_UID              NTOP_BASE_ID+425
+#define SRC_PROC_USER_NAME        NTOP_BASE_ID+372
+#define SRC_FATHER_PROC_PID       NTOP_BASE_ID+373
+#define SRC_FATHER_PROC_NAME      NTOP_BASE_ID+374
+#define SRC_FATHER_PROC_PKG_NAME  NTOP_BASE_ID+561
+#define SRC_FATHER_PROC_UID       NTOP_BASE_ID+564
+#define SRC_FATHER_PROC_USER_NAME NTOP_BASE_ID+565
+#define SRC_PROC_ACTUAL_MEMORY    NTOP_BASE_ID+383
+#define SRC_PROC_PEAK_MEMORY      NTOP_BASE_ID+384
+#define SRC_PROC_AVERAGE_CPU_LOAD NTOP_BASE_ID+385
+#define SRC_PROC_NUM_PAGE_FAULTS  NTOP_BASE_ID+386
+#define SRC_PROC_PCTG_IOWAIT      NTOP_BASE_ID+393
+#define SRC_PROC_PKG_NAME         NTOP_BASE_ID+540
+#define SRC_PROC_CMDLINE          NTOP_BASE_ID+556
+#define SRC_PROC_CONTAINER_ID     NTOP_BASE_ID+558
+
+#define DST_PROC_PID              NTOP_BASE_ID+375
+#define DST_PROC_NAME             NTOP_BASE_ID+376
+#define DST_PROC_UID              NTOP_BASE_ID+426
+#define DST_PROC_USER_NAME        NTOP_BASE_ID+377
+#define DST_FATHER_PROC_PID       NTOP_BASE_ID+378
+#define DST_FATHER_PROC_NAME      NTOP_BASE_ID+379
+#define DST_FATHER_PROC_PKG_NAME  NTOP_BASE_ID+563
+#define DST_FATHER_PROC_UID       NTOP_BASE_ID+567
+#define DST_FATHER_PROC_USER_NAME NTOP_BASE_ID+568
+#define DST_PROC_ACTUAL_MEMORY    NTOP_BASE_ID+387
+#define DST_PROC_PEAK_MEMORY      NTOP_BASE_ID+388
+#define DST_PROC_AVERAGE_CPU_LOAD NTOP_BASE_ID+389
+#define DST_PROC_NUM_PAGE_FAULTS  NTOP_BASE_ID+390
+#define DST_PROC_PCTG_IOWAIT      NTOP_BASE_ID+394
+#define DST_PROC_PKG_NAME         NTOP_BASE_ID+541
+#define DST_PROC_CMDLINE          NTOP_BASE_ID+557
+#define DST_PROC_CONTAINER_ID     NTOP_BASE_ID+559
 
 /* SIP */
 #define SIP_CALL_ID NTOP_BASE_ID+130
@@ -284,37 +340,12 @@
 #define RTP_IN_PKT_DROP  NTOP_BASE_ID+430
 #define RTP_OUT_PKT_DROP NTOP_BASE_ID+431
 
-/* Process */
-#define SRC_PROC_PID              NTOP_BASE_ID+168
-#define SRC_PROC_NAME             NTOP_BASE_ID+169
-#define SRC_PROC_UID              NTOP_BASE_ID+425
-#define SRC_PROC_USER_NAME        NTOP_BASE_ID+372
-#define SRC_FATHER_PROC_PID       NTOP_BASE_ID+373
-#define SRC_FATHER_PROC_NAME      NTOP_BASE_ID+374
-#define SRC_PROC_ACTUAL_MEMORY    NTOP_BASE_ID+383
-#define SRC_PROC_PEAK_MEMORY      NTOP_BASE_ID+384
-#define SRC_PROC_AVERAGE_CPU_LOAD NTOP_BASE_ID+385
-#define SRC_PROC_NUM_PAGE_FAULTS  NTOP_BASE_ID+386
-#define SRC_PROC_PCTG_IOWAIT      NTOP_BASE_ID+393
-
-#define DST_PROC_PID              NTOP_BASE_ID+375
-#define DST_PROC_NAME             NTOP_BASE_ID+376
-#define DST_PROC_UID              NTOP_BASE_ID+426
-#define DST_PROC_USER_NAME        NTOP_BASE_ID+377
-#define DST_FATHER_PROC_PID       NTOP_BASE_ID+378
-#define DST_FATHER_PROC_NAME      NTOP_BASE_ID+379
-#define DST_PROC_ACTUAL_MEMORY    NTOP_BASE_ID+387
-#define DST_PROC_PEAK_MEMORY      NTOP_BASE_ID+388
-#define DST_PROC_AVERAGE_CPU_LOAD NTOP_BASE_ID+389
-#define DST_PROC_NUM_PAGE_FAULTS  NTOP_BASE_ID+390
-#define DST_PROC_PCTG_IOWAIT      NTOP_BASE_ID+394
-
 /* HTTP */
 #define HTTP_URL NTOP_BASE_ID+180
 #define HTTP_METHOD NTOP_BASE_ID+360
 #define HTTP_RET_CODE NTOP_BASE_ID+181
 #define HTTP_REFERER NTOP_BASE_ID+182
-#define HTTP_UA NTOP_BASE_ID+183
+#define HTTP_USER_AGENT NTOP_BASE_ID+183
 #define HTTP_MIME NTOP_BASE_ID+184
 #define HTTP_HOST NTOP_BASE_ID+187
 #define HTTP_SITE      NTOP_BASE_ID+361
@@ -494,5 +525,27 @@
 #define APPLICATION_NAME                       NTOP_BASE_ID+427 /* App-Id */
 #define USER_NAME                              NTOP_BASE_ID+428 /* User-Id */
 
+#define L7_PROTO_RISK_NAME    NTOP_BASE_ID+510
+#define TLS_REQUESTED_SNI     NTOP_BASE_ID+511
+#define TLS_SERVER_NAMES      NTOP_BASE_ID+512
+#define TLS_ALPN              NTOP_BASE_ID+513
+#define TLS_CERT_ISSUER_DN    NTOP_BASE_ID+514
+#define TLS_CERT_SUBJECT_DN   NTOP_BASE_ID+515
+#define SRC_ADDR_LOCAL        NTOP_BASE_ID+516
+#define SRC_ADDR_BLACKLISTED  NTOP_BASE_ID+517
+#define SRC_ADDR_SERVICES     NTOP_BASE_ID+518
+#define DST_ADDR_LOCAL        NTOP_BASE_ID+519
+#define DST_ADDR_BLACKLISTED  NTOP_BASE_ID+520
+#define DST_ADDR_SERVICES     NTOP_BASE_ID+521
+#define SRC_NAME              NTOP_BASE_ID+522
+#define DST_NAME              NTOP_BASE_ID+523
+#define COMMUNITY_ID          NTOP_BASE_ID+524
+#define L7_CATEGORY           NTOP_BASE_ID+509
+#define FLOW_RISK             NTOP_BASE_ID+508
+#define SRC_ASN               NTOP_BASE_ID+525
+#define DST_ASN               NTOP_BASE_ID+526
+#define PROBE_IP              NTOP_BASE_ID+527
+#define L4_PROTO_NAME         NTOP_BASE_ID+528
+#define L7_CATEGORY_ID        NTOP_BASE_ID+529
 
 #endif /* _NTOP_FLOW_H_ */
