@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -90,7 +90,11 @@
 #include <zmq.h>
 #include <assert.h>
 #include <fcntl.h>
-#if defined(linux)
+#ifndef WIN32
+#include <grp.h>
+#endif
+//#include <libgen.h>
+#if defined(__linux__)
 #include <linux/ethtool.h> // ethtool
 #include <linux/sockios.h> // sockios
 #include <ifaddrs.h>
@@ -109,7 +113,7 @@ extern "C" {
 #include <pcap/bpf.h> /* Used for bpf_filter() */
 #endif
 
-#include "ndpi_main.h"
+#include "ndpi_api.h"
 #include "lua.h"
 #include "lauxlib.h"
 #include "lualib.h"
@@ -122,18 +126,32 @@ extern "C" {
 #include <linux/netfilter.h> /* for NF_ACCEPT */
 #include <libnfnetlink/libnfnetlink.h>
 #include <libnetfilter_queue/libnetfilter_queue.h>
+#include <ifaddrs.h> /* SilicomHwBypass */
 #endif
 #include "json.h"
 #include <sqlite3.h>
-#include "hiredis.h"
+#include <hiredis.h>
 #ifdef HAVE_LDAP
 #include <ldap.h>
 #endif
 #ifdef HAVE_ZLIB
 #include <zlib.h>
 #endif
+
+#ifdef WIN32
+/* 
+See
+https://translate.google.co.uk/translate?sl=auto&tl=en&u=http%3A%2F%2Fbugsfixed.blogspot.com%2F2017%2F05%2Fvcpkg.html
+*/
+#define CURL_STATICLIB
+#endif
 #include <curl/curl.h>
 
+#ifdef WIN32
+#pragma comment(lib, "crypt32.lib")
+#pragma comment(lib, "wldap32.lib") 
+#endif
+
 #include "third-party/uthash.h"
 
 #ifdef HAVE_MYSQL
@@ -141,22 +159,33 @@ extern "C" {
 #include <errmsg.h>
 #endif
 
+#ifdef HAVE_MAXMINDDB
+#include <maxminddb.h>
+#endif
+
 #ifdef HAVE_LIBCAP
 #include <sys/capability.h>
 #include <sys/prctl.h>
 #endif
 };
 
-#ifdef HAVE_EBPF
-#include "ebpf_flow.h"
-#endif
-
 #include <fstream>
 #include <map>
+#include <unordered_map>
+
+#if !defined(__clang__) && (__GNUC__ <= 4) && (__GNUC_MINOR__ < 8) && !defined(WIN32)
+#include <cstdatomic>
+#else
+#include <atomic>
+#endif
+
+#include <utility>
 #include <set>
+#include <algorithm>
 #include <vector>
 #include <list>
 #include <iostream>
+#include <type_traits>
 #include <string>
 #include <sstream>
 #include <queue>
@@ -165,63 +194,109 @@ extern "C" {
 using namespace std;
 
 #include "mongoose.h"
-#include "patricia.h"
 #include "ntop_defines.h"
 #include "Mutex.h"
 #include "RwLock.h"
+#include "Bitmask.h"
+#include "Bloom.h"
+#include "MonitoredMetric.h"
+#include "MonitoredCounter.h"
+#include "MonitoredGauge.h"
 #include "MDNS.h"
 #include "AddressTree.h"
-#include "VlanAddressTree.h"
-#include "AddressList.h"
+#include "VLANAddressTree.h"
+#include "BroadcastDomains.h"
+#include "Cardinality.h"
+#include "PeerStats.h"
 #include "IpAddress.h"
-#include "ntop_typedefs.h"
-#include "Trace.h"
-#include "NtopGlobals.h"
-#include "Checkpointable.h"
+#include "Ping.h"
+#include "ContinuousPingStats.h"
+#include "ContinuousPing.h"
 #include "TrafficStats.h"
+#include "TcpPacketStats.h"
+#include "DSCPStats.h"
+#include "Bitmap.h"
+#if defined(NTOPNG_PRO)
+#include "BinAnalysis.h"
+#endif
+#include "ntop_typedefs.h"
+#include "ScoreStats.h"
+#include "ViewScoreStats.h"
+#include "Score.h"
+#include "FrequentStringItems.h"
+#include "MostVisitedList.h"
+#include "Trace.h"
+#include "ProtoStats.h"
+#include "FlowRiskAlerts.h"
+#include "Utils.h"
+#include "Bitmap128.h"
+#include "NtopGlobals.h"
+#include "Alert.h"
+#include "AlertableEntity.h"
+#include "HostAlertableEntity.h"
+#include "OtherAlertableEntity.h"
+#include "NetworkInterfaceAlertableEntity.h"
+#include "InterfaceMemberAlertableEntity.h"
+#include "BehaviouralCounter.h"
+#ifdef NTOPNG_PRO
+#include "BehaviorAnalysis.h"
+#endif
+
 #include "nDPIStats.h"
+#include "InterarrivalStats.h"
+#include "FlowStats.h"
 #ifdef NTOPNG_PRO
 #include "CustomAppMaps.h"
 #include "CustomAppStats.h"
 #endif
+#include "ThroughputStats.h"
 #include "GenericTrafficElement.h"
+#include "BlacklistUsageStats.h"
+#include "BlacklistStats.h"
+#include "AlertCounter.h"
+#include "NetworkStats.h"
+#include "ContainerStats.h"
+#include "ParsedFlowCore.h"
+#include "ParsedeBPF.h"
+#include "ParsedFlow.h"
+#ifdef HAVE_EBPF
+#include "ebpf_flow.h"
+#endif
 
 #ifdef NTOPNG_PRO
 #include "Profile.h"
 #include "Profiles.h"
 #include "CountMinSketch.h"
+#include "AlertExclusionsInfo.h"
+#include "AlertExclusions.h"
 #ifndef HAVE_NEDGE
 #include "FlowProfile.h"
 #include "FlowProfiles.h"
+#include "SubInterface.h"
+#include "SubInterfaces.h"
 #endif
 #include "CounterTrend.h"
 #include "LRUMacIP.h"
 #include "FlowInterfacesStats.h"
-#include "HostPoolStats.h"
 #ifdef HAVE_LDAP
 #include "LdapAuthenticator.h"
 #endif
 #endif
-#include "FrequentStringItems.h"
-#include "FrequentNumericItems.h"
-#include "FrequentTrafficItems.h"
+#include "HostPoolStats.h"
 #include "HostPools.h"
+#include "Fingerprint.h"
 #include "Prefs.h"
-#include "Utils.h"
-#include "CommunityIdFlowHash.h"
+#include "SerializableElement.h"
 #include "DnsStats.h"
-#include "NetworkStats.h"
-#ifndef HAVE_NEDGE
 #include "SNMP.h"
-#endif
 #include "NetworkDiscovery.h"
 #include "ICMPstats.h"
-#include "Grouper.h"
+#include "ICMPinfo.h"
 #include "FlowGrouper.h"
 #include "PacketStats.h"
-#include "ProtoStats.h"
-#include "TcpPacketStats.h"
 #include "EthStats.h"
+#include "RoundTripStats.h"
+#include "SyslogStats.h"
 
 #include "LocalTrafficStats.h"
 #include "PacketDumperGeneric.h"
@@ -229,31 +304,50 @@ using namespace std;
 #include "PacketDumperTuntap.h"
 #include "TimelineExtract.h"
 #include "TcpFlowStats.h"
-#include "StoreManager.h"
+#include "SQLiteStoreManager.h"
 #include "StatsManager.h"
-#include "AlertsManager.h"
+#include "AlertStore.h"
+#include "SQLiteAlertStore.h"
 #include "DB.h"
+#if defined(HAVE_KAFKA) && defined(NTOPNG_PRO)
+#include "KafkaProducer.h"
+#include "KafkaClient.h"
+#endif
 #ifdef HAVE_MYSQL
 #include "MySQLDB.h"
 #endif
 #include "InterfaceStatsHash.h"
+#include "GenericHash.h"
 #include "GenericHashEntry.h"
-#if defined(NTOPNG_PRO) && defined(HAVE_NINDEX)
-#include "nindex_api.h"
-#endif
+#include "MacHash.h"
 #ifdef HAVE_RADIUS
 #include <radcli/radcli.h>
 #endif
 
-
-#include "TimeseriesPoint.h"
-#include "TimeseriesRingStatus.h"
-#include "TimeseriesRing.h"
+#include "Condvar.h"
 #include "TimeseriesExporter.h"
-#include "TimeseriesRing.h"
-#include "HostTimeseriesPoint.h"
+#include "InfluxDBTimeseriesExporter.h"
+#include "L4Stats.h"
+#include "AlertsQueue.h"
+#include "LuaEngineFunctions.h"
+#include "LuaEngine.h"
 #include "SPSCQueue.h"
-#include "NetworkInterfaceTsPoint.h"
+#include "SyslogLuaEngine.h"
+#include "FifoQueue.h"
+#include "StringFifoQueue.h"
+#include "AlertFifoQueue.h"
+#include "FifoSerializerQueue.h"
+#include "RRDTimeseriesExporter.h"
+#include "RecipientQueue.h"
+#include "Recipients.h"
+#if defined(NTOPNG_PRO)
+#include "PeriodicityStats.h"
+#include "PeriodicityHash.h"
+#include "ServiceMap.h"
+#include "PeriodicityMap.h"
+#endif
+#include "UsedPorts.h"
+#include "ObservationPointIdTrafficStats.h"
 #include "NetworkInterface.h"
 #ifndef HAVE_NEDGE
 #include "PcapInterface.h"
@@ -262,85 +356,105 @@ using namespace std;
 #ifdef HAVE_PF_RING
 #include "PF_RINGInterface.h"
 #endif
-#include "AlertCounter.h"
-#include "FlowAlertCounter.h"
-#include "GenericHash.h"
 #include "VirtualHost.h"
 #include "VirtualHostHash.h"
 #include "HTTPstats.h"
 #include "Redis.h"
 #ifndef HAVE_NEDGE
 #include "ElasticSearch.h"
-#include "Logstash.h"
+#ifndef WIN32
+#include "SyslogDump.h"
 #endif
-#ifdef HAVE_NINDEX
-#include "NIndexFlowDB.h"
+#endif
+#if defined(NTOPNG_PRO) && defined(HAVE_CLICKHOUSE)
+#include "ClickHouseImport.h"
+#include "ClickHouseFlowDB.h"
+#include "ClickHouseAlertStore.h"
 #endif
 #ifdef NTOPNG_PRO
 #include "NtopPro.h"
 #include "DnsHostMapping.h"
 #include "TrafficShaper.h"
 #include "L7Policer.h"
-#ifdef HAVE_MYSQL
-#include "BatchedMySQLDB.h"
-#include "BatchedMySQLDBEntry.h"
-#endif
 #include "LuaHandler.h"
-#ifndef WIN32
-#include "NagiosManager.h"
-#endif
-#include "FlowChecker.h"
-#include "FrequentStringItems.h"
-#include "FrequentNumericItems.h"
-#include "FrequentTrafficItems.h"
 #ifdef HAVE_NEDGE
+#include "HwBypass.h"
+#include "SilicomHwBypass.h"
 #include "NetfilterInterface.h"
+#else
+#include "nTapInterface.h"
 #endif
 #endif
 #ifndef HAVE_NEDGE
 #include "ParserInterface.h"
-#include "CollectorInterface.h"
+#include "ListeningPorts.h"
+#include "ZMQParserInterface.h"
+#include "ZMQPublisher.h"
+#include "ZMQCollectorInterface.h"
+#include "SyslogParserInterface.h"
+#include "SyslogCollectorInterface.h"
 #include "ZCCollectorInterface.h"
 #include "DummyInterface.h"
 #include "ExportInterface.h"
 #endif
 
+#if defined(HAVE_KAFKA) && defined(NTOPNG_PRO)
+#include "KafkaCollectorInterface.h"
+#endif
+
 #include "Geolocation.h"
-#include "Vlan.h"
+#include "VLAN.h"
 #include "AutonomousSystem.h"
+#include "ObservationPoint.h"
+#include "OperatingSystem.h"
 #include "Country.h"
+#include "MacStats.h"
 #include "Mac.h"
+#include "PartializableFlowTrafficStats.h"
+#include "ViewInterfaceFlowStats.h"
+#include "FlowTrafficStats.h"
+#include "HostStats.h"
+#include "HostChecksStatus.h"
+#include "ActiveHostWalkerInfo.h"
 #include "Host.h"
+#include "DoHDoTStats.h"
+#include "LocalHostStats.h"
 #include "LocalHost.h"
 #include "RemoteHost.h"
+#ifdef NTOPNG_PRO
+#include "AssetManagement.h"
+#endif
+#include "IEC104Stats.h"
 #include "Flow.h"
 #include "FlowHash.h"
-#include "MacHash.h"
-#include "VlanHash.h"
+#include "VLANHash.h"
 #include "AutonomousSystemHash.h"
+#include "ObservationPointHash.h"
+#include "OperatingSystemHash.h"
 #include "CountriesHash.h"
 #include "HostHash.h"
-#ifdef NTOPNG_PRO
-#include "AggregatedFlow.h"
-#include "AggregatedFlowHash.h"
-#endif
+#include "ThreadedActivityStats.h"
 #include "ThreadedActivity.h"
 #include "ThreadPool.h"
+#include "PeriodicScript.h"
 #include "PeriodicActivities.h"
-#include "LuaEngine.h"
 #include "MacManufacturers.h"
 #include "AddressResolution.h"
 #include "HTTPserver.h"
 #include "Paginator.h"
+#include "FlowAlert.h"
+#include "Check.h"
+#include "FlowCheck.h"
+#include "HostAlert.h"
+#include "HostCheck.h"
+#include "FlowAlertsLoader.h"
+#include "ChecksLoader.h"
+#include "FlowChecksLoader.h"
+#include "FlowChecksExecutor.h"
+#include "HostChecksLoader.h"
+#include "HostChecksExecutor.h"
 #include "Ntop.h"
 
-#ifdef WIN32
-extern "C" {
-  const char *strcasestr(const char *haystack, const char *needle);
-  int strncasecmp(const char *s1, const char *s2, unsigned int n);
-};
-#endif
-
 #ifdef NTOPNG_PRO
 #include "ntoppro_defines.h"
 #endif
