@@ -1,5 +1,5 @@
 --
--- (C) 2013-18 - ntop.org
+-- (C) 2013-23 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -11,14 +11,15 @@ local page_utils = require("page_utils")
 
 sendHTTPContentTypeHeader('text/html')
 
-page_utils.print_header(i18n("as_stats.autonomous_systems"))
 
-active_page = "hosts"
+page_utils.set_active_menu_entry(page_utils.menu_entries.autonomous_systems)
+
 dofile(dirs.installdir .. "/scripts/lua/inc/menu.lua")
 
+page_utils.print_page_title(i18n("as_stats.autonomous_systems"))
+
 print [[
-      <hr>
-      <div id="table-as"></div>
+	  <div id="table-as"></div>
 	 <script>
 	 var url_update = "]]
 print (ntop.getHttpPrefix())
@@ -33,7 +34,7 @@ print [[
 			url: url_update ,
 	 ]]
 
-print('title: "' .. i18n("as_stats.autonomous_systems") .. '",\n')
+print('title: "",\n')
 print ('rowCallback: function ( row ) { return as_table_setID(row); },')
 
 -- Set the preference table
@@ -66,6 +67,7 @@ print [[
                          {
 			     title: "]] print(i18n("chart")) print[[",
 				 field: "column_chart",
+				 hidden: ]] print(ternary(areASTimeseriesEnabled(interface.getId()), "false", "true")) print[[,
 				 sortable: false,
                              css: {
 			        textAlign: 'center'
@@ -83,15 +85,22 @@ print [[
 			     }
 
 				 },
-			     {
-			     title: "]] print(i18n("show_alerts.alerts")) print[[",
-				 field: "column_alerts",
-				 sortable: false,
-                             css: {
-			        textAlign: 'center'
-			     }
-
-				 },
+         {
+          title: "]] print(i18n("score")) print[[",
+          field: "column_score",
+          sortable: true,
+                      css: {
+          textAlign: 'center'
+            }
+          },
+          {
+           title: "]] print(i18n("host_score_ratio")) print[[",
+           field: "column_host_score_ratio",
+           sortable: false,
+                       css: {
+           textAlign: 'center'
+             }
+           },
 			     {
 			     title: "]] print(i18n("name")) print[[",
 				 field: "column_asname",
@@ -110,6 +119,14 @@ print [[
 			     }
 
 				 },
+         {
+         title: "]] print(i18n("flow_details.alerted_flows")) print[[",
+       field: "column_alerted_flows",
+       sortable: true,
+                           css: {
+            textAlign: 'center'
+         },
+       },
 ]]
 
 print [[
