@@ -1,5 +1,5 @@
 --
--- (C) 2013-17 - ntop.org
+-- (C) 2013-23 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -7,11 +7,13 @@ package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 
 require "lua_utils"
 
+local page_utils = require("page_utils")
+
 sendHTTPContentTypeHeader('text/html')
 
 
-ntop.dumpFile(dirs.installdir .. "/httpdocs/inc/header.inc")
-active_page = "admin"
+page_utils.set_active_menu_entry(page_utils.menu_entries.export_data)
+
 dofile(dirs.installdir .. "/scripts/lua/inc/menu.lua")
 
 print [[
@@ -38,19 +40,19 @@ print [[
        <b>]] print(i18n("export_data.hosts")) print[[:</b>
        <br>
 
-       <div class="form-group form-inline">
-         <div class="btn-group" data-toggle="buttons" id="export_hosts_buttons" name="export_hosts_buttons">
-           <label class="btn btn-default active">
-             <input type="radio" id="all_hosts" name="mode" value="all" autocomplete="off" data-toggle="toggle"  checked="checked">]] print(i18n("export_data.all_hosts")) print[[
+       <div class="form-group mb-3 form-inline">
+         <div class="btn-group btn-group-toggle" data-bs-toggle="buttons" id="export_hosts_buttons" name="export_hosts_buttons">
+           <label class="btn btn-secondary active">
+             <input type="radio" id="all_hosts" name="mode" value="all" autocomplete="off" data-bs-toggle="toggle"  checked="checked">]] print(i18n("export_data.all_hosts")) print[[
            </label>
-           <label class="btn btn-default">
-             <input type="radio" id="local_hosts" name="mode" value="local" autocomplete="off" data-toggle=" toggle">]] print(i18n("export_data.local_hosts")) print[[
+           <label class="btn btn-secondary">
+             <input type="radio" id="local_hosts" name="mode" value="local" autocomplete="off" data-bs-toggle=" toggle">]] print(i18n("export_data.local_hosts")) print[[
            </label>
-           <label class="btn btn-default">
-             <input type="radio" id="remote_hosts" name="mode" value="remote" autocomplete="off" data-toggle=" toggle">]] print(i18n("export_data.remote_hosts")) print[[
+           <label class="btn btn-secondary">
+             <input type="radio" id="remote_hosts" name="mode" value="remote" autocomplete="off" data-bs-toggle=" toggle">]] print(i18n("export_data.remote_hosts")) print[[
            </label>
-           <label class="btn btn-default">
-             <input type="radio" id="single_host" name="mode" value="filtered" autocomplete="off" data-toggle=" toggle">]] print(i18n("export_data.single")) print[[
+           <label class="btn btn-secondary">
+             <input type="radio" id="single_host" name="mode" value="filtered" autocomplete="off" data-bs-toggle=" toggle">]] print(i18n("export_data.single")) print[[
            </label>
          </div>
 
@@ -72,8 +74,8 @@ print [[
      </div>
 
      <div class='col-md-2'>
-       <div class="btn-group pull-right">
-         <input type="submit" value="]] print(i18n("export_data.export_json_data")) print[[" class="btn btn-default pull-right">
+       <div class="btn-group float-right">
+         <input type="submit" value="]] print(i18n("export_data.export_json_data")) print[[" class="btn btn-secondary float-right">
        </div>
      </div>
    </div>
@@ -95,8 +97,8 @@ print [[
        source: function (query, process) {
                return $.get(']]
 print (ntop.getHttpPrefix())
-print [[/lua/find_host.lua', { query: query }, function (data) {
-                     return process(data.results);
+print [[/lua/rest/v2/get/host/find.lua', { query: query }, function (data) {
+        return process(data.rsp.results);
       });
       }, afterSelect: function(item) {
         $('#hostIPSearch').val(item.ip.split("@")[0]);
