@@ -1,8 +1,8 @@
 --
--- (C) 2013-18 - ntop.org
+-- (C) 2013-23 - ntop.org
 --
 
-dirs = ntop.getDirs()
+local dirs = ntop.getDirs()
 package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 
 require "lua_utils"
@@ -11,25 +11,21 @@ require "voip_utils"
 
 local json = require ("dkjson")
 
-flow_key = _GET["flow_key"]
-if(flow_key == nil) then
- flow = nil
-else
- interface.select(ifname)
- flow = interface.findFlowByKey(tonumber(flow_key))
-end
+local flow_key = _GET["flow_key"]
+local flow_hash_id = _GET["flow_hash_id"]
 
-throughput_type = getThroughputType()
+local flow = interface.findFlowByKeyAndHashId(tonumber(flow_key), tonumber(flow_hash_id))
+
+local throughput_type = getThroughputType()
 
 sendHTTPContentTypeHeader('text/html')
---sendHTTPHeader('application/json')
 
 if(flow == nil) then
- print('{}')
+   print('{}')
 else
 
- diff0 = os.time()-flow["seen.first"]
- diff = os.time()-flow["seen.last"]
+   diff0 = os.time()-flow["seen.first"]
+   diff = os.time()-flow["seen.last"]
    -- Default values
    thpt = 0
    thpt_display = bitsToSize(0)
@@ -80,7 +76,7 @@ else
       proc = flow.client_process
       print ('"'..proc.pid..'": {')
 
-      if proc.actual_memory and proc.actual_memory > 0 then
+      if((proc.actual_memory ~= nil) and (proc.actual_memory > 0) and (proc.average_cpu_load ~= nil)) then
         -- average_cpu_load
         print('"average_cpu_load": "')
 	load = round(proc.average_cpu_load, 2)
@@ -113,7 +109,7 @@ else
       proc = flow.server_process
       print ('"'..proc.pid..'": {')
 
-      if((proc.actual_memory ~= nil) and (proc.actual_memory > 0)) then
+      if((proc.actual_memory ~= nil) and (proc.actual_memory > 0) and (proc.average_cpu_load ~= nil)) then
         -- average_cpu_load
         load = round(proc.average_cpu_load, 2)
         print('"average_cpu_load": "')
@@ -166,7 +162,7 @@ else
         if(((called_party == nil) or (called_party == "")) and ((calling_party == nil) or (calling_party == ""))) then
           print(', "sip.calling_called_party":"'..calling_party .. ' ' .. called_party..'"')
         else
-          print(', "sip.calling_called_party":"'..calling_party .. ' <i class=\\\"fa fa-exchange fa-lg\\\"></i> ' .. called_party..'"')
+          print(', "sip.calling_called_party":"'..calling_party .. ' <i class=\\\"fas fa-exchange-alt fa-lg\\\"></i> ' .. called_party..'"')
         end
 
         print(', "sip.rtp_codecs":"'..getFlowValue(info, "SIP_RTP_CODECS")..'"')
@@ -399,9 +395,7 @@ else
             interface.select(ifname)
             rtp_host = interface.getHostInfo(address_ip)
             if(rtp_host ~= nil) then
-              print('<A HREF=\\\"'..ntop.getHttpPrefix()..'/lua/host_details.lua?host='..address_ip.. '\\\">')
-              print(address_ip)
-              print('</A>')
+              print(hostinfo2detailshref(rtp_host, nil, address_ip))
             end
           else
             print(address_ip)
@@ -409,23 +403,24 @@ else
         end
         if((getFlowValue(info, "SIP_RTP_L4_SRC_PORT")~=nil) and (getFlowValue(info, "SIP_RTP_L4_SRC_PORT")~="") and (sip_rtp_src_addr == 1)) then
           --print(':'..getFlowValue(info, "SIP_RTP_L4_SRC_PORT"))
-          print(':<A HREF=\\\"'..ntop.getHttpPrefix()..'/lua/port_details.lua?port='..getFlowValue(info, "SIP_RTP_L4_SRC_PORT").. '\\\">')
+          print(':<A HREF=\\\"'..ntop.getHttpPrefix()..'/lua/flows_stats.lua?port='..getFlowValue(info, "SIP_RTP_L4_SRC_PORT").. '\\\">')
           print(getFlowValue(info, "SIP_RTP_L4_SRC_PORT"))
           print('</A>')
         end
         if((sip_rtp_src_addr == 1) or ((getFlowValue(info, "SIP_RTP_IPV4_DST_ADDR")~=nil) and (getFlowValue(info, "SIP_RTP_IPV4_DST_ADDR")~=""))) then
-          print(' <i class=\\\"fa fa-exchange fa-lg\\\"></i> ')
+          print(' <i class=\\\"fas fa-exchange-alt fa-lg\\\"></i> ')
         end
         if((getFlowValue(info, "SIP_RTP_IPV4_DST_ADDR")~=nil) and (getFlowValue(info, "SIP_RTP_IPV4_DST_ADDR")~="")) then
           sip_rtp_dst_addr = 1
           local address_ip = getFlowValue(info, "SIP_RTP_IPV4_DST_ADDR")
-          if (address_ip ~= "0.0.0.0") then
+
+	  if (address_ip ~= "0.0.0.0") then
             interface.select(ifname)
+	    
             rtp_host = interface.getHostInfo(address_ip)
+	    
             if(rtp_host ~= nil) then
-              print('<A HREF=\\\"'..ntop.getHttpPrefix()..'/lua/host_details.lua?host='..address_ip.. '\\\">')
-              print(address_ip)
-              print('</A>')
+	      print(hostinfo2detailshref(rtp_host, nil, address_ip))
             end
           else
             print(address_ip)
@@ -433,7 +428,7 @@ else
         end
         if((getFlowValue(info, "SIP_RTP_L4_DST_PORT")~=nil) and (getFlowValue(info, "SIP_RTP_L4_DST_PORT")~="") and (sip_rtp_dst_addr == 1)) then
           --print(':'..getFlowValue(info, "SIP_RTP_L4_DST_PORT"))
-          print(':<A HREF=\\\"'..ntop.getHttpPrefix()..'/lua/port_details.lua?port='..getFlowValue(info, "SIP_RTP_L4_DST_PORT").. '\\\">')
+          print(':<A HREF=\\\"'..ntop.getHttpPrefix()..'/lua/flows_stats.lua?port='..getFlowValue(info, "SIP_RTP_L4_DST_PORT").. '\\\">')
           print(getFlowValue(info, "SIP_RTP_L4_DST_PORT"))
           print('</A>')
         end
