@@ -1,5 +1,5 @@
 --
--- (C) 2013-18 - ntop.org
+-- (C) 2013-23 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -7,6 +7,7 @@ package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 require "lua_utils"
 require "mac_utils"
 local json = require("dkjson")
+
 sendHTTPContentTypeHeader('text/html')
 
 -- Table parameters
@@ -54,12 +55,9 @@ else
 end
 
 local source_macs_only = false
-local dhcp_macs_only = false
 
-if devices_mode == "source_macs_only" then   
-   source_macs_only = true   
-elseif devices_mode == "dhcp_macs_only" then
-   dhcp_macs_only = true   
+if devices_mode == "source_macs_only" then
+   source_macs_only = true
 end
 
 interface.select(ifname)
@@ -71,7 +69,7 @@ if(sortOrder == "desc") then sOrder = false else sOrder = true end
 
 local macs_stats = interface.getMacsInfo(sortColumn, perPage, to_skip, sOrder,
 					 source_macs_only, manufacturer,
-					 nil, device_type, "", dhcp_macs_only)
+					 nil, device_type, "", nil)
 
 local total_rows = 0
 
