@@ -1,5 +1,5 @@
 --
--- (C) 2013-18 - ntop.org
+-- (C) 2013-23 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -8,6 +8,7 @@ require "lua_utils"
 require "vlan_utils"
 
 local json = require("dkjson")
+
 sendHTTPContentTypeHeader('text/html')
 
 -- Table parameters
@@ -68,7 +69,7 @@ vlans_stats = vlans_stats["VLANs"]
 local res_formatted = {}
 
 for _, vlan in ipairs(vlans_stats) do
-   local record = vlan2record(getInterfaceId(ifname), vlan)
+   local record = vlan2record(interface.getId(), vlan)
    res_formatted[#res_formatted + 1] = record
 end
 
