@@ -1,5 +1,5 @@
 --
--- (C) 2013-18 - ntop.org
+-- (C) 2013-23 - ntop.org
 --
 
 local dirs = ntop.getDirs()
@@ -14,7 +14,7 @@ local ifstats = interface.getStats()
 local distr_type = _GET["distr"]
 
 if((distr_type == nil) or (distr_type == "size")) then
-   what = ifstats["pktSizeDistribution"]
+   what = ifstats["pktSizeDistribution"]["size"]
 elseif distr_type == "ipver" then
    print "[\n"
    print("\t { \"label\": \"IPv6\", \"value\": ".. ifstats.eth.IPv6_packets .." },")
@@ -52,8 +52,8 @@ for key, value in pairs(what) do
 	 if(num > 0) then
 	    print ",\n"
 	 end
-	 	 
-	 print("\t { \"label\": \"" .. pkt_distribution[key] .."\", \"value\": ".. value .." }") 
+
+	 print("\t { \"label\": \"" .. pkt_distribution[key] .."\", \"value\": ".. value .." }")
 	 num = num + 1
 	 sum = sum + value
       end
@@ -61,7 +61,7 @@ for key, value in pairs(what) do
 end
 
 if(sum < tot) then
-   print("\t, { \"label\": \"Other\", \"value\": ".. (tot-sum) .." }") 
+   print("\t, { \"label\": \"Other\", \"value\": ".. (tot-sum) .." }")
 end
 
 print "\n]"
