@@ -1,5 +1,5 @@
 --
--- (C) 2013-18 - ntop.org
+-- (C) 2013-23 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -11,7 +11,8 @@ local page_utils = require("page_utils")
 
 sendHTTPContentTypeHeader('text/html')
 
-page_utils.print_header(i18n("layer_2"))
+
+page_utils.set_active_menu_entry(page_utils.menu_entries.devices)
 
 if (group_col == nil) then
    group_col = "mac"
@@ -19,8 +20,6 @@ end
 
 local have_nedge = ntop.isnEdge()
 
-active_page = "hosts"
-
 dofile(dirs.installdir .. "/scripts/lua/inc/menu.lua")
 
 local base_url = ntop.getHttpPrefix() .. "/lua/macs_stats.lua"
@@ -28,12 +27,11 @@ local page_params = {}
 
 local devices_mode = ""
 local devices_mode_filter = ""
-local dhcp_macs_only = false
 
 if(not isEmptyString(_GET["devices_mode"])) then
    devices_mode = _GET["devices_mode"]
    page_params["devices_mode"] = _GET["devices_mode"]
-   devices_mode_filter = '<span class="glyphicon glyphicon-filter"></span>'
+   devices_mode_filter = '<span class="fas fa-filter"></span>'
 end
 
 local manufacturer = nil
@@ -41,7 +39,7 @@ local manufacturer_filter = ""
 if(not isEmptyString(_GET["manufacturer"])) then
    manufacturer = _GET["manufacturer"]
    page_params["manufacturer"] = manufacturer
-   manufacturer_filter = '<span class="glyphicon glyphicon-filter"></span>'
+   manufacturer_filter = '<span class="fas fa-filter"></span>'
 end
 
 local device_type = nil
@@ -49,11 +47,12 @@ local devtype_filter = ""
 if(not isEmptyString(_GET["device_type"])) then
    device_type = tonumber(_GET["device_type"])
    page_params["device_type"] = device_type
-   devtype_filter = '<span class="glyphicon glyphicon-filter"></span>'
+   devtype_filter = '<span class="fas fa-filter"></span>'
 end
 
+page_utils.print_page_title('Mac List')
+
 print [[
-      <hr>
       <div id="table-mac"></div>
 	 <script>
 	 var url_update = "]]
@@ -63,10 +62,10 @@ print(getPageUrl(ntop.getHttpPrefix().."/lua/get_macs_data.lua", page_params))
 print ('";')
 ntop.dumpFile(dirs.installdir .. "/httpdocs/inc/mac_stats_id.inc")
 
-print [[ 
+print [[
            $("#table-mac").datatable({
-                        title: "Mac List",
-			url: url_update , 
+              title: '',
+			url: url_update ,
 ]]
 
 local title
@@ -77,13 +76,6 @@ if devices_mode == "source_macs_only" then
    else
       title = i18n("mac_stats.layer_2_source_devices", {device_type=""})
    end
-elseif devices_mode == "dhcp_macs_only" then
-   dhcp_macs_only = true
-   if device_type then
-      title = i18n("mac_stats.layer_2_dev_devices", {device_type=discover.devtype2string(device_type).." DHCP"})
-   else
-      title = i18n("mac_stats.layer_2_source_devices", {device_type=" DHCP"})
-   end
 else
    if device_type then
       title = i18n("mac_stats.dev_layer_2_devices", {device_type=discover.devtype2string(device_type)})
@@ -96,11 +88,10 @@ if manufacturer ~= nil then
  title = i18n("mac_stats.layer_2_devices_with_manufacturer",{title=title, manufacturer=manufacturer})
 end
 
-print('title: "'..title..'",\n')
 
 print ('rowCallback: function ( row ) { return mac_table_setID(row); },')
 print[[
-        tableCallback: function()  { $("#dt-bottom-details > .pull-left > p").first().append('. ]]
+        tableCallback: function()  { $("#dt-bottom-details > .float-left > p").first().append('. ]]
 print(i18n('mac_stats.idle_devices_not_listed'))
 print[['); },]]
 
@@ -114,45 +105,36 @@ print ('sort: [ ["' .. getDefaultTableSort("macs") ..'","' .. getDefaultTableSor
 print('buttons: [')
 
    -- Filter MACS
+   -- table.clone needed to modify some parameters while keeping the original unchanged
    local macs_params = table.clone(page_params)
    macs_params.devices_mode = nil
-   print('\'<div class="btn-group"><button class="btn btn-link dropdown-toggle" data-toggle="dropdown">'..i18n("mac_stats.filter_macs")..devices_mode_filter..'<span class="caret"></span></button> <ul class="dropdown-menu" role="menu" style="min-width: 90px;"><li><a href="')
+   print('\'<div class="btn-group"><button class="btn btn-link dropdown-toggle" data-bs-toggle="dropdown">'..i18n("mac_stats.filter_macs")..devices_mode_filter..'<span class="caret"></span></button> <ul class="dropdown-menu scrollable-dropdown" role="menu" style="min-width: 90px;"><li><a class="dropdown-item" href="')
    print(getPageUrl(base_url, macs_params))
    print('">'..i18n("mac_stats.all_devices")..'</a></li>')
 
    -- Source MACs only
-   print('<li')
-   if devices_mode == "source_macs_only" then print(' class="active"') end
-   print('><a href="')
+   print('<li><a class="dropdown-item '.. (devices_mode == "source_macs_only" and 'active' or '') ..'" href="')
    macs_params.devices_mode = "source_macs_only"
    print(getPageUrl(base_url, macs_params))
    print('">'..i18n("mac_stats.source_macs")..'</a></li>')
 
-   -- DHCP MACs only
-   print('<li')
-   if devices_mode == "dhcp_macs_only" then print(' class="active"') end
-   print('><a href="')
-   macs_params.devices_mode = "dhcp_macs_only"
-   print(getPageUrl(base_url, macs_params))
-   print('">'..i18n("mac_stats.dhcp_only")..'</a></li>')
    print("</div>'")
 
    -- Filter Manufacturers
+   -- table.clone needed to modify some parameters while keeping the original unchanged
    local manufacturer_params = table.clone(page_params)
    manufacturer_params.manufacturer = nil
    print[[, '\
-       <div class="btn-group pull-right">\
-       <button class="btn btn-link dropdown-toggle" data-toggle="dropdown">]] print(i18n("mac_stats.manufacturer")) print(manufacturer_filter) print[[<span class="caret"></span></button>\
-       <ul class="dropdown-menu" role="menu" id="flow_dropdown">\
-          <li><a href="]] print(getPageUrl(base_url, manufacturer_params)) print[[">]] print(i18n("mac_stats.all_manufacturers")) print[[</a></li>\
+       <div class="btn-group float-right">\
+       <button class="btn btn-link dropdown-toggle" data-bs-toggle="dropdown">]] print(i18n("mac_stats.manufacturer")) print(manufacturer_filter) print[[<span class="caret"></span></button>\
+       <ul class="dropdown-menu scrollable-dropdown" role="menu" id="flow_dropdown">\
+          <li><a class="dropdown-item" href="]] print(getPageUrl(base_url, manufacturer_params)) print[[">]] print(i18n("mac_stats.all_manufacturers")) print[[</a></li>\
    ]]
 
 for manuf, count in pairsByKeys(interface.getMacManufacturers(nil, nil, device_type), asc) do
    local _manuf = string.gsub(string.gsub(manuf, "'", "&#39;"), "\"", "&quot;")
       manufacturer_params.manufacturer = manuf
-      print('<li')
-      if manufacturer == manuf then print(' class="active"') end
-      print('><a href="'..getPageUrl(base_url, manufacturer_params)..'">'.._manuf..' ('..count..')'..'</a></li>')
+      print('<li><a class="dropdown-item '.. (manufacturer == manuf and 'active' or '') ..'" href="'..getPageUrl(base_url, manufacturer_params)..'">'.._manuf..' ('..count..')'..'</a></li>')
    end
    print[[
        </ul>\
@@ -160,20 +142,19 @@ for manuf, count in pairsByKeys(interface.getMacManufacturers(nil, nil, device_t
    ']]
 
    -- Filter Device Type
+   -- table.clone needed to modify some parameters while keeping the original unchanged
    local devicetype_params = table.clone(page_params)
    devicetype_params.device_type = nil
    print[[, '\
-       <div class="btn-group pull-right">\
-       <button class="btn btn-link dropdown-toggle" data-toggle="dropdown">]] print(i18n("details.device_type")) print(devtype_filter) print[[<span class="caret"></span></button>\
-       <ul class="dropdown-menu" role="menu" id="flow_dropdown">\
-          <li><a href="]] print(getPageUrl(base_url, devicetype_params)) print[[">]] print(i18n("mac_stats.all_devices")) print[[</a></li>\
+       <div class="btn-group float-right">\
+       <button class="btn btn-link dropdown-toggle" data-bs-toggle="dropdown">]] print(i18n("details.device_type")) print(devtype_filter) print[[<span class="caret"></span></button>\
+       <ul class="dropdown-menu scrollable-dropdown" role="menu" id="flow_dropdown">\
+          <li><a class="dropdown-item" href="]] print(getPageUrl(base_url, devicetype_params)) print[[">]] print(i18n("mac_stats.all_devices")) print[[</a></li>\
    ]]
 
    for typeidx, count in pairsByKeys(interface.getMacDeviceTypes(nil, nil, manufacturer, device_type), asc) do
       devicetype_params.device_type = typeidx
-      print('<li')
-      if typeidx == device_type then print(' class="active"') end
-      print('><a href="'..getPageUrl(base_url, devicetype_params)..'">'.. discover.devtype2string(typeidx) ..' ('..count..')'..'</a></li>')
+      print('<li><a class="dropdown-item '.. (typeidx == device_type and 'active' or '') ..'" href="'..getPageUrl(base_url, devicetype_params)..'">'.. discover.devtype2string(typeidx) ..' ('..count..')'..'</a></li>')
    end
    print[[
        </ul>\
@@ -256,7 +237,7 @@ print [[
 			     title: "]] print(i18n("breakdown")) print[[",
 				 field: "column_breakdown",
 				 sortable: false,
-	 	             css: { 
+	 	             css: {
 			        textAlign: 'center'
 			     }
 				 },
@@ -264,7 +245,7 @@ print [[
 			     title: "]] print(i18n("throughput")) print[[",
 				 field: "column_thpt",
 				 sortable: true,
-	 	             css: { 
+	 	             css: {
 			        textAlign: 'right'
 			     }
 				 },
@@ -272,7 +253,7 @@ print [[
 			     title: "]] print(i18n("traffic")) print[[",
 				 field: "column_traffic",
 				 sortable: true,
-	 	             css: { 
+	 	             css: {
 			        textAlign: 'right'
 			     }
 				 }
