@@ -1,5 +1,5 @@
 --
--- (C) 2017-18 - ntop.org
+-- (C) 2017-22 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -9,15 +9,24 @@ local os_utils = require "os_utils"
 
 local callback_utils = {}
 
+local clock_start = os.clock()
+
 -- ########################################################
 
 -- Iterates available interfaces, excluding PCAP interfaces.
 -- Each valid interface is select-ed and passed to the callback.
-function callback_utils.foreachInterface(ifnames, condition, callback)
+-- NOTE: direction must only be used by second.lua
+function callback_utils.foreachInterface(ifnames, condition, callback, update_direction_stats)
    for _,_ifname in pairs(ifnames) do
       if(ntop.isShutdown()) then return true end
 
+      -- NOTE: "eth" will be overwritten here for emulated directions
       interface.select(_ifname)
+
+      if update_direction_stats then
+	 interface.updateDirectionStats()
+      end
+
       local ifstats = interface.getStats()
 
       if condition == nil or condition(ifstats.id) then
@@ -59,7 +68,9 @@ local function getBatchedIterator(batched_function, field, function_params)
             iterator_finished = true
          end
 
-         if iterator_finished then return nil end
+         if iterator_finished then
+	    return nil
+	 end
 
          -- we need to load new slots from C
          if(debug_enabled) then
@@ -78,8 +89,9 @@ local function getBatchedIterator(batched_function, field, function_params)
          loaded_elems = slot[field]
 
          if(debug_enabled) then
-            io.write("getBatchedIterator["..field.."](numElems=".. table.len(loaded_elems) ..", nextSlot=".. nextSlot ..")\n")
+            io.write("getBatchedIterator["..field.."] nextSlot=".. nextSlot ..")\n")
          end
+
       end
 
       for key, value in pairs(loaded_elems) do
@@ -89,6 +101,12 @@ local function getBatchedIterator(batched_function, field, function_params)
    end
 end
 
+-- A batched iterator over the active flows
+-- @param flows_filter A table containing flow filters matching those specified in Paginator.cpp
+function callback_utils.getFlowsIterator(flows_filter)
+   return getBatchedIterator(interface.getBatchedFlowsInfo, "flows",  flows_filter)
+end
+
 -- A batched iterator over the local hosts with timeseries
 function callback_utils.getLocalHostsTsIterator(...)
    return getBatchedIterator(interface.getBatchedLocalHostsTs, "hosts", { ... })
@@ -116,22 +134,48 @@ end
 
 -- ########################################################
 
--- Iterates each active host on the ifname interface for RRD creation.
--- Each host is passed to the callback with some more information.
-function callback_utils.foreachLocalRRDHost(ifname, deadline, with_ts, callback)
+-- Iterates each active flow on the ifname interface.
+-- Each flow is passed to the callback with some more information.
+function callback_utils.foreachFlow(ifname, deadline, callback, ...)
    interface.select(ifname)
 
+   local iterator = callback_utils.getFlowsIterator({...})
+
+   for flow_key, flow in iterator do
+
+      if(ntop.isShutdown()) then return true end
+
+      if ((deadline ~= nil) and (os.time() >= deadline)) then
+	 -- Out of time
+	 return false
+      end
+
+      if callback(flow_key, flow) == false then
+	 return false
+      end
+   end
+
+   return true
+end
+
+-- ########################################################
+
+-- Iterates each active host on the ifname interface for timeseries creation.
+-- Each host is passed to the callback with some more information.
+function callback_utils.foreachLocalTimeseriesHost(ifname, with_ts, with_one_way_traffic_hosts, callback)
    local iterator
+   
+   interface.select(ifname)
 
    if with_ts then
-      iterator = callback_utils.getLocalHostsTsIterator()
+      iterator = callback_utils.getLocalHostsTsIterator(nil --[[ show_details --]], nil --[[ maxHits --]], nil --[[ anomalousOnly --]], with_one_way_traffic_hosts)
    else
-      iterator = callback_utils.getLocalHostsIterator(false --[[no details]])
+      iterator = callback_utils.getLocalHostsIterator(false --[[ show_details --]], nil --[[ maxHits --]], nil --[[ anomalousOnly --]], with_one_way_traffic_hosts)
    end
 
    for hostname, host_ts in iterator do
       if(ntop.isShutdown()) then return true end
-      if ((deadline ~= nil) and (os.time() >= deadline)) then
+      if ntop.isDeadlineApproaching() then
 	 -- Out of time
 	 return false
       end
@@ -148,7 +192,7 @@ end
 
 -- Iterates each active host on the ifname interface.
 -- Each host is passed to the callback with some more information.
-function callback_utils.foreachHost(ifname, deadline, callback)
+function callback_utils.foreachHost(ifname, callback)
    interface.select(ifname)
 
    local iterator = callback_utils.getHostsIterator(false --[[ no details ]])
@@ -156,7 +200,7 @@ function callback_utils.foreachHost(ifname, deadline, callback)
    for hostname, hoststats in iterator do
       if(ntop.isShutdown()) then return true end
 
-      if ((deadline ~= nil) and (os.time() >= deadline)) then
+      if ntop.isDeadlineApproaching() then
 	 -- Out of time
 	 return false
       end
@@ -173,7 +217,7 @@ end
 
 -- Iterates each active host on the ifname interface.
 -- Each host is passed to the callback with some more information.
-function callback_utils.foreachLocalHost(ifname, deadline, callback)
+function callback_utils.foreachLocalHost(ifname, callback)
    interface.select(ifname)
 
    local iterator = callback_utils.getLocalHostsIterator(false --[[ no details ]])
@@ -181,7 +225,7 @@ function callback_utils.foreachLocalHost(ifname, deadline, callback)
    for hostname, hoststats in iterator do
       if(ntop.isShutdown()) then return true end
 
-      if ((deadline ~= nil) and (os.time() >= deadline)) then
+      if ntop.isDeadlineApproaching() then
 	 -- Out of time
 	 return false
       end
@@ -196,7 +240,7 @@ end
 
 -- Iterates each device on the ifname interface.
 -- Each device is passed to the callback with some more information.
-function callback_utils.foreachDevice(ifname, deadline, callback)
+function callback_utils.foreachDevice(ifname, callback)
    interface.select(ifname)
 
    local devices_stats = callback_utils.getDevicesIterator()
@@ -205,7 +249,7 @@ function callback_utils.foreachDevice(ifname, deadline, callback)
       if(ntop.isShutdown()) then return true end
       devicename = hostinfo2hostkey(devicestats) -- make devicename the combination of mac address and vlan
 
-      if ((deadline ~= nil) and (os.time() >= deadline)) then
+      if ntop.isDeadlineApproaching() then
          -- Out of time
          return false
       end
@@ -223,6 +267,7 @@ end
 function callback_utils.uploadTSdata()
    local ts_utils = require("ts_utils_core")
    local drivers = ts_utils.listActiveDrivers()
+   ts_utils.setup()
 
    for _, driver in ipairs(drivers) do
       driver:export()
@@ -230,4 +275,8 @@ function callback_utils.uploadTSdata()
 end
 -- ########################################################
 
+if(trace_script_duration ~= nil) then
+   io.write(debug.getinfo(1,'S').source .." executed in ".. (os.clock()-clock_start)*1000 .. " ms\n")
+end
+
 return callback_utils
