@@ -1,7 +1,9 @@
 --
--- (C) 2014-18 - ntop.org
+-- (C) 2014-22 - ntop.org
 --
 local dirs = ntop.getDirs()
+
+require("lua_utils")
 local ts_utils = require("ts_utils")
 local os_utils = require("os_utils")
 local tracker = require "tracker"
@@ -59,19 +61,24 @@ end
 
 -- ################################################################
 
+-- TODO delete HOST_V4_BY_MAC_SERIALIZED_KEY and HOST_V6_BY_MAC_SERIALIZED_KEY as well
 local function delete_host_redis_keys(interface_id, host_info)
    local status = "OK"
-   local serialized_k, dns_k, devnames_k, devtypes_k
+   local hostkey = hostinfo2hostkey(host_info, nil, true)
+   local serialized_k, dns_k, devnames_k, devtypes_k, drop_k, label_k, dhcp_k
 
    if not isMacAddress(host_info["host"]) then
       -- this is an IP address, see HOST_SERIALIZED_KEY (ntop_defines.h)
-      serialized_k = string.format("ntopng.serialized_hosts.ifid_%u__%s@%d", interface_id, host_info["host"], host_info["vlan"] or "0")
+      serialized_k = string.format("ntopng.serialized_hosts.ifid_%d__%s@%d", interface_id, host_info["host"], host_info["vlan"] or "0")
       dns_k = string.format("ntopng.dns.cache.%s", host_info["host"]) -- neither vlan nor ifid implemented for the dns cache
-   elseif isIPv4(host_info["host"]) or isIPv6(host_info["host"]) then
+      drop_k = "ntopng.prefs.drop_host_traffic"
+      label_k = getHostAltNamesKey(hostkey)
+   else
       -- is a mac address, see MAC_SERIALIED_KEY (see ntop_defines.h)
-      serialized_k = string.format("ntopng.serialized_macs.ifid_%u__%s", interface_id, host_info["host"])
+      serialized_k = string.format("ntopng.serialized_macs.ifid_%d__%s", interface_id, host_info["host"])
       devnames_k = string.format("ntopng.cache.devnames.%s", host_info["host"])
       devtypes_k = string.format("ntopng.prefs.device_types.%s", host_info["host"])
+      dhcp_k = getDhcpNameKey(interface_id, host_info["host"])
    end
 
    if not dry_run then
@@ -79,6 +86,9 @@ local function delete_host_redis_keys(interface_id, host_info)
       if devnames_k   then ntop.delCache(devnames_k) end
       if devtypes_k   then ntop.delCache(devtypes_k) end
       if dns_k        then ntop.delCache(dns_k) end
+      if dhcp_k       then ntop.delCache(dhcp_k) end
+      if drop_k       then ntop.delHashCache(drop_k, hostkey) end
+      if label_k      then ntop.delCache(label_k) end
    end
 
    return {status = status}
@@ -88,22 +98,43 @@ end
 
 local function delete_host_mysql_flows(interface_id, host_info)
    local status = "OK"
+   local addr = host_info["host"]
+   local vlan = host_info["vlan"] or 0
+   local q
 
-   if ntop.getPrefs()["is_dump_flows_to_mysql_enabled"] then
-      local addr = host_info["host"]
-      local vlan = host_info["vlan"] or 0
-      local q
+   if ntop.isClickHouseEnabled() then
+      local historical_flow_utils = require "historical_flow_utils"
+ 
+      local where_clause
 
       if isIPv4(addr) then
-	 q = string.format("DELETE FROM %s WHERE (IP_SRC_ADDR = INET_ATON('%s') OR IP_DST_ADDR = INET_ATON('%s')) AND VLAN_ID = %u and INTERFACE_ID = %u",
-			   "flowsv4", addr, addr, vlan, interface_id)
+         where_clause = string.format("(IPV4_SRC_ADDR = ('%s') OR IPV4_DST_ADDR = ('%s')) AND VLAN_ID = %u AND INTERFACE_ID = %d",
+            addr, addr, vlan, interface_id)
+      else
+         where_clause = string.format("(IPV6_SRC_ADDR = ('%s') OR IPV6_DST_ADDR = ('%s')) AND VLAN_ID = %u AND INTERFACE_ID = %d",
+            addr, addr, vlan, interface_id)
+      end
+
+      where_clause = historical_flow_utils.fixWhereTypes(where_clause)
+
+      q = string.format("ALTER TABLE `flows` DELETE WHERE %s", where_clause)
+
+      if not dry_run and q then
+         interface.execSQLQuery(q)
+      end
+
+   elseif ntop.getPrefs()["is_dump_flows_to_mysql_enabled"] then
+
+      if isIPv4(addr) then
+         q = string.format("DELETE FROM %s WHERE (IP_SRC_ADDR = INET_ATON('%s') OR IP_DST_ADDR = INET_ATON('%s')) AND VLAN_ID = %u and INTERFACE_ID = %d",
+            "flowsv4", addr, addr, vlan, interface_id)
       elseif isIPv6(addr) then
-	 q = string.format("DELETE FROM %s WHERE (IP_SRC_ADDR = '%s' OR IP_DST_ADDR = '%s') AND VLAN_ID = %u AND INTERFACE_ID = %u",
-			   "flowsv6", addr, addr, vlan, interface_id)
+         q = string.format("DELETE FROM %s WHERE (IP_SRC_ADDR = '%s' OR IP_DST_ADDR = '%s') AND VLAN_ID = %u AND INTERFACE_ID = %d",
+            "flowsv6", addr, addr, vlan, interface_id)
       end
 
       if not dry_run and q then
-	 interface.execSQLQuery(q)
+         interface.execSQLQuery(q)
       end
    end
 
@@ -112,20 +143,110 @@ end
 
 -- ################################################################
 
-function delete_data_utils.delete_host(interface_id, host_info)
+local function _delete_host(interface_id, host_info)
+   local old_ifname = interface.getStats().name
+   local is_mac = isMacAddress(host_info["host"])
+   interface.select(getInterfaceName(interface_id))
+
+   if is_mac then
+      interface.deleteMacData(host_info["host"])
+   else
+      interface.deleteHostData(hostinfo2hostkey(host_info))
+   end
+
    local h_ts = delete_host_timeseries_data(interface_id, host_info)
    local h_rk = delete_host_redis_keys(interface_id, host_info)
    local h_db = delete_host_mysql_flows(interface_id, host_info)
 
-   -- TRACKER HOOK
-   tracker.log('delete_host', { hostinfo2hostkey(host_info) })
-
+   interface.select(old_ifname)
    return {delete_host_timeseries_data = h_ts, delete_host_redis_keys = h_rk, delete_host_mysql_flows = h_db}
 end
 
 -- ################################################################
 
-local function delete_interfaces_redis_keys(interfaces_list)
+function delete_data_utils.delete_host(interface_id, host_info)
+   -- TRACKER HOOK
+   tracker.log('delete_host', { hostinfo2hostkey(host_info) })
+
+   return _delete_host(interface_id, host_info)
+end
+
+-- ################################################################
+
+function delete_data_utils.delete_network(interface_id, netaddr, mask, vlan)
+   mask = tonumber(mask)
+
+   -- TRACKER HOOK
+   tracker.log('delete_network', { hostinfo2hostkey({host=netaddr.."/"..mask, vlan=vlan}) })
+
+   if not isIPv4(netaddr) then
+      return {delete_network = {status = "ERR_INVALID_IPV4_NETWORK"}}
+   end
+
+   if (mask < 24) or (mask > 32) then
+      return {delete_network = {status = "ERR_NETWORK_TOO_BIG"}}
+   end
+
+   local prefix = ntop.networkPrefix(netaddr, tonumber(mask))
+   local parts = string.split(prefix, "%.")
+   local start = tonumber(parts[4])
+
+   local size = 1 << (32-mask)
+
+   for i=0,size-1 do
+      parts[4] = tostring(start + i)
+
+      local ip = table.concat(parts, ".")
+      local status = _delete_host(interface_id, {host=ip, vlan=vlan})
+
+      for what, what_res in pairs(status) do
+	 if what_res["status"] ~= "OK" then
+	    return status
+	 end
+      end
+   end
+
+   -- Delete network ts data
+   if not ts_utils.delete("subnet", {ifid=interface_id, subnet=prefix.."/"..mask}) then
+      return {delete_network_timeseries = {status = "ERR_TS_DELETE"}}
+   end
+
+   return {delete_network = {status = "OK"}}
+end
+
+-- ################################################################
+
+local function delete_keys_patterns(keys_patterns, preserve_prefs)
+  for _, pattern in pairs(keys_patterns) do
+    local matching_keys = ntop.getKeysCache(pattern)
+
+    for matching_key, _ in pairs(matching_keys or {}) do
+	    if((not preserve_prefs) or
+		  ((not starts(matching_key, "ntopng.prefs.")) and
+		   (not starts(matching_key, "ntopng.user.")))) then
+	       if not dry_run then
+		  ntop.delCache(matching_key)
+	       end
+	    end
+	 end
+  end
+end
+
+-- ################################################################
+
+local function delete_system_interface_redis(preserve_prefs)
+  local keys_patterns = {
+    "ntopng.prefs.snmp_devices*",
+    "ntopng.prefs.system_rtt_hosts*",
+    "cachedsnmp*",
+  }
+
+  delete_keys_patterns(keys_patterns, preserve_prefs)
+end
+
+-- ################################################################
+
+local function delete_interfaces_redis_keys(interfaces_list, preserve_prefs)
    local pref_prefix = "ntopng.prefs"
    local status = "OK"
 
@@ -137,55 +258,42 @@ local function delete_interfaces_redis_keys(interfaces_list)
 	 -- examples:
 	 --  ntopng.prefs.0.host_pools.pool_ids
 	 --  ntopng.prefs.0.host_pools.details.0
-	 string.format("%s.%u.*", pref_prefix, if_id),
+	 string.format("%s.%d.*", pref_prefix, if_id),
 	 -- examples:
 	 --  ntopng.profiles_counters.ifid_0
 	 --  ntopng.serialized_host_pools.ifid_0
-	 string.format("ntopng.*ifid_%u", if_id),
+	 string.format("ntopng.*ifid_%d", if_id),
 	 -- examples:
 	 --  ntopng.serialized_macs.ifid_0__52:54:00:3B:CB:B3
 	 --  ntopng.serialized_hosts.ifid_0__192.168.2.131@0
-	 string.format("*.ifid_%u_*", if_id),
+	 string.format("*.ifid_%d_*", if_id),
 	 -- examples:
 	 --  ntopng.cache.engaged_alerts_cache_ifid_4_5mins
 	 --  ntopng.cache.engaged_alerts_cache_ifid_4_min
-	 string.format("ntopng.*_ifid_%u_*", if_id),
+	 string.format("ntopng.*_ifid_%d_*", if_id),
 	 -- examples:
 	 -- ntopng.prefs.ifid_0.custom_nDPI_proto_categories
 	 -- ntopng.prefs.ifid_0.is_traffic_mirrored
-	 string.format("*.ifid_%u.*", if_id),
+	 string.format("*.ifid_%d.*", if_id),
 	 -- examples:
 	 --  ntopng.prefs.iface_2.packet_drops_alert
 	 --  ntopng.prefs.iface_3.scaling_factor
-	 string.format("%s.iface_%u.*", pref_prefix, if_id),
+	 string.format("%s.iface_%d.*", pref_prefix, if_id),
 	 -- examples:
 	 --  ntopng.prefs.enp1s0f0.xxx
 	 string.format("%s.%s.*", pref_prefix, if_name),
 	 -- examples:
 	 --  ntopng.prefs.enp2s0f0_not_idle
 	 string.format("%s.%s_*", pref_prefix, if_name),
+	 -- examples:
+	 --  ntopng.dhcp.0.cache
+	 string.format("ntopng.dhcp.%d.*", if_id),
       }
 
-      for _, pattern in pairs(keys_patterns) do
-	 local matching_keys = ntop.getKeysCache(pattern)
+      delete_keys_patterns(keys_patterns, preserve_prefs)
 
-	 for matching_key, _ in pairs(matching_keys or {}) do
-	    if not dry_run then
-	       ntop.delCache(matching_key)
-	    end
-	 end
-      end
-   end
-
-   if ntop.isnEdge() then
-      -- clear captive portal users
-      -- (their host pools have already been deleted above)
-      local ntop_users = ntop.getUsers()
-
-      for username, user_details in pairs(ntop_users) do
-	 if user_details["group"] == "captive_portal" then
-	    ntop.deleteUser(username)
-	 end
+      if(if_id == getSystemInterfaceId()) then
+        delete_system_interface_redis(preserve_prefs)
       end
    end
 
@@ -199,7 +307,7 @@ local function delete_interfaces_data(interfaces_list)
    local data_dir = ntop.getDirs()["workingdir"]
 
    for if_id, if_name in pairs(interfaces_list) do
-      local if_dir = os_utils.fixPath(string.format("%s/%u/", data_dir, if_id))
+      local if_dir = os_utils.fixPath(string.format("%s/%d/", data_dir, if_id))
 
       if not dry_run then
 	 if not ts_utils.delete("" --[[ all schemas ]], {ifid=if_id}) then
@@ -220,14 +328,6 @@ end
 
 -- ################################################################
 
-local function delete_interfaces_influx_data(interfaces_list)
-   local status = "OK"
-   -- TODO
-   return {status = status}
-end
-
--- ################################################################
-
 local function delete_interfaces_db_flows(interfaces_list)
    local db_utils = require "db_utils"
    local status = "OK"
@@ -249,7 +349,9 @@ end
 local function delete_interfaces_ids(interfaces_list)
    local status = "OK"
 
-   for if_id, if_name in pairs(interfaces_list) do
+   for if_id, _ in pairs(interfaces_list) do
+      local if_name = ntop.getHashCache(ALL_INTERFACES_HASH_KEYS, if_id)
+
       -- delete the interface from the all interfaces hash
       -- this will cause the id to be re-used
       if not dry_run then
@@ -311,10 +413,11 @@ end
 local function delete_interfaces_from_list(interfaces_list, preserve_interface_ids, preserve_redis_keys)
    local if_dt = delete_interfaces_data(interfaces_list)
    local if_db = delete_interfaces_db_flows(interfaces_list)
+   local preserve_prefs = ternary(ntop.isnEdge(), true, false)
 
    local if_rk
    if not preserve_redis_keys then
-      if_rk = delete_interfaces_redis_keys(interfaces_list)
+      if_rk = delete_interfaces_redis_keys(interfaces_list, preserve_prefs)
    end
 
    -- last step is to also free the ids that can thus be recycled
@@ -372,12 +475,9 @@ end
 
 -- ################################################################
 
-function delete_data_utils.request_delete_active_interface_data(if_name)
-   local if_id = getInterfaceId(if_name)
-
-   if tonumber(if_id) >= 0 then
-      ntop.setHashCache(ACTIVE_INTERFACES_DELETE_HASH, tostring(if_id), if_name)
-   end
+function delete_data_utils.request_delete_active_interface_data(if_id)
+   local if_name = getInterfaceName(if_id)
+   ntop.setHashCache(ACTIVE_INTERFACES_DELETE_HASH, tostring(if_id), if_name)
 end
 
 -- ################################################################
@@ -392,12 +492,11 @@ function delete_data_utils.delete_active_interface_data_requested(if_name)
    if not isEmptyString(if_name) then
       -- Check if a delete has been requested for a particular interface
       local if_id = getInterfaceId(if_name)
-
-      if tonumber(if_id) >= 0 then
-	 local req = ntop.getHashCache(ACTIVE_INTERFACES_DELETE_HASH, tostring(if_id))
-	 if not isEmptyString(req) then
-	    return true
-	 end
+      if tonumber(if_id) >= -1 then
+         local req = ntop.getHashCache(ACTIVE_INTERFACES_DELETE_HASH, tostring(if_id))
+         if not isEmptyString(req) then
+            return true
+         end
       end
    else
       -- Check if there's at least a data delete request
@@ -441,43 +540,74 @@ end
 -- ################################################################
 
 -- NOTE: this has 1 day accuracy
-function delete_data_utils.harvestDateBasedDirTree(dir, retention, now)
+function delete_data_utils.harvestDateBasedDirTree(dir, retention, now, verbose)
+   if not ntop.exists(dir) then
+      return
+   end
+
+   if verbose then traceError(TRACE_NORMAL, TRACE_CONSOLE, string.format('Deleting files in %s older than %u days', dir, retention)) end
+
    for year in pairs(ntop.readdir(dir) or {}) do
       local year_path = os_utils.fixPath(dir .. "/" .. year)
       local num_deleted_months = 0
       local tot_months = 0
 
       for month in pairs(ntop.readdir(year_path) or {}) do
-	 local month_path = os_utils.fixPath(year_path .. "/" .. month)
-	 local num_deleted_days = 0
-	 local tot_days = 0
+         local month_path = os_utils.fixPath(year_path .. "/" .. month)
+         local num_deleted_days = 0
+         local tot_days = 0
 
-	 for day in pairs(ntop.readdir(month_path) or {}) do
-	    local tstamp = os.time({day=tonumber(day), month=tonumber(month), year=tonumber(year), hour=0, min=0, sec=0})
-	    local days_diff = (now - tstamp) / 86400
+         for day in pairs(ntop.readdir(month_path) or {}) do
+            if(tonumber(day) ~= nil) then
+               local tstamp = os.time({day=tonumber(day), month=tonumber(month), year=tonumber(year), hour=0, min=0, sec=0})
+               local days_diff = (now - tstamp) / 86400
 
-	    if(days_diff > retention) then
-	       local day_path = os_utils.fixPath(month_path .. "/" .. day)
-	       --tprint(os.date('PURGE %Y-%m-%d %H:%M:%S', tstamp))
-	       ntop.rmdir(day_path)
-	       num_deleted_days = num_deleted_days + 1
-	    end
+               if(days_diff > retention) then
+                  local day_path = os_utils.fixPath(month_path .. "/" .. day)
+                  if verbose then traceError(TRACE_NORMAL, TRACE_CONSOLE, os.date('PURGE day: %Y-%m-%d', tstamp)) end
 
-	    tot_days = tot_days + 1
-	 end
+                  if not dry_run then
+                     ntop.rmdir(day_path)
+                  end
 
-	 if num_deleted_days == tot_days then
-	    --tprint("PURGE month: ".. month .."/" .. year)
-	    ntop.rmdir(month_path)
-	    num_deleted_months = num_deleted_months + 1
-	 end
+                  num_deleted_days = num_deleted_days + 1
+               else
+                  if verbose then traceError(TRACE_NORMAL, TRACE_CONSOLE, os.date('Keep day: %Y-%m-%d', tstamp)) end
+               end
 
-	 tot_months = tot_months + 1
+               tot_days = tot_days + 1
+            end
+         end
+
+         if num_deleted_days == tot_days then
+            if verbose then traceError(TRACE_NORMAL, TRACE_CONSOLE, "PURGE month: ".. month .."/" .. year) end
+
+            if not dry_run then
+               ntop.rmdir(month_path)
+            end
+
+            num_deleted_months = num_deleted_months + 1
+         else
+            if verbose then
+               traceError(TRACE_NORMAL, TRACE_CONSOLE,
+                  string.format("Keep month %u/%u: it still has %u days", month, year, tot_days-num_deleted_months))
+            end
+         end
+
+         tot_months = tot_months + 1
       end
 
       if num_deleted_months == tot_months then
-	 --tprint("PURGE year " .. year)
-	 ntop.rmdir(year_path)
+        if verbose then traceError(TRACE_NORMAL, TRACE_CONSOLE, "PURGE year: ".. year) end
+
+        if not dry_run then
+          ntop.rmdir(year_path)
+        end
+      else
+         if verbose then
+            traceError(TRACE_NORMAL, TRACE_CONSOLE,
+               string.format("Keep year %u: it still has %u months", year, tot_months-num_deleted_months))
+         end
       end
    end
 end
