@@ -1,8 +1,9 @@
 --
--- (C) 2017-18 - ntop.org
+-- (C) 2017-22 - ntop.org
 --
-local json = require "dkjson"
 
+local json = require "dkjson"
+local page_utils = require "page_utils"
 require "xmlSimple"
 
 local discover = {}
@@ -11,6 +12,56 @@ discover.debug = false
 
 discover.progress_string = "discovery.progess"
 
+discover.osinfo = {
+  [0] = {"Unknown",       ''},
+  [1] = {"Linux",         '<i class=\'fab fa-linux fa-lg\'></i>'},
+  [2] = {"Windows",       '<i class=\'fab fa-windows fa-lg\'></i>'},
+  [3] = {"MacOS",         '<i class=\'fab fa-apple fa-lg\'></i>'},
+  [4] = {"iOS",           '<i class=\'fab fa-apple fa-lg\'></i>'},
+  [5] = {"Android",       '<i class=\'fab fa-android fa-lg\'></i>'},
+  [6] = {"LaserJET",      'LasetJET'},
+  [7] = {"AppleAirport",  'Apple Airport'},
+  -- NOTE: keep in sync with OperatingSystem in ntop_typedefs.h
+}
+
+-- #################################
+
+function discover.getOsName(id)
+   local info = discover.osinfo[tonumber(id)]
+
+   if(info) then
+    return(info[1])
+   end
+
+   return("")
+end
+
+-- #################################
+
+function discover.getOsIcon(id)
+  local info = discover.osinfo[tonumber(id)]
+
+   if(info) then
+    return(info[2] .. " ")
+   end
+
+   return("")
+end
+
+-- #################################
+
+function discover.getOsAndIcon(id)
+  local name = discover.getOsName(id)
+
+  if(isEmptyString(name)) then
+    return(id)
+  else
+    return(discover.getOsIcon(id) .. name)
+  end
+end
+
+-- #################################
+
 discover.apple_osx_versions = {
    ['4'] = 'Mac OS X 10.0 (Cheetah)',
    ['5'] = 'Mac OS X 10.1 (Puma)',
@@ -26,6 +77,8 @@ discover.apple_osx_versions = {
    ['15'] = 'OS X 10.11 (El Capitan)',
    ['16'] = 'OS X 10.12 (Sierra)',
    ['17'] = 'OS X 10.13 (High Sierra)',
+   ['18'] = 'macOS 10.14 (Mojave)',
+   ['19'] = 'macOS 10.15 (Catalina)'
 }
 
 discover.apple_products = {
@@ -134,47 +187,49 @@ discover.apple_products = {
 
 discover.asset_icons = {
    ['unknown']     = '',
-   ['printer']     = '<i class="fa fa-print fa-lg devtype-icon" aria-hidden="true"></i>', -- 1
-   ['video']       = '<i class="fa fa-video-camera fa-lg devtype-icon" aria-hidden="true"></i>', -- 2
-   ['workstation'] = '<i class="fa fa-desktop fa-lg devtype-icon" aria-hidden="true"></i>', -- ... and so on
-   ['laptop']      = '<i class="fa fa-laptop fa-lg devtype-icon" aria-hidden="true"></i>',
-   ['tablet']      = '<i class="fa fa-tablet fa-lg devtype-icon" aria-hidden="true"></i>',
-   ['phone']       = '<i class="fa fa-mobile fa-lg devtype-icon" aria-hidden="true"></i>',
-   ['tv']          = '<i class="fa fa-television fa-lg devtype-icon" aria-hidden="true"></i>',
-   ['networking']  = '<i class="fa fa-arrows fa-lg devtype-icon" aria-hidden="true"></i>',
-   ['wifi']        = '<i class="fa fa-wifi fa-lg devtype-icon" aria-hidden="true"></i>',
-   ['nas']         = '<i class="fa fa-database fa-lg devtype-icon" aria-hidden="true"></i>',
-   ['multimedia']  = '<i class="fa fa-music fa-lg devtype-icon" aria-hidden="true"></i>',
-   ['iot']         = '<i class="fa fa-thermometer fa-lg devtype-icon" aria-hidden="true"></i>',
+   ['printer']     = '<i class="fas fa-print fa-lg devtype-icon" aria-hidden="true"></i>', -- 1
+   ['video']       = '<i class="fas fa-video fa-lg devtype-icon" aria-hidden="true"></i>', -- 2
+   ['workstation'] = '<i class="fas fa-desktop fa-lg devtype-icon" aria-hidden="true"></i>', -- ... and so on
+   ['laptop']      = '<i class="fas fa-laptop fa-lg devtype-icon" aria-hidden="true"></i>',
+   ['tablet']      = '<i class="fas fa-tablet fa-lg devtype-icon" aria-hidden="true"></i>',
+   ['phone']       = '<i class="fas fa-mobile fa-lg devtype-icon" aria-hidden="true"></i>',
+   ['tv']          = '<i class="fas fa-tv fa-lg devtype-icon" aria-hidden="true"></i>',
+   ['networking']  = '<i class="fas fa-arrows-alt fa-lg devtype-icon" aria-hidden="true"></i>',
+   ['wifi']        = '<i class="fas fa-wifi fa-lg devtype-icon" aria-hidden="true"></i>',
+   ['nas']         = '<i class="fas fa-database fa-lg devtype-icon" aria-hidden="true"></i>',
+   ['multimedia']  = '<i class="fas fa-music fa-lg devtype-icon" aria-hidden="true"></i>',
+   ['iot']         = '<i class="fas fa-thermometer fa-lg devtype-icon" aria-hidden="true"></i>',
    -- IMPORTANT: please keep in sync asset_icons with id2label
 }
 
 discover.extra_asset_icons = {
-   ['lightbulb']      = '<i class="fa fa-lightbulb-o fa-lg devtype-icon" aria-hidden="true"></i>',
-   ['phone']          = '<i class="fa fa-phone fa-lg devtype-icon" aria-hidden="true"></i>',
-   ['health']         = '<i class="fa fa-medkit fa-lg devtype-icon" aria-hidden="true"></i>',
+   ['lightbulb']      = '<i class="fas fa-lightbulb fa-lg devtype-icon" aria-hidden="true"></i>',
+   ['phone']          = '<i class="fas fa-phone fa-lg devtype-icon" aria-hidden="true"></i>',
+   ['health']         = '<i class="fas fa-medkit fa-lg devtype-icon" aria-hidden="true"></i>',
 }
 
 local id2label = {
+ -- ID      string_id    label					special_purpose_device
    [0]  = { 'unknown', i18n("device_types.unknown") },
-   [1]  = { 'printer', i18n("device_types.printer") },
-   [2]  = { 'video', i18n("device_types.video") },
+   [1]  = { 'printer', i18n("device_types.printer"), 		true },
+   [2]  = { 'video', i18n("device_types.video"),		true },
    [3]  = { 'workstation', i18n("device_types.workstation") },
    [4]  = { 'laptop', i18n("device_types.laptop") },
    [5]  = { 'tablet', i18n("device_types.tablet") },
    [6]  = { 'phone', i18n("device_types.phone") },
-   [7]  = { 'tv', i18n("device_types.tv") },
-   [8]  = { 'networking', i18n("device_types.networking") },
-   [9]  = { 'wifi', i18n("device_types.wifi") },
-   [10] = { 'nas', i18n("device_types.nas") },
-   [11] = { 'multimedia', i18n("device_types.multimedia") },
-   [12] = { 'iot', i18n("device_types.iot") },
+   [7]  = { 'tv', i18n("device_types.tv"),			true },
+   [8]  = { 'networking', i18n("device_types.networking"),	true },
+   [9]  = { 'wifi', i18n("device_types.wifi"),			true },
+   [10] = { 'nas', i18n("device_types.nas"),			true },
+   [11] = { 'multimedia', i18n("device_types.multimedia"),	true },
+   [12] = { 'iot', i18n("device_types.iot"),			true },
    -- IMPORTANT: please keep in sync asset_icons with id2label
 }
 
-discover.ghost_icon = '<i class="fa fa-snapchat-ghost fa-lg" aria-hidden="true"></i>'
-discover.android_icon = '<i class="fa fa-android fa-lg" aria-hidden="true"></i>'
-discover.apple_icon = '<i class="fa fa-apple fa-lg" aria-hidden="true"></i>'
+discover.ghost_icon = '<i class="fas fa-ghost fa-lg" aria-hidden="true"></i>'
+discover.android_icon = '<i class="fab fa-android fa-lg" aria-hidden="true"></i>'
+discover.apple_icon = '<i class="fab fa-apple fa-lg" aria-hidden="true"></i>'
+discover.MAX_DISCOVERED_DEVICES = 8192
 
 -- ################################################################################
 
@@ -203,8 +258,14 @@ end
 
 -- ################################################################################
 
-function discover.getCachedDiscoveryKey(interface_name)
-   return "ntopng.cache.ifid_"..getInterfaceId(interface_name)..".device_discovery"
+local function getCachedDiscoveryKey(interface_name)
+   return "ntopng.cache.ifid_"..getInterfaceId(interface_name)..".device_discovery_status"
+end
+
+-- ################################################################################
+
+local function getCachedDiscoveredDevicesKey(interface_name)
+   return "ntopng.cache.ifid_"..getInterfaceId(interface_name)..".discovered_devices"
 end
 
 -- ################################################################################
@@ -217,7 +278,7 @@ end
 
 function discover.interfaceNetworkDiscoveryEnabled(ifid)
    -- must be discoverable and must have not been disabled by the interface preferences
-   return interface.isDiscoverableInterface() 
+   return interface.isDiscoverableInterface()
           and not (ntop.getPref(discover.getInterfaceNetworkDiscoveryEnabledKey(ifid)) == "false")
 end
 
@@ -235,7 +296,7 @@ end
 
 -- ################################################################################
 
-function discover.clearNetworkDiscovery(ifid)   
+function discover.clearNetworkDiscovery(ifid)
    ntop.delCache(getRequestNetworkDiscoveryKey(ifid))
 end
 
@@ -252,9 +313,9 @@ function isPrinterMDNS(mdns)
 
    if(discover.debug) then
       tprint(mdns)
-      io.write(debug.traceback())      
+      io.write(debug.traceback())
    end
-   
+
    if((mdns["_printer._tcp.local"] ~= nil)
       or (mdns["_scanner._tcp.local"] ~= nil)
       or (mdns["_pdl-datastream._tcp.local"] ~= nil)
@@ -269,7 +330,7 @@ end
 
 function isNASMDNS(mdns)
    if(mdns == nil) then return false end
-   
+
    if((mdns["_edcp._udp.local"] ~= nil) or (mdns["_afpovertcp._tcp.local"] ~= nil) or (mdns["_smb._tcp.local"] ~= nil)) then
       if(mdns["_companion-link._tcp.local"] == nil) then
 	 return true
@@ -281,6 +342,25 @@ end
 
 -- ################################################################################
 
+function isGHomeMDNS(mdns)
+    if(mdns == nil) then return false end
+
+    if(discover.debug) then
+      tprint(mdns)
+      io.write(debug.traceback())
+    end
+
+    if((mdns["_googlecast._tcp.local"] ~= nil)
+      or (mdns["_googlezone._tcp.local"] ~= nil)
+    ) then
+      return true
+    end
+
+    return false
+end
+
+-- ################################################################################
+
 function discover.printDeviceTypeSelectorOptions(device_type, skip_unknown)
    device_type = tonumber(device_type)
 
@@ -299,7 +379,7 @@ end
 -- ################################################################################
 
 function discover.printDeviceTypeSelector(device_type, field_name)
-   print [[<div class="form-group"><select name="]] print(field_name) print[[" class="form-control">\
+   print [[<div class="form-group mb-3"><select name="]] print(field_name) print[[" class="form-select">\
    <option value="0"></option>]]
 
    discover.printDeviceTypeSelectorOptions(device_type, true)
@@ -319,6 +399,16 @@ end
 
 -- ################################################################################
 
+function discover.isValidDevtype(devtype)
+   for k,v in pairs(id2label) do
+      if(v[1] == devtype) then return(true) end
+   end
+
+   return(false)
+end
+
+-- ################################################################################
+
 function discover.devtype2id(devtype)
    for k,v in pairs(id2label) do
       if(v[1] == devtype) then return k end
@@ -350,6 +440,46 @@ end
 
 -- ################################################################################
 
+function discover.isSpecialPurposeDevice(id)
+   local v = id2label[id]
+
+   if(v) then
+      return(v[3] or false)
+   end
+
+   return(false)
+end
+
+-- ################################################################################
+
+function discover.getGeneralPurposeDevicesList()
+   local rv = {}
+
+   for k, v in pairsByField(id2label, 2, asc) do
+      if not v[3] then
+	 rv[#rv + 1] = v[2]
+      end
+   end
+
+   return rv
+end
+
+-- ################################################################################
+
+function discover.getSpecialPurposeDevicesList()
+      local rv = {}
+
+   for k, v in pairsByField(id2label, 2, asc) do
+      if v[3] then
+	 rv[#rv + 1] = v[2]
+      end
+   end
+
+   return rv
+end
+
+-- ################################################################################
+
 local function getbaseURL(url)
    local name = url:match( "([^/]+)$" )
 
@@ -391,15 +521,15 @@ local function probeSSH(ip)
 	 local _osvers = string.split(osvers, "-")
 	 if(_osvers ~= nil) then
 	    local n
-	    
+
 	    if(use_last) then
 	       n = #_osvers
 	    else
 	       n = 1
 	    end
-	    
+
 	    osvers = _osvers[n]
-	    
+
 	    -- tprint(_osvers)
 	    return(trimSpace(osvers))
 	 end
@@ -411,15 +541,15 @@ end
 
 local function appendSSHOS(mac, ip)
    local r = probeSSH(ip)
-   
+
    if((r ~= nil) and (r ~= "")) then
       if(string.contains(r, "Debian")
 	    or string.contains(r, "Raspbian")
 	    or string.contains(r, "dropbear")
 	 or string.contains(r, "Ubuntu")) then
-	 interface.setMacOperatingSystem(mac, 1) -- 1 = Linux
+	 interface.setHostOperatingSystem(ip, 1) -- 1 = Linux
       elseif(string.contains(r, "MS")) then
-	 interface.setMacOperatingSystem(mac, 2) -- 2 = windows
+	 interface.setHostOperatingSystem(ip, 2) -- 2 = windows
       end
 
       return(" ("..r..")")
@@ -529,19 +659,23 @@ local function findDevice(ip, mac, manufacturer, _mdns, ssdp_str, ssdp_entries,
       ret = '</i>'..discover.asset_icons[icon]..' ' .. discover.apple_icon
 
       if(osx ~= nil) then ret = ret .. osx end
-      interface.setMacOperatingSystem(mac, 3) -- 3 = OSX
+      interface.setHostOperatingSystem(ip, 3) -- 3 = OSX
       if(discover.debug) then io.write(debug.traceback()) end
       return icon, ret, nil
    elseif(mdns["_nvstream_dbd._tcp.local"] ~= nil) then
-      interface.setMacOperatingSystem(mac, 2) -- 2 = windows
+      interface.setHostOperatingSystem(ip, 2) -- 2 = windows
       if(discover.debug) then io.write(debug.traceback()) end
       return 'workstation', discover.asset_icons['workstation']..' (Windows)', nil
+   elseif(isGHomeMDNS(mdns)) then
+      -- Google Home
+      if(discover.debug) then io.write(debug.traceback()) end
+      return 'multimedia', discover.asset_icons['multimedia'], nil
    elseif(mdns["_workstation._tcp.local"] ~= nil) then
-      interface.setMacOperatingSystem(mac, 1) -- 1 = Linux
+      interface.setHostOperatingSystem(ip, 1) -- 1 = Linux
       if(discover.debug) then io.write(debug.traceback()) end
       return 'workstation', discover.asset_icons['workstation']..' (Linux)', nil
    else
-      interface.setMacOperatingSystem(mac, 0) -- Unknown
+      interface.setHostOperatingSystem(ip, 0) -- Unknown
    end
 
    if(string.contains(friendlyName, "TV")) then
@@ -618,7 +752,7 @@ local function findDevice(ip, mac, manufacturer, _mdns, ssdp_str, ssdp_entries,
       if(discover.debug) then io.write(debug.traceback()) end
       return 'networking', discover.asset_icons['networking'], nil
    elseif(string.contains(manufacturer, "routerboard")
-	     or string.contains(manufacturer, "netgear") -- 
+	     or string.contains(manufacturer, "netgear") --
 	     or string.contains(manufacturer, "tp-link")
    ) then
       if(discover.debug) then io.write(debug.traceback()) end
@@ -652,11 +786,12 @@ local function findDevice(ip, mac, manufacturer, _mdns, ssdp_str, ssdp_entries,
 	     or string.contains(manufacturer, "samsung electro")
 	     or string.contains(manufacturer, "htc corporation")
 	     or string.contains(manufacturer, "huawei")
-	     or string.contains(manufacturer, "xiaomi communications")
+	     or (string.contains(manufacturer, "xiaomi communications") or string.starts(mac,"44:D1:96"))
+             or string.contains(manufacturer, "oneplus")
 	     or string.contains(manufacturer, "mobile communications") -- LG Electronics (Mobile Communications)
    ) then
       if(discover.debug) then io.write(debug.traceback()) end
-      interface.setMacOperatingSystem(mac, 5) -- 5 = Android
+      interface.setHostOperatingSystem(ip, 5) -- 5 = Android
       return 'phone', discover.asset_icons['phone'].. ' ' ..discover.android_icon, nil
    elseif((string.contains(manufacturer, "hewlett packard")
 	      or string.contains(manufacturer, "hon hai"))
@@ -690,7 +825,7 @@ local function findDevice(ip, mac, manufacturer, _mdns, ssdp_str, ssdp_entries,
       if(discover.debug) then io.write(debug.traceback()) end
       return 'workstation', discover.asset_icons['workstation'].." (VM)", nil
    elseif(string.contains(manufacturer, "xerox")
-	     or string.contains(manufacturer, "ricoh")	  
+	     or string.contains(manufacturer, "ricoh")
    ) then
       local l = ""
 
@@ -699,15 +834,15 @@ local function findDevice(ip, mac, manufacturer, _mdns, ssdp_str, ssdp_entries,
       return 'printer', discover.asset_icons['printer']..l, snmpName
    elseif(string.contains(manufacturer, "apple")) then
       if(string.contains(hostname, "iphone") or string.contains(symName, "iphone")) then
-	 interface.setMacOperatingSystem(mac, 4) -- 4 = iOS
+	 interface.setHostOperatingSystem(ip, 4) -- 4 = iOS
 	 if(discover.debug) then io.write(debug.traceback()) end
 	 return 'phone', discover.asset_icons['phone']..' ('  .. discover.apple_icon .. ' iPhone)', nil
       elseif(string.contains(hostname, "ipad") or string.contains(symName, "ipad")) then
-	 interface.setMacOperatingSystem(mac, 4) -- 4 = iOS
+	 interface.setHostOperatingSystem(ip, 4) -- 4 = iOS
 	 if(discover.debug) then io.write(debug.traceback()) end
 	 return 'tablet', discover.asset_icons['tablet']..' ('  .. discover.apple_icon .. 'iPad)', nil
       elseif(string.contains(hostname, "ipod") or string.contains(symName, "ipod")) then
-	 interface.setMacOperatingSystem(mac, 4) -- 4 = iOS
+	 interface.setHostOperatingSystem(ip, 4) -- 4 = iOS
 	 if(discover.debug) then io.write(debug.traceback()) end
 	 return 'phone', discover.asset_icons['phone']..' ('  .. discover.apple_icon .. 'iPod)', nil
       else
@@ -728,7 +863,7 @@ local function findDevice(ip, mac, manufacturer, _mdns, ssdp_str, ssdp_entries,
 	 end
 
 	 if(snmpName ~= nil) then ret = ret .. " ["..snmpName.."]" end
-	 interface.setMacOperatingSystem(mac, 3) -- 3 = OSX
+	 interface.setHostOperatingSystem(ip, 3) -- 3 = OSX
 	 if(discover.debug) then io.write(debug.traceback()) end
 	 return what, ret, snmpName
       end
@@ -775,14 +910,14 @@ local function findDevice(ip, mac, manufacturer, _mdns, ssdp_str, ssdp_entries,
    end
 
    if(string.starts(hostname, "desktop-") or string.starts(symName, "desktop-")) then
-      interface.setMacOperatingSystem(mac, 2) -- 2 = windows
+      interface.setHostOperatingSystem(ip, 2) -- 2 = windows
       if(discover.debug) then io.write(debug.traceback()) end
       return 'workstation', discover.asset_icons['workstation']..' (Windows)', nil
    elseif(string.contains(hostname, "thinkpad") or string.contains(symName, "thinkpad")) then
       if(discover.debug) then io.write(debug.traceback()) end
       return 'laptop', discover.asset_icons['laptop'], nil
    elseif(string.contains(hostname, "android") or string.contains(symName, "android")) then
-      interface.setMacOperatingSystem(mac, 5) -- 5 = Android
+      interface.setHostOperatingSystem(ip, 5) -- 5 = Android
       if(discover.debug) then io.write(debug.traceback()) end
       return 'phone', discover.asset_icons['phone']..' ' ..discover.android_icon, nil
    elseif(string.contains(hostname, "%-NAS") or string.contains(symName, "%-NAS")) then
@@ -807,7 +942,7 @@ local function findDevice(ip, mac, manufacturer, _mdns, ssdp_str, ssdp_entries,
    else
       snmpName = ' ('..snmpName..')'
    end
-   
+
    if(string.contains(manufacturer, "ubiquiti")) then
       if(discover.debug) then io.write(debug.traceback()) end
       return 'networking', discover.asset_icons['networking']..snmpName, nil
@@ -832,14 +967,14 @@ local function findDevice(ip, mac, manufacturer, _mdns, ssdp_str, ssdp_entries,
    ssh_rsp = probeSSH(ip)
 
    -- Last resort is HTTP
-   http_rsp = ntop.httpGet("http://"..ip, "", "", 1)
+   http_rsp = ntop.httpGet("http://"..ip, nil, nil, 1)
    if((http_rsp ~= nil) and (http_rsp.HTTP_HEADER ~= nil)) then
       local server = http_rsp.HTTP_HEADER["server"]
 
       if(server ~= nil) then
 	 if(discover.debug) then io.write("[SSH] "..server) end
 	 if(string.contains(server, "Ubuntu") or string.contains(server, "Debian") or string.contains(server, "Linux")) then
-	    interface.setMacOperatingSystem(mac, 1) -- 1 = Linux
+	    interface.setHostOperatingSystem(ip, 1) -- 1 = Linux
 	    if(discover.debug) then io.write(debug.traceback()) end
 	    return 'workstation', discover.asset_icons['workstation']..' (Linux)', nil
 	 elseif(string.contains(server, "Apache")) then
@@ -850,7 +985,7 @@ local function findDevice(ip, mac, manufacturer, _mdns, ssdp_str, ssdp_entries,
 	    return 'workstation', discover.asset_icons['workstation']..appendSSHOS(mac, ip), nil
 	 elseif(string.contains(server, "Microsoft")) then
 	    if(discover.debug) then io.write(debug.traceback()) end
-	    interface.setMacOperatingSystem(mac, 2) -- 2 = windows
+	    interface.setHostOperatingSystem(ip, 2) -- 2 = windows
 	    return 'workstation', discover.asset_icons['workstation']..' (Windows)', nil
 	 elseif(string.contains(server, "Virata%-EmWeb") or string.contains(server, "HP%-ChaiSOE") -- Usually LaserJet
 		   or string.contains(server, "EWS%-NIC5") -- Xerox
@@ -881,7 +1016,26 @@ end
 local function analyzeSSDP(ssdp)
    local rsp = {}
 
+   -- Cleanup to avoid SSRF:
+   -- - Don't be tricked into issuing queries to ourselves
+   -- - Only query the same IP of the one that has been discovered
+   for url, host in pairs(ssdp) do
+      if ntop.isLocalInterfaceAddress(host) then
+	 -- No queries to ourselves
+	 ssdp[url] = nil
+      end
+
+      if not url:starts("http://"..host) and not url:starts("https://"..host) then
+	 -- Not using HTTP or HTTPs, or the host is telling us to contact
+	 -- another host different from itself. Skip to avoid SSRF
+	 ssdp[url] = nil
+      end
+   end
+
+   -- NOTE: use page_utils.safe_html to protect agains malicious SSDP responses trying to inject code
    for url,host in pairs(ssdp) do
+      url = ntop.httpPurifyParam(url) -- Cleanup against XSS
+
       local hresp = ntop.httpGet(url, "", "", 3 --[[ seconds ]])
       local manufacturer = ""
       local modelDescription = ""
@@ -898,13 +1052,13 @@ local function analyzeSSDP(ssdp)
 	 if(r.root ~= nil) then
 	    if(r.root.device ~= nil) then
 	       if(r.root.device.friendlyName ~= nil) then
-		  friendlyName = r.root.device.friendlyName:value()
+		  friendlyName = page_utils.safe_html(r.root.device.friendlyName:value())
 	       end
 	       if(r.root.device.modelName ~= nil) then
-		  modelName = r.root.device.modelName:value()
+		  modelName = page_utils.safe_html(r.root.device.modelName:value())
 	       end
 	       if(r.root.device.modelDescription ~= nil) then
-		  modelDescription = r.root.device.modelDescription:value()
+		  modelDescription = page_utils.safe_html(r.root.device.modelDescription:value())
 	       end
 	    end
 	 end
@@ -912,7 +1066,7 @@ local function analyzeSSDP(ssdp)
 	 if(r.root ~= nil) then
 	    if(r.root.device ~= nil) then
 	       if(r.root.device.manufacturer ~= nil) then
-		  manufacturer = r.root.device.manufacturer:value()
+		  manufacturer = page_utils.safe_html(r.root.device.manufacturer:value())
 	       end
 
 	       if(r.root.device.serviceList ~= nil) then
@@ -921,10 +1075,10 @@ local function analyzeSSDP(ssdp)
 
 		  for k,v in pairs(serviceList) do
 		     if(v.serviceId ~= nil) then
-			local value = v.serviceId:value()
+			local value = page_utils.safe_html(v.serviceId:value())
 			if(value ~= nil) then
 			   if(discover.debug) then io.write(value.."\n") end
-			   
+
 			   table.insert(services, value)
 			end
 		     end
@@ -966,7 +1120,7 @@ local function analyzeSSDP(ssdp)
 				 base_url = base_url .. "/"
 			      end
 
-			      icon = "<img src="..base_url..v.url:value()..">"
+			      icon = "<img src="..ntop.httpPurifyParam(base_url..v.url:value())..">"
 			      -- io.write(icon.."\n")
 			      lastwidth = width -- Pick the smallest icon
 			   end
@@ -1013,21 +1167,26 @@ local function discoverARP()
    local res = {}
    local ghost_macs  = {}
    local ghost_found = false
-
-   local arp_mdns = interface.arpScanHosts()
-
+   local arp_mdns = interface.arpScanHosts() -- List of hosts that reply to ARP
+   local now = os.time()
+   
    if(discover.debug) then io.write("Completed ARP discovery...\n") end
 
    if(arp_mdns == nil) then
       status = discoverStatus("ERROR", i18n("discover.err_unable_to_arp_discovery"))
    else
       -- Add the known macs to the list
-      local known_macs = interface.getMacsInfo(nil, 999, 0, false, true, nil) or {}
+      local known_macs = interface.getMacsInfo(nil, 999, 0, false, true, nil, nil, nil, nil, nil) or {}
 
       for _,hmac in pairs(known_macs.macs) do
-	 if((hmac["bytes.sent"] > 0) and (hmac["location"] == "lan")) then -- Skip silent/wan hosts
+	 if((hmac["bytes.sent"] > 0)
+	       and ((now - hmac["seen.last"])  < 60)
+	    and (hmac["location"] == "lan")) then -- Skip silent/wan hosts
+
 	    if(arp_mdns[hmac.mac] == nil) then
+	       -- This is a known host that has *NOT* been observed via ARP scan
 	       local ips = interface.findHostByMac(hmac.mac) or {}
+	       
 	       if(discover.debug) then io.write("Missing MAC "..hmac.mac.."\n") end
 
 	       for k,v in pairs(ips) do
@@ -1045,10 +1204,19 @@ end
 
 -- #############################################################################
 
-function discovery2config(interface_name)
-   local cached = ntop.getCache(discover.getCachedDiscoveryKey(interface_name))
+local function discovery2config(interface_name)
+   local cached = ntop.getCache(getCachedDiscoveryKey(interface_name))
    local disc = json.decode(cached)
 
+   if isEmptyString(cached) or not disc then
+      return
+   end
+
+   disc["devices"] = ntop.lrangeCache(getCachedDiscoveredDevicesKey(interface_name), 0, -1) or {}
+   for i=1, #disc["devices"] do
+      disc["devices"][i] = json.decode(disc["devices"][i])
+   end
+
    if(disc and false) then
       for _,dev in pairs(disc.devices) do
 	 if(dev.device_type.."" ~= "unknown") then
@@ -1057,7 +1225,7 @@ function discovery2config(interface_name)
       end
    end
 
-   return(cached)
+   return disc
 end
 
 -- #############################################################################
@@ -1075,7 +1243,7 @@ function discover.discover2table(interface_name, recache)
 
    if recache ~= true then
       if not isEmptyString(cached) then
-	 return json.decode(cached) or {status = discoverStatus("ERROR", i18n("discover.error_unable_to_decode_json"))}
+	 return cached or {status = discoverStatus("ERROR", i18n("discover.error_unable_to_decode_json"))}
       else
 	 return {status = discoverStatus("NOCACHE", i18n("discover.error_no_discovery_cached"))}
       end
@@ -1089,7 +1257,7 @@ function discover.discover2table(interface_name, recache)
    end
 
    setDiscoveryProgress("[10 %]")
-   
+
    local arp_mdns = arp_d["arp_mdns"] or {}
    local ghost_macs = arp_d["ghost_macs"]
    local ghost_found = arp_d["ghost_found"]
@@ -1107,7 +1275,7 @@ function discover.discover2table(interface_name, recache)
 
 	 -- This is an ARP entry
 	 if(discover.debug) then io.write("Attempting to resolve "..ip.."\n") end
-	 local sym = ntop.getResolvedName(ip) -- dummy resolution just to fill-up the cache
+	 local sym = ntop.getResolvedName(ip) -- placeholder resolution just to fill-up the cache
 
 	 interface.mdnsQueueNameToResolve(ip)
 
@@ -1137,14 +1305,14 @@ function discover.discover2table(interface_name, recache)
    end
 
    setDiscoveryProgress("[30 %]")
-   
+
    if(discover.debug) then io.write("Analyzing SSDP...\n") end
    ssdp = analyzeSSDP(ssdp)
 
    local show_services = false
 
    setDiscoveryProgress("[40 %]")
-   
+
    if(discover.debug) then io.write("Collecting MDNS responses\n") end
    local mdns = interface.mdnsReadQueuedResponses()
 
@@ -1160,7 +1328,7 @@ function discover.discover2table(interface_name, recache)
    end
 
    setDiscoveryProgress("[50 %]")
-   
+
    if(discover.debug) then io.write("Collecting SNMP responses\n") end
    local snmp = {}
 
@@ -1174,7 +1342,7 @@ function discover.discover2table(interface_name, recache)
       interface.snmpGetBatch(ip, snmp_community, "1.3.6.1.2.1.1.1.0", 0)
    end
    setDiscoveryProgress("[60 %]")
-   
+
    if(discover.debug) then io.write("Collecting MDNS OSX responses\n") end
    osx_devices = interface.mdnsReadQueuedResponses()
    if(discover.debug) then io.write("Collected MDNS OSX responses\n") end
@@ -1207,10 +1375,11 @@ function discover.discover2table(interface_name, recache)
    end
 
    setDiscoveryProgress("[80 %]")
-   
+
    -- Time to pack the results in a table...
    local status = discoverStatus("OK")
    local res = {}
+   local too_many_devices_discovered = false
 
    for mac, ip in pairsByValues(arp_mdns, asc) do
       if((string.find(mac, ":") == nil)
@@ -1273,7 +1442,7 @@ function discover.discover2table(interface_name, recache)
 	 entry["os_type"] = mac_info.operatingSystem
       end
 
-      interface.setMacDeviceType(mac, discover.devtype2id(device_type), false) -- false means don't overwrite if already set to ~= unknown
+      ntop.setMacDeviceType(mac, discover.devtype2id(device_type), false) -- false means don't overwrite if already set to ~= unknown
 
       entry["device_type"] = device_type
       entry["device_label"] = device_label
@@ -1289,14 +1458,31 @@ function discover.discover2table(interface_name, recache)
       end
 
       res[#res + 1] = entry
+      if #res >= discover.MAX_DISCOVERED_DEVICES then
+	 too_many_devices_discovered = true
+	 break
+      end
+
       ::continue::
    end
 
    setDiscoveryProgress("")
-   
-   local response = {status = status, devices = res, ghost_found = ghost_found, discovery_timestamp = os.time()}
 
-   ntop.setCache(discover.getCachedDiscoveryKey(interface_name), json.encode(response))
+   -- Cleanup old data
+   ntop.delCache(getCachedDiscoveryKey(interface_name))
+   ntop.delCache(getCachedDiscoveredDevicesKey(interface_name))
+
+   -- Save the discovery status
+   local response = {status = status, ghost_found = ghost_found, discovery_timestamp = os.time(), too_many_devices_discovered = too_many_devices_discovered}
+   ntop.setCache(getCachedDiscoveryKey(interface_name), json.encode(response))
+
+   -- Save the actually discovered devices
+   local records_cache = getCachedDiscoveredDevicesKey(interface_name)
+
+   -- res is already trimmed at MAX_DISCOVERED_DEVICES, no need to check it
+   for i=1, #res do
+      ntop.lpushCache(records_cache, json.encode(res[i]))
+   end
 
    return response
 end
