@@ -1,5 +1,5 @@
 --
--- (C) 2013-17 - ntop.org
+-- (C) 2013-23 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -7,20 +7,23 @@ package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 
 require "lua_utils"
 
+local page_utils = require("page_utils")
+
 sendHTTPContentTypeHeader('text/html')
 
-ntop.dumpFile(dirs.installdir .. "/httpdocs/inc/header.inc")
 
-active_page = "hosts"
+page_utils.set_active_menu_entry(page_utils.menu_entries.operating_systems)
+
 dofile(dirs.installdir .. "/scripts/lua/inc/menu.lua")
 
+page_utils.print_page_title(i18n("os_stats.hosts_by_operating_system"))
+
 print [[
-      <hr>
-      <div id="table-os"></div>
+	  <div id="table-os"></div>
 	 <script>
 	 var url_update = "]]
 print (ntop.getHttpPrefix())
-print [[/lua/get_grouped_hosts_data.lua?grouped_by=os]]
+print [[/lua/get_oses_data.lua]]
 
 print ('";')
 ntop.dumpFile(dirs.installdir .. "/httpdocs/inc/os_stats_id.inc")
@@ -31,7 +34,7 @@ print [[
 			url: url_update ,
 	 ]]
 
-print('title: "' .. i18n("os_stats.hosts_by_operating_system") .. '",\n')
+print('title: "",\n')
 print ('rowCallback: function ( row ) { return os_table_setID(row); },')
 
 -- Set the preference table
@@ -61,10 +64,22 @@ print [[
 			        textAlign: 'left'
 			     }
 
-				 },
-			  ]]
+				 }, {
+					title: "]] print(i18n("chart")) print[[",
+					field: "column_chart",
+					sortable: false,]]
 
-print [[
+local charts_enabled = areOSTimeseriesEnabled(interface.getId())
+
+if not charts_enabled then
+   print("hidden: true,\n")
+end
+
+print[[
+						css: {
+				textAlign: 'center'
+			     }
+				 },
 			     {
 			     title: "]] print(i18n("hosts_stats.hosts")) print[[",
 				 field: "column_hosts",
@@ -77,13 +92,13 @@ print [[
 			     {
 			     title: "]] print(i18n("show_alerts.alerts")) print[[",
 				 field: "column_alerts",
-				 sortable: true,
+				 sortable: false,
                              css: {
 			        textAlign: 'center'
 			     }
 
 				 },
-		
+
 			     {
 			     title: "]] print(i18n("seen_since")) print[[",
 				 field: "column_since",
@@ -93,10 +108,6 @@ print [[
 			     }
 
 				 },
-
-]]
-
-print [[
 			     {
 			     title: "]] print(i18n("breakdown")) print[[",
 				 field: "column_breakdown",
