@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,11 +31,12 @@ AutonomousSystemHash::AutonomousSystemHash(NetworkInterface *_iface, u_int _num_
 
 /* ************************************ */
 
-AutonomousSystem* AutonomousSystemHash::get(IpAddress *ipa) {
-  u_int32_t asn;
-  ntop->getGeolocation()->getAS(ipa, &asn, NULL /* Don't care about AS name here */);
-  u_int32_t hash = asn;
+AutonomousSystem* AutonomousSystemHash::get(IpAddress *ipa, bool is_inline_call) {
+  u_int32_t asn, hash;
 
+  ntop->getGeolocation()->getAS(ipa, &asn, NULL /* Don't care about AS name here */);
+  hash = asn;
+  
   hash %= num_hashes;
 
   if(table[hash] == NULL) {
@@ -43,18 +44,21 @@ AutonomousSystem* AutonomousSystemHash::get(IpAddress *ipa) {
   } else {
     AutonomousSystem *head;
 
-    locks[hash]->lock(__FILE__, __LINE__);
+    if(!is_inline_call)
+      locks[hash]->rdlock(__FILE__, __LINE__);
+
     head = (AutonomousSystem*)table[hash];
 
     while(head != NULL) {
-      if((!head->idle()) && head->equal(asn))
+      if(!head->idle() && head->equal(asn))
 	break;
       else
 	head = (AutonomousSystem*)head->next();
     }
-    
-    locks[hash]->unlock(__FILE__, __LINE__);
-    
+
+    if(!is_inline_call)
+      locks[hash]->unlock(__FILE__, __LINE__);
+
     return(head);
   }
 }
