@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,7 @@
 
 /* **************************************************** */
 
-DummyInterface::DummyInterface() : ParserInterface("dummy") {
+DummyInterface::DummyInterface() : ZMQParserInterface("dummy") {
   ntop->getTrace()->traceEvent(TRACE_NORMAL, "Initialized dummy interface");
 }
 
@@ -69,7 +69,7 @@ void DummyInterface::forgeFlow(u_int iteration) {
 	     iteration, iteration*1500,
 	     now-120, now-60, sport, dport);
 
-    parseFlow(payload, sizeof(payload), 1 /* source_id */, this /* iface */);
+    parseJSONFlow(payload, sizeof(payload), 1 /* source_id */);
   }
 
   if(id == 0) sleep(ntop->getPrefs()->get_housekeeping_frequency());
@@ -82,11 +82,13 @@ static void* packetPollLoop(void* ptr) {
   u_int32_t iteration = 0;
 
   /* Wait until the initialization completes */
-  while(!iface->isRunning()) sleep(1);
+  while(iface->isStartingUp()) sleep(1);
 
   while(iface->isRunning()) {
     iface->forgeFlow(++iteration);
     _usleep(10);
+
+    iface->purgeIdle(time(NULL));
   }
   
   ntop->getTrace()->traceEvent(TRACE_NORMAL, "Flow collection is over.");
