@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -34,23 +34,48 @@ static u_int16_t max_num_loops = 0;
 
 Flow* FlowHash::find(IpAddress *src_ip, IpAddress *dst_ip,
 		     u_int16_t src_port, u_int16_t dst_port, 
-		     u_int16_t vlanId, u_int8_t protocol,
-		     bool *src2dst_direction) {
-  // ntop->getTrace()->traceEvent(TRACE_NORMAL, "%u:%u / %u:%u", src_ip->key(), src_port, dst_ip->key(), dst_port);
-
-  /* Removed vlanId due to eBPF */
-  u_int32_t hash = ((src_ip->key()+dst_ip->key()+src_port+dst_port/* +vlanId */+protocol) % num_hashes);
+		     VLANid vlanId, u_int16_t observation_point_id,
+		     u_int32_t private_flow_id, u_int8_t protocol,
+		     const ICMPinfo * const icmp_info,
+		     bool *src2dst_direction,
+		     bool is_inline_call,
+		     Flow **unswapped_flow) {
+  u_int32_t hash = ((src_ip->key() + dst_ip->key()
+		     + (icmp_info ? icmp_info->key() : 0)
+		     + private_flow_id
+		     + src_port + dst_port + vlanId + protocol) % num_hashes);
   Flow *head = (Flow*)table[hash];
   u_int16_t num_loops = 0;
 
+  *unswapped_flow = NULL;
+  
+  // ntop->getTrace()->traceEvent(TRACE_NORMAL, "%u:%u / %u:%u [icmp: %u][key: %u][icmp info key: %u][head: 0x%x]", src_ip->key(), src_port, dst_ip->key(), dst_port, icmp_info ? 1 : 0, hash, icmp_info ? icmp_info->key() : 0, head);
+
+  if(!head)
+    return(NULL);
+
+  if(!is_inline_call)
+    locks[hash]->rdlock(__FILE__, __LINE__);
+
   while(head) {
-    if((!head->idle() && !head->is_ready_to_be_purged())
-       && head->equal(src_ip, dst_ip, src_port, dst_port, vlanId, protocol, src2dst_direction)) {
+    if(!head->idle()
+       && head->equal(src_ip, dst_ip, src_port, dst_port, vlanId, observation_point_id, private_flow_id, protocol, icmp_info, src2dst_direction)) {
       if(num_loops > max_num_loops) {
 	ntop->getTrace()->traceEvent(TRACE_INFO, "DEBUG: [Num loops: %u][hashId: %u]", num_loops, hash);
 	max_num_loops = num_loops;
       }
-      return(head);
+
+      if(head->is_swap_requested()) {
+	/*
+	  This is a good flow but in the wrong direction
+	  hence we remember the flow unswapped but merge
+	  data with the flow with the correct direction
+ 	 */
+	*unswapped_flow = head;	
+	head = NULL;
+      }
+      
+      break;
     } else
       head = (Flow*)head->next(), num_loops++;
   }
@@ -60,5 +85,31 @@ Flow* FlowHash::find(IpAddress *src_ip, IpAddress *dst_ip,
     max_num_loops = num_loops;
   }
 
-  return(NULL);
+  if(!is_inline_call)
+    locks[hash]->unlock(__FILE__, __LINE__);
+
+  return(head);
 }
+
+/* ************************************ */
+
+Flow* FlowHash::findByKeyAndHashId(u_int32_t key, u_int32_t hash_id) {
+  u_int32_t hash = key % num_hashes;
+  Flow *head     = (Flow*)table[hash];
+
+  if(head == NULL) return(NULL);
+
+  locks[hash]->rdlock(__FILE__, __LINE__);
+
+  while(head) {
+    if((!head->idle()) && (head->get_hash_entry_id() == hash_id))
+      break;
+    else
+      head = (Flow*)head->next();
+  }
+
+  locks[hash]->unlock(__FILE__, __LINE__);
+
+  return((Flow*)head);
+}
+
