@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,10 +30,11 @@ struct http_walk_info {
 
 /* *************************************** */
 
-HTTPstats::HTTPstats(HostHash *_h) {
+HTTPstats::HTTPstats(Host *_host) {
   struct timeval tv;
 
-  h = _h, warning_shown = false;
+  host = _host;
+  h = host->getInterface()->get_hosts_hash(), warning_shown = false;
   memset(&query, 0, sizeof(query));
   memset(&response, 0, sizeof(response));
   memset(&query_rate, 0, sizeof(query_rate));
@@ -43,7 +44,9 @@ HTTPstats::HTTPstats(HostHash *_h) {
 
   gettimeofday(&tv, NULL);
   memcpy(&last_update_time, &tv, sizeof(struct timeval));
-  if((virtualHosts = new (std::nothrow) VirtualHostHash(NULL, 1, 4096)) == NULL) {
+  if((virtualHosts = new (std::nothrow) VirtualHostHash(host->getInterface(),
+							8   /* Buckets */,
+							64  /* Maximum number of virtual hosts */)) == NULL) {
     ntop->getTrace()->traceEvent(TRACE_WARNING, "Internal error: are you running out of memory?");
   }
 }
@@ -194,6 +197,47 @@ void HTTPstats::getResponsesDelta(const struct http_response_stats *r0, const st
 
 /* *************************************** */
 
+u_int32_t HTTPstats::getSentNumQueries() {
+   u_int32_t num_get = 0, num_post = 0, num_head = 0, num_put = 0, num_other = 0;
+
+  getRequests(&query[AS_SENDER], &num_get, &num_post, &num_head, &num_put, &num_other);
+  
+  return num_get+num_post+num_head+num_put+num_other;
+ 
+}
+
+/* *************************************** */
+
+u_int32_t HTTPstats::getSentNumResponses() {
+  u_int32_t num_1xx = 0, num_2xx = 0, num_3xx = 0, num_4xx = 0, num_5xx =0;
+
+  getResponses(&response[AS_SENDER], &num_1xx, &num_2xx, &num_3xx, &num_4xx, &num_5xx);
+  
+  return num_1xx+num_2xx+num_3xx+num_4xx+num_5xx;
+}
+
+/* *************************************** */
+
+u_int32_t HTTPstats::getRcvdNumQueries() {
+  u_int32_t num_get = 0, num_post = 0, num_head = 0, num_put = 0, num_other = 0;
+
+  getRequests(&query[AS_RECEIVER], &num_get, &num_post, &num_head, &num_put, &num_other);
+  
+  return num_get+num_post+num_head+num_put+num_other;
+}
+
+/* *************************************** */
+
+u_int32_t HTTPstats::getRcvdNumResponses() {
+  u_int32_t num_1xx = 0, num_2xx = 0, num_3xx = 0, num_4xx = 0, num_5xx =0;
+
+  getResponses(&response[AS_RECEIVER], &num_1xx, &num_2xx, &num_3xx, &num_4xx, &num_5xx);
+  
+  return num_1xx+num_2xx+num_3xx+num_4xx+num_5xx;
+}
+
+/* *************************************** */
+
 void HTTPstats::luaAddCounters(lua_State *vm, bool as_sender) {
   u_int32_t num_get = 0, num_post = 0, num_head = 0, num_put = 0, num_other = 0;
   u_int32_t num_1xx = 0, num_2xx = 0, num_3xx = 0, num_4xx = 0, num_5xx =0;
@@ -455,37 +499,27 @@ json_object* HTTPstats::getJSONObject() {
 
 /* ******************************************* */
 
-void HTTPstats::incRequest(struct http_query_stats *q, const char *method) {
-  if(method[0] == 'G') q->num_get++;
-  else if((method[0] == 'P') && (method[1] == 'O')) q->num_post++;
-  else if(method[0] == 'H') q->num_head++;
-  else if((method[0] == 'P') && (method[1] == 'U')) q->num_put++;
-  else q->num_other++;
+void HTTPstats::incStats(bool as_client, const FlowHTTPStats *fts) {
+  struct http_query_stats *q = as_client ? &query[AS_SENDER] : &query[AS_RECEIVER];
+  struct http_response_stats *r = as_client ? &response[AS_RECEIVER] : &response[AS_SENDER];
+
+  if(fts->num_get)   q->num_get   += fts->num_get;
+  if(fts->num_post)  q->num_post  += fts->num_post;
+  if(fts->num_head)  q->num_head  += fts->num_head;
+  if(fts->num_put)   q->num_put   += fts->num_put;
+  if(fts->num_other) q->num_other += fts->num_other;
+
+  if(fts->num_1xx)   r->num_1xx += fts->num_1xx;
+  if(fts->num_2xx)   r->num_2xx += fts->num_2xx;
+  if(fts->num_3xx)   r->num_3xx += fts->num_3xx;
+  if(fts->num_4xx)   r->num_4xx += fts->num_4xx;
+  if(fts->num_5xx)   r->num_5xx += fts->num_5xx;
 }
 
 /* ******************************************* */
 
-void HTTPstats::incResponse(struct http_response_stats *r, const char *return_code) {
-  const char *code;
-
-  if(!return_code)
-    return;
-
-  code = strchr(return_code, ' ');
-  if(!code) return;
-
-  switch(code[1]) {
-  case '1': r->num_1xx++; break;
-  case '2': r->num_2xx++; break;
-  case '3': r->num_3xx++; break;
-  case '4': r->num_4xx++; break;
-  case '5': r->num_5xx++; break;
-  }
-}
-
-/* ******************************************* */
-
-bool HTTPstats::updateHTTPHostRequest(char *virtual_host_name, u_int32_t num_requests,
+bool HTTPstats::updateHTTPHostRequest(time_t t,
+				      char *virtual_host_name, u_int32_t num_requests,
 				      u_int32_t bytes_sent, u_int32_t bytes_rcvd) {
   VirtualHost *vh;
   bool rc = false;
@@ -503,32 +537,23 @@ bool HTTPstats::updateHTTPHostRequest(char *virtual_host_name, u_int32_t num_req
     Flow::~Flow() on the same instance and thus
     create a memory leak
   */
-  m.lock(__FILE__, __LINE__);
   if((vh = virtualHosts->get(virtual_host_name)) == NULL) {
     if(virtualHosts->hasEmptyRoom()) {
-      if((vh = new VirtualHost(h, virtual_host_name)) == NULL) {
+      if((vh = new (std::nothrow) VirtualHost(h, virtual_host_name)) == NULL) {
 	ntop->getTrace()->traceEvent(TRACE_WARNING, "Internal error: are you running out of memory?");
       } else {
-	if(virtualHosts->add(vh) == false) {
+	if(virtualHosts->add(vh, false /* no need to lock the bucket, purgeIdle is sequential wrt to this */) == false) {
 	  /* Unable to add a new virtual host */
 	  delete vh;
 	  vh = NULL;
 	} else
 	  rc = true;
       }
-    } else {
-      if(!warning_shown) {
-	ntop->getTrace()->traceEvent(TRACE_WARNING,
-				     "Too many virtual hosts %u: enlarge hash",
-				     virtualHosts->getNumEntries());
-	warning_shown = true;
-      }
     }
   }
-  m.unlock(__FILE__, __LINE__);
 
   if(vh)
-    vh->incStats(num_requests, bytes_sent, bytes_rcvd);
+    vh->incStats(t, num_requests, bytes_sent, bytes_rcvd);
 
   return(rc);
 }
@@ -546,20 +571,23 @@ static bool update_http_stats(GenericHashEntry *node, void *user_data, bool *mat
 
 /* ******************************************* */
 
-void HTTPstats::updateStats(struct timeval *tv) {
-  float tdiff_msec = Utils::timeval2ms(tv) - Utils::timeval2ms(&last_update_time);
+void HTTPstats::updateStats(const struct timeval *tv) {
+  float tdiff_msec = Utils::msTimevalDiff(tv, &last_update_time);
   const u_int8_t indices[2] = { AS_SENDER, AS_RECEIVER };
-  
-  if(tdiff_msec < 1000) return;  // too early
 
   // refresh the last update time with the new values
   // also refresh the statistics on request variations
   if(virtualHosts) {
     u_int32_t begin_slot = 0;
     bool walk_all = true;
-    
-    virtualHosts->walk(&begin_slot, walk_all, update_http_stats, tv);
-    virtualHosts->purgeIdle();
+
+    virtualHosts->walk(&begin_slot, walk_all, update_http_stats, (void*)tv);
+    virtualHosts->purgeIdle(tv, false, false);
+    virtualHosts->purgeQueuedIdleEntries(); /* Actually reclaim memory (delete*) of idle virtual hosts */
+  }
+
+  if(tdiff_msec < 1000) {
+    return;  // too early
   }
 
   for(u_int8_t i = 0; i < 2 ; i++) {
@@ -576,14 +604,14 @@ void HTTPstats::updateStats(struct timeval *tv) {
       dq -> rate_post  = makeRate(d_post, tdiff_msec),
       dq -> rate_head  = makeRate(d_head, tdiff_msec),
       dq -> rate_put   = makeRate(d_put, tdiff_msec),
-      dq -> rate_other = makeRate(d_other, tdiff_msec),      
+      dq -> rate_other = makeRate(d_other, tdiff_msec),
       dr -> rate_1xx   = makeRate(d_1xx, tdiff_msec),
       dr -> rate_2xx   = makeRate(d_2xx, tdiff_msec),
       dr -> rate_3xx   = makeRate(d_3xx, tdiff_msec),
       dr -> rate_4xx   = makeRate(d_4xx, tdiff_msec),
       dr -> rate_5xx   = makeRate(d_5xx, tdiff_msec);
   }
-  
+
   last_update_time.tv_sec  = tv->tv_sec, last_update_time.tv_usec = tv->tv_usec;
   memcpy(&last_query_sample,    &query,    sizeof(query));
   memcpy(&last_response_sample, &response, sizeof(response));
