@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,7 +30,7 @@ HostHash::HostHash(NetworkInterface *_iface, u_int _num_hashes, u_int _max_hash_
 
 /* ************************************ */
 
-Host* HostHash::get(u_int16_t vlanId, IpAddress *key) {
+Host* HostHash::get(VLANid vlanId, IpAddress *key, bool is_inline_call, u_int16_t observation_point_id) {
   u_int32_t hash = (key->key() % num_hashes);
 
   if(table[hash] == NULL) {
@@ -38,20 +38,24 @@ Host* HostHash::get(u_int16_t vlanId, IpAddress *key) {
   } else {
     Host *head;
 
-    locks[hash]->lock(__FILE__, __LINE__);
+    if(!is_inline_call)
+      locks[hash]->rdlock(__FILE__, __LINE__);
+
     head = (Host*)table[hash];
     
-    while(head != NULL) {      
+    while(head != NULL) { 
       if((!head->idle())
-	 && (!head->is_ready_to_be_purged())
 	 && (head->get_vlan_id() == vlanId)
+	 && (head->get_observation_point_id() == observation_point_id)
 	 && (head->get_ip() != NULL)
 	 && (head->get_ip()->compare(key) == 0))
 	break;
       else
 	head = (Host*)head->next();
     }
-    locks[hash]->unlock(__FILE__, __LINE__);
+
+    if(!is_inline_call)
+      locks[hash]->unlock(__FILE__, __LINE__);
 
     return(head);
   }
