@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2015-18 - ntop.org
+ * (C) 2015-22 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,7 @@
 
 #include "ntop_includes.h"
 
-/* #define HOST_POOLS_DEBUG 1 */
+// #define HOST_POOLS_DEBUG 1
 
 /* *************************************** */
 
@@ -30,9 +30,7 @@ HostPools::HostPools(NetworkInterface *_iface) {
 #ifdef NTOPNG_PRO
   children_safe = forge_global_dns = NULL;
   routing_policy_id = NULL;
-#endif
 
-#ifdef NTOPNG_PRO
   if((children_safe = (bool*)calloc(MAX_NUM_HOST_POOLS, sizeof(bool))) == NULL)
     throw 1;
 
@@ -44,40 +42,27 @@ HostPools::HostPools(NetworkInterface *_iface) {
 
   for(int i = 0; i < MAX_NUM_HOST_POOLS; i++) routing_policy_id[i] = DEFAULT_ROUTING_TABLE_ID;
 
-  stats = stats_shadow = NULL;
-
-  if((volatile_members = (volatile_members_t**)calloc(MAX_NUM_HOST_POOLS, sizeof(volatile_members_t*))) == NULL
-     || (volatile_members_lock            = new Mutex*[MAX_NUM_HOST_POOLS]) == NULL
-     || (pool_shaper = (u_int16_t*)calloc(MAX_NUM_HOST_POOLS, sizeof(u_int16_t))) == NULL
+  if((pool_shaper = (u_int16_t*)calloc(MAX_NUM_HOST_POOLS, sizeof(u_int16_t))) == NULL
      || (schedule_bitmap = (u_int32_t*)calloc(MAX_NUM_HOST_POOLS, sizeof(u_int32_t))) == NULL
      || (enforce_quotas_per_pool_member   = (bool*)calloc(MAX_NUM_HOST_POOLS, sizeof(bool))) == NULL
      || (enforce_shapers_per_pool_member  = (bool*)calloc(MAX_NUM_HOST_POOLS, sizeof(bool))) == NULL)
     throw 1;
-
-  for(int i = 0; i < MAX_NUM_HOST_POOLS; i++) {
-    if((volatile_members_lock[i] = new Mutex()) == NULL)
-      throw 2;
-  }
 #endif
 
+  stats = stats_shadow = NULL;
+
   if((num_active_hosts_inline          = (int32_t*)calloc(sizeof(int32_t), MAX_NUM_HOST_POOLS)) == NULL
      || (num_active_hosts_offline      = (int32_t*)calloc(sizeof(int32_t), MAX_NUM_HOST_POOLS)) == NULL
      || (num_active_l2_devices_inline  = (int32_t*)calloc(sizeof(int32_t), MAX_NUM_HOST_POOLS)) == NULL
      || (num_active_l2_devices_offline = (int32_t*)calloc(sizeof(int32_t), MAX_NUM_HOST_POOLS)) == NULL)
     throw 1;
 
-  latest_swap = 0;
-  if((swap_lock = new Mutex()) == NULL)
-    throw 3;
-
   if(_iface)
     iface = _iface;
 
   reloadPools();
 
-#ifdef NTOPNG_PRO
   loadFromRedis();
-#endif
 
   max_num_pools = MAX_NUM_HOST_POOLS;
 
@@ -86,8 +71,6 @@ HostPools::HostPools(NetworkInterface *_iface) {
 
 /* *************************************** */
 
-#ifdef NTOPNG_PRO
-
 void HostPools::deleteStats(HostPoolStats ***hps) {
   if(hps) {
     if(*hps) {
@@ -100,8 +83,6 @@ void HostPools::deleteStats(HostPoolStats ***hps) {
   }
 }
 
-#endif
-
 /* *************************************** */
 
 HostPools::~HostPools() {
@@ -116,15 +97,15 @@ HostPools::~HostPools() {
 
   if(tree_shadow)   delete tree_shadow;
   if(tree)          delete tree;
-  if(swap_lock)     delete swap_lock;
+
+  dumpToRedis();
+  if(stats)        deleteStats(&stats);
+  if(stats_shadow) deleteStats(&stats_shadow);
 
 #ifdef NTOPNG_PRO
   if(children_safe)     free(children_safe);
   if(forge_global_dns)  free(forge_global_dns);
   if(routing_policy_id) free(routing_policy_id);
-
-  dumpToRedis();
-
   if(pool_shaper)
     free(pool_shaper);
   if(schedule_bitmap)
@@ -134,54 +115,12 @@ HostPools::~HostPools() {
   if(enforce_shapers_per_pool_member)
     free(enforce_shapers_per_pool_member);
 
-  if(stats)        deleteStats(&stats);
-  if(stats_shadow) deleteStats(&stats_shadow);
-
-  if(volatile_members_lock) {
-    for(int i = 0; i < MAX_NUM_HOST_POOLS; i++) {
-      if(volatile_members_lock[i])
-	delete volatile_members_lock[i];
-    }
-
-    delete []volatile_members_lock;
-  }
-
-  if(volatile_members) {
-    for(int pool_id = 0; pool_id < MAX_NUM_HOST_POOLS; pool_id++) {
-      volatile_members_t *current, *tmp;
-
-      HASH_ITER(hh, volatile_members[pool_id], current, tmp) {
-	HASH_DEL(volatile_members[pool_id], current);
-	free(current->host_or_mac);
-	free(current);
-      }
-
-      if(volatile_members[pool_id])
-	free(volatile_members[pool_id]);
-    }
-
-    free(volatile_members);
-  }
-
 #endif
 }
 
 /* *************************************** */
 
-#ifdef NTOPNG_PRO
-void HostPools::swap(VlanAddressTree *new_trees, HostPoolStats **new_stats) {
-#else
-void HostPools::swap(VlanAddressTree *new_trees) {
-#endif
-  swap_lock->lock(__FILE__, __LINE__);
-
-  while(time(NULL) - latest_swap < 1) {
-    swap_lock->unlock(__FILE__, __LINE__);
-    sleep(1); /* Force at least 1 sec. time between consecutive swaps */
-    swap_lock->lock(__FILE__, __LINE__);
-  }
-
-#ifdef NTOPNG_PRO
+void HostPools::swap(VLANAddressTree *new_trees, HostPoolStats **new_stats) {
   /* Swap statistics */
   if(new_stats) {
     if(stats) {
@@ -191,7 +130,6 @@ void HostPools::swap(VlanAddressTree *new_trees) {
 
     stats = new_stats;
   }
-#endif
 
   /* Swap address trees */
   if(new_trees) {
@@ -202,99 +140,6 @@ void HostPools::swap(VlanAddressTree *new_trees) {
 
     tree = new_trees;
   }
-
-  latest_swap = time(NULL);
-  swap_lock->unlock(__FILE__, __LINE__);
-}
-
-/* *************************************** */
-
-#ifdef NTOPNG_PRO
-
-void HostPools::reloadVolatileMembers(VlanAddressTree *_trees) {
-  volatile_members_t *current, *tmp;
-  char *at, *member;
-  bool rc;
-  u_int16_t vlan_id;
-
-  if(!_trees)
-    return;
-
-  for(int pool_id = 0; pool_id < MAX_NUM_HOST_POOLS; pool_id++) {
-
-    if(!volatile_members[pool_id])
-      continue;
-
-    volatile_members_lock[pool_id]->lock(__FILE__, __LINE__);
-
-    if(stats && stats[pool_id]) { /* The pool exists */
-      HASH_ITER(hh, volatile_members[pool_id], current, tmp) {
-	member = strdup(current->host_or_mac);
-
-	if((at = strchr(member, '@'))) {
-	  vlan_id = atoi(at + 1);
-	  *at = '\0';
-	} else
-	  vlan_id = 0;
-
-	if(!(rc = _trees->addAddress(vlan_id, member, pool_id))
-#ifdef HOST_POOLS_DEBUG
-	    || true
-#endif
-	     )
-	  ntop->getTrace()->traceEvent(TRACE_NORMAL,
-				       "%s VOLATILE tree node for %s [vlan %i] [host pool: %i]",
-				       rc ? "Successfully added" : "Unable to add",
-				       member, vlan_id,
-				       pool_id);
-
-
-	free(member);
-      }
-    } else { /* The pool no longer exists */
-      HASH_ITER(hh, volatile_members[pool_id], current, tmp) {
-	HASH_DEL(volatile_members[pool_id], current);
-	free(current->host_or_mac);
-	free(current);
-      }
-    }
-
-    volatile_members_lock[pool_id]->unlock(__FILE__, __LINE__);
-  }
-
-#ifdef HAVE_NEDGE
-  /* Note: we must re-evaluate the active flows as a captive portal host may be blocked now */
-  if(iface && (iface->getIfType() == interface_type_NETFILTER))
-    ((NetfilterInterface *) iface)->setPolicyChanged();
-#endif
-};
-
-/* *************************************** */
-
-void HostPools::addVolatileMember(char *host_or_mac, u_int16_t host_pool_id, time_t lifetime) {
-  volatile_members_t *m;
-
-  if(!host_or_mac || host_pool_id >= MAX_NUM_HOST_POOLS)
-    return;
-
-  volatile_members_lock[host_pool_id]->lock(__FILE__, __LINE__);
-
-  HASH_FIND_STR(volatile_members[host_pool_id], host_or_mac, m);
-
-  if(m == NULL) {
-    m = (volatile_members_t*)calloc(1, sizeof(volatile_members_t));
-    m->host_or_mac = strdup(host_or_mac);
-    HASH_ADD_STR(volatile_members[host_pool_id], host_or_mac, m);
-  }
-  m->lifetime = time(NULL) + lifetime;
-
-#ifdef HOST_POOLS_DEBUG
-  ntop->getTrace()->traceEvent(TRACE_NORMAL,
-    "Adding %s VOLATILE MEMBER to the hash table [host pool: %i] [lifetime: %i]",
-    host_or_mac, host_pool_id, lifetime);
-#endif
-
-  volatile_members_lock[host_pool_id]->unlock(__FILE__, __LINE__);
 }
 
 /* *************************************** */
@@ -319,33 +164,20 @@ void HostPools::dumpToRedis() {
       }
     }
   }
-
-  // Save the deadline time for quota expiration, assuming quota is reset at midnight
-  snprintf(buf, sizeof(buf), "%u",
-	   Utils::roundTime(time(0), 86400, ntop->get_time_offset()) - 86400);
-  redis->hashSet(key, (char *)"deadline", buf);
 }
 
 /* *************************************** */
 
 void HostPools::loadFromRedis() {
-  char key[128], buf[32], *value;
+  char key[CONST_MAX_LEN_REDIS_KEY], buf[32], *value;
   json_object *obj;
   enum json_tokener_error jerr = json_tokener_success;
   Redis *redis = ntop->getRedis();
-  time_t deadline = 0;
 
   snprintf(key, sizeof(key), HOST_POOL_SERIALIZED_KEY, iface->get_id());
 
   if((!redis) || (!stats) || (!iface)) return;
 
-  if(redis->hashGet(key, (char *)"deadline", buf, sizeof(buf)) == 0) {
-    sscanf(buf, "%lu", &deadline);
-
-    if(time(0) > deadline)
-      return; /* Expired */
-  }
-
   if((value = (char *) malloc(POOL_MAX_SERIALIZED_LEN)) == NULL) {
     ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to allocate memory to deserialize %s", key);
     return;
@@ -372,13 +204,44 @@ void HostPools::loadFromRedis() {
 
 /* *************************************** */
 
-void HostPools::incPoolNumDroppedFlows(u_int16_t pool_id) {
-  HostPoolStats *hps = getPoolStats(pool_id);
+void HostPools::luaStats(lua_State *vm) {
+  if(vm) {
+    lua_newtable(vm);
 
-  if(!hps) return;
+    for(int i = 0; i < MAX_NUM_HOST_POOLS; i++)
+      luaStats(vm, i);
+  }
+};
 
-  hps->incNumDroppedFlows();
-}
+/* *************************************** */
+
+void HostPools::luaStats(lua_State *vm, u_int16_t pool_id) {
+  HostPoolStats *hps;
+
+  if(vm) {
+    if(stats && pool_id < MAX_NUM_HOST_POOLS) {
+      if((hps = stats[pool_id])) {
+	/* Must use the assigned hps as stats can be swapped
+	   and accesses such as stats[pool_id] could yield a NULL value */
+	hps->lua(vm, iface);
+	lua_push_uint64_table_entry(vm, "num_hosts", getNumPoolHosts(pool_id));
+	lua_rawseti(vm, -2, pool_id);
+      }
+    }
+  }
+};
+
+/* *************************************** */
+
+void HostPools::updateStats(const struct timeval *tv) {
+  HostPoolStats *hps;
+
+  if(stats && tv) {
+    for(int i = 0; i < MAX_NUM_HOST_POOLS; i++)
+      if((hps = stats[i]))
+	hps->updateStats(tv); /* Use hps, stats[i] can become NULL after a swap */
+  }
+};
 
 /* *************************************** */
 
@@ -396,36 +259,15 @@ void HostPools::incPoolStats(u_int32_t when, u_int16_t host_pool_id, u_int16_t n
 
 /* *************************************** */
 
-void HostPools::updateStats(struct timeval *tv) {
-  HostPoolStats *hps;
+#ifdef NTOPNG_PRO
 
-  if(stats && tv) {
-    for(int i = 0; i < MAX_NUM_HOST_POOLS; i++)
-      if((hps = stats[i]))
-	hps->updateStats(tv); /* Use hps, stats[i] can become NULL after a swap */
-  }
-};
+void HostPools::incPoolNumDroppedFlows(u_int16_t pool_id) {
+  HostPoolStats *hps = getPoolStats(pool_id);
 
-/* *************************************** */
+  if(!hps) return;
 
-void HostPools::luaStats(lua_State *vm) {
-  HostPoolStats *hps;
-
-  if(vm) {
-    lua_newtable(vm);
-
-    if(stats) {
-      for(int i = 0; i < MAX_NUM_HOST_POOLS; i++) {
-	if((hps = stats[i])) {
-	  /* Must use the assigned hps as stats can be swapped
-	     and accesses such as stats[i] could yield a NULL value */
-	  hps->lua(vm, iface);
-	  lua_rawseti(vm, -2, i);
-	}
-      }
-    }
-  }
-};
+  hps->incNumDroppedFlows();
+}
 
 /* *************************************** */
 
@@ -464,139 +306,6 @@ void HostPools::checkPoolsStatsReset() {
   }
 }
 
-/* *************************************** */
-
-void HostPools::luaVolatileMembers(lua_State *vm) {
-  volatile_members_t *current, *tmp;
-  int i;
-  time_t now = time(NULL);
-
-  if(!vm)
-    return;
-
-  lua_newtable(vm);
-
-  for(int pool_id = 0; pool_id < MAX_NUM_HOST_POOLS; pool_id++) {
-
-    if(!volatile_members[pool_id])
-      continue;
-
-    volatile_members_lock[pool_id]->lock(__FILE__, __LINE__);
-
-    if(stats && stats[pool_id]) { /* The pool exists */
-      lua_newtable(vm);
-
-      i = 0;
-      HASH_ITER(hh, volatile_members[pool_id], current, tmp) {
-	lua_newtable(vm);
-
-	lua_push_str_table_entry(vm, "member", current->host_or_mac);
-	lua_push_float_table_entry(vm, "residual_lifetime", current->lifetime - now);
-	lua_push_bool_table_entry(vm, "expired", current->lifetime - now < 0);
-
-	lua_rawseti(vm, -2, ++i);
-      }
-
-      lua_rawseti(vm, -2, pool_id);
-    }
-
-    volatile_members_lock[pool_id]->unlock(__FILE__, __LINE__);
-  }
-};
-
-/* *************************************** */
-
-void HostPools::addToPool(char *host_or_mac,
-			  u_int16_t user_pool_id,
-			  int32_t lifetime_secs) {
-  char key[128], pool_buf[16];
-
-#ifdef HOST_POOLS_DEBUG
-  ntop->getTrace()->traceEvent(TRACE_NORMAL,
-			       "Adding %s as %s host pool member [pool id: %i]",
-			       host_or_mac,
-			       lifetime_secs <= 0 ? "PERMANENT" : "VOLATILE",
-			       user_pool_id);
-#endif
-
-  if(lifetime_secs > 0)
-    addVolatileMember(host_or_mac, user_pool_id, (u_int32_t)lifetime_secs);
-
-  else {
-    snprintf(pool_buf, sizeof(pool_buf), "%u", user_pool_id);
-    snprintf(key, sizeof(key), HOST_POOL_MEMBERS_KEY, iface->get_id(), pool_buf);
-    ntop->getRedis()->sadd(key, host_or_mac); /* New member added */
-  }
-
-  reloadPools();
-}
-
-/* *************************************** */
-
-void HostPools::purgeExpiredVolatileMembers() {
-  volatile_members_t *current, *tmp;
-  bool purged = false;
-  time_t now = time(NULL);
-
-  for(int pool_id = 0; pool_id < MAX_NUM_HOST_POOLS; pool_id++) {
-    volatile_members_lock[pool_id]->lock(__FILE__, __LINE__);
-
-    HASH_ITER(hh, volatile_members[pool_id], current, tmp) {
-#ifdef HOST_POOLS_DEBUG
-      ntop->getTrace()->traceEvent(TRACE_NORMAL,
-				   "Checking VOLATILE MEMBER %s [pool id: %i] for expiration...",
-				   current->host_or_mac,
-				   pool_id);
-#endif
-
-      if(current->lifetime < now) {
-	purged = true;
-
-#ifdef HOST_POOLS_DEBUG
-	ntop->getTrace()->traceEvent(TRACE_NORMAL,
-				     "Purging expired VOLATILE MEMBER %s [pool id: %i]",
-				     current->host_or_mac,
-				     pool_id);
-#endif
-
-	HASH_DEL(volatile_members[pool_id], current);
-	free(current->host_or_mac);
-	free(current);
-      }
-    }
-
-    volatile_members_lock[pool_id]->unlock(__FILE__, __LINE__);
-  }
-
-  if(purged)
-    reloadPools();
-}
-
-/* *************************************** */
-
-void HostPools::removeVolatileMemberFromPool(char *host_or_mac, u_int16_t user_pool_id) {
-  volatile_members_t *m;
-  bool purged = false;
-
-  if(user_pool_id == NO_HOST_POOL_ID || user_pool_id >= MAX_NUM_HOST_POOLS || !host_or_mac)
-    return;
-
-  volatile_members_lock[user_pool_id]->lock(__FILE__, __LINE__);
-
-  HASH_FIND_STR(volatile_members[user_pool_id], host_or_mac, m);
-  if(m) {
-    HASH_DEL(volatile_members[user_pool_id], m);
-    free(m->host_or_mac);
-    free(m);
-    purged = true;
-  }
-
-  volatile_members_lock[user_pool_id]->unlock(__FILE__, __LINE__);
-
-  if(purged)
-    reloadPools();
-}
-
 #endif
 
 /* *************************************** */
@@ -605,7 +314,6 @@ void HostPools::lua(lua_State *vm) {
   u_int32_t hosts = 0, l2_devices = 0;
   u_int32_t cur_hosts = 0, cur_l2 = 0;
   u_int32_t active_pools = 0;
-  char buf[8];
 
   lua_newtable(vm);
 
@@ -614,6 +322,8 @@ void HostPools::lua(lua_State *vm) {
     if((cur_l2 = getNumPoolL2Devices(i))) l2_devices += cur_l2;
 
     if(cur_hosts || cur_l2) {
+      char buf[32];
+      
       lua_newtable(vm);
       lua_push_uint64_table_entry(vm, "num_hosts", cur_hosts);
       lua_push_uint64_table_entry(vm, "num_l2_devices", cur_l2);
@@ -647,43 +357,35 @@ void HostPools::reloadPools() {
   char kname[CONST_MAX_LEN_REDIS_KEY];
   char **pools, **pool_members, *at, *member;
   int num_pools, num_members, actual_num_members;
-  u_int16_t _pool_id, vlan_id;
-  VlanAddressTree *new_tree;
-#ifdef NTOPNG_PRO
+  u_int16_t _pool_id;
+  VLANid vlan_id;
+  VLANAddressTree *new_tree;
   HostPoolStats **new_stats;
-#endif
   Redis *redis = ntop->getRedis();
 
   if(!iface || (iface->get_id() == -1))
     return;
 
-  if((new_tree = new VlanAddressTree) == NULL
-#ifdef NTOPNG_PRO
-     || (new_stats = new HostPoolStats*[MAX_NUM_HOST_POOLS]) == NULL
-#endif
-     ) {
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "Not enough memory");
-    return;
-  }
-
-#ifdef NTOPNG_PRO
+  new_tree = new (std::nothrow) VLANAddressTree;
+  new_stats = new (std::nothrow) HostPoolStats*[MAX_NUM_HOST_POOLS];
   for(u_int32_t i = 0; i < MAX_NUM_HOST_POOLS; i++)
     new_stats[i] = NULL;
-#endif
 
-  snprintf(kname, sizeof(kname), HOST_POOL_IDS_KEY, iface->get_id());
+  snprintf(kname, sizeof(kname), HOST_POOL_IDS_KEY);
 
-#ifdef NTOPNG_PRO
   /* Always allocate default pool stats */
   if(stats && stats[0]) /* Duplicate existing statistics */
-    new_stats[0] = new HostPoolStats(*stats[0]);
+    new_stats[0] = new (std::nothrow) HostPoolStats(*stats[0]);
   else /* Brand new statistics */
-    new_stats[0] = new HostPoolStats(iface);
-#endif
+    new_stats[0] = new (std::nothrow) HostPoolStats(iface);
+
+  /* Set the default pool name into the stats */
+  if(new_stats[0]) new_stats[0]->updateName(DEFAULT_POOL_NAME);
 
   /* Keys are pool ids */
-  num_pools = redis->smembers(kname, &pools);
-
+  if((num_pools = redis->smembers(kname, &pools)) == -1)
+    return; /* Something went wrong with redis? */
+  
   for(int i = 0; i < num_pools; i++) {
     if(!pools[i])
       continue;
@@ -693,19 +395,26 @@ void HostPools::reloadPools() {
       ntop->getTrace()->traceEvent(TRACE_WARNING, "Ignoring pool [pool id: %2d]. "
 				   "Maximum number of host pools for this license is %u, inclusive of the Not Assigned pool.",
 				   _pool_id, MAX_NUM_HOST_POOLS);
+
+      free(pools[i]);
       continue;
     }
 
-#ifdef NTOPNG_PRO
+    snprintf(kname, sizeof(kname), HOST_POOL_DETAILS_KEY, _pool_id);
+
     if(_pool_id != 0) { /* Pool id 0 stats already updated */
       if(stats && stats[_pool_id]) /* Duplicate existing statistics */
-	new_stats[_pool_id] = new HostPoolStats(*stats[_pool_id]);
+	new_stats[_pool_id] = new (std::nothrow) HostPoolStats(*stats[_pool_id]);
       else /* Brand new statistics */
-	new_stats[_pool_id] = new HostPoolStats(iface);
+	new_stats[_pool_id] = new (std::nothrow) HostPoolStats(iface);
     }
-#endif
 
-    snprintf(kname, sizeof(kname), HOST_POOL_DETAILS_KEY, iface->get_id(), _pool_id);
+    /* Initialize the name */
+    if(new_stats[_pool_id]) {
+      char name_rsp[POOL_MAX_NAME_LEN];
+      redis->hashGet(kname, (char*)"name", name_rsp, sizeof(name_rsp));
+      new_stats[_pool_id]->updateName(name_rsp);
+    }
 
 #ifdef NTOPNG_PRO
     char rsp[16] = { 0 };
@@ -727,13 +436,14 @@ void HostPools::reloadPools() {
 
 #ifdef HOST_POOLS_DEBUG
     redis->hashGet(kname, (char*)"name", rsp, sizeof(rsp));
-    ntop->getTrace()->traceEvent(TRACE_NORMAL, "Loading pool [iteration: %u][pool_id: %u][name: %s]"
+    ntop->getTrace()->traceEvent(TRACE_NORMAL, "Loading pool [%s][iteration: %u][pool_id: %u][name: %s]"
 				 "[children_safe: %i]"
 				 "[forge_global_dns: %i]"
 				 "[pool_shaper: %i]"
 				 "[schedule_bitmap: %i]"
 				 "[enforce_quotas_per_pool_member: %i]"
 				 "[enforce_shapers_per_pool_member: %i]",
+				 iface->get_name(),
 				 i, _pool_id,
 				 rsp, children_safe[_pool_id], forge_global_dns[_pool_id],
 				 pool_shaper[_pool_id], schedule_bitmap[_pool_id],
@@ -743,7 +453,7 @@ void HostPools::reloadPools() {
 
 #endif /* NTOPNG_PRO */
 
-    snprintf(kname, sizeof(kname), HOST_POOL_MEMBERS_KEY, iface->get_id(), pools[i]);
+    snprintf(kname, sizeof(kname), HOST_POOL_MEMBERS_KEY, pools[i]);
 
     /* Pool members are the elements of the list */
     if((num_members = redis->smembers(kname, &pool_members)) > 0) {
@@ -792,21 +502,15 @@ void HostPools::reloadPools() {
 
   if(pools) free(pools);
 
-#ifdef NTOPNG_PRO
-  if(ntop->getPrefs()->isCaptivePortalEnabled())
-    reloadVolatileMembers(new_tree /* Reload only on the new */);
   swap(new_tree, new_stats);
-#else
-  swap(new_tree);
-#endif
 
   iface->refreshHostPools();
 }
 
 /* *************************************** */
 
-bool HostPools::findMacPool(u_int8_t *mac, u_int16_t *found_pool) {
-  VlanAddressTree *cur_tree; /* must use this as tree can be swapped */
+bool HostPools::findMacPool(const u_int8_t * const mac, u_int16_t *found_pool) {
+  VLANAddressTree *cur_tree; /* must use this as tree can be swapped */
   int16_t ret;
 
   if(!tree || !(cur_tree = tree))
@@ -833,8 +537,9 @@ bool HostPools::findMacPool(Mac *mac, u_int16_t *found_pool) {
 
 /* *************************************** */
 
-bool HostPools::findIpPool(IpAddress *ip, u_int16_t vlan_id, u_int16_t *found_pool, patricia_node_t **found_node) {
-  VlanAddressTree *cur_tree; /* must use this as tree can be swapped */
+bool HostPools::findIpPool(IpAddress *ip, VLANid vlan_id,
+			   u_int16_t *found_pool, ndpi_patricia_node_t **found_node) {
+  VLANAddressTree *cur_tree; /* must use this as tree can be swapped */
 #ifdef HOST_POOLS_DEBUG
   char buf[128];
 #endif
@@ -842,15 +547,15 @@ bool HostPools::findIpPool(IpAddress *ip, u_int16_t vlan_id, u_int16_t *found_po
   if(!tree || !(cur_tree = tree))
     return(false);
 
-  *found_node = (patricia_node_t*)ip->findAddress(cur_tree->getAddressTree(vlan_id));
+  *found_node = (ndpi_patricia_node_t*)ip->findAddress(cur_tree->getAddressTree(vlan_id));
 
   if(*found_node) {
 #ifdef HOST_POOLS_DEBUG
       ntop->getTrace()->traceEvent(TRACE_NORMAL,
 				   "Found pool for %s [pool id: %i]",
-				   ip->print(buf, sizeof(buf)), (*found_node)->user_data);
+				   ip->print(buf, sizeof(buf)), ndpi_patricia_get_node_u64(*found_node));
 #endif
-      *found_pool = (*found_node)->user_data;
+      *found_pool = ndpi_patricia_get_node_u64(*found_node);
       return(true);
   }
 
@@ -861,7 +566,7 @@ bool HostPools::findIpPool(IpAddress *ip, u_int16_t vlan_id, u_int16_t *found_po
 
 u_int16_t HostPools::getPool(Host *h) {
   u_int16_t pool_id;
-  patricia_node_t *node;
+  ndpi_patricia_node_t *node;
   bool found = false;
 
   if(h) {
@@ -893,3 +598,17 @@ u_int16_t HostPools::getPool(Host *h) {
 
   return pool_id;
 }
+
+/* *************************************** */
+
+/* Returns a pool id, given a pool name. When no pool id is found, the default pool id is returned */
+u_int16_t HostPools::getPoolByName(const char * pool_name) {
+  HostPoolStats *hps;
+
+  for(int i = 0; i < MAX_NUM_HOST_POOLS; i++) {
+    if((hps = getPoolStats(i)) && hps->getName().compare(pool_name) == 0)
+      return i;
+  }
+
+  return NO_HOST_POOL_ID;
+}
