@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,15 +31,25 @@ IpAddress::IpAddress() {
 
 /* ******************************************* */
 
-void IpAddress::set(char *sym_addr) {
+IpAddress::IpAddress(const IpAddress& ipa) {
+  set(&ipa);
+  memcpy(&addr, &ipa.addr, sizeof(addr));
+  compute_key();
+}
+
+/* ******************************************* */
+
+void IpAddress::set(const char* const sym_addr) {
+  if(sym_addr == NULL) return;
+  
   if(strchr(sym_addr, '.')) {
-    addr.ipVersion = 4, addr.localHost = 0, addr.ipType.ipv4 = inet_addr(sym_addr);
+    addr.ipVersion = 4, addr.ipType.ipv4 = inet_addr(sym_addr);
   } else {
     if(inet_pton(AF_INET6, sym_addr, &addr.ipType.ipv6) <= 0) {
       /* We failed */
-      addr.ipVersion = 4, addr.localHost = 0, addr.ipType.ipv4 = 0;
+      addr.ipVersion = 4, addr.ipType.ipv4 = 0;
     } else {
-      addr.ipVersion = 6, addr.localHost = 0;
+      addr.ipVersion = 6;
     }
   }
 
@@ -50,27 +60,39 @@ void IpAddress::set(char *sym_addr) {
 
 void IpAddress::set(union usa *ip) {
   if(ip->sin.sin_family != AF_INET6) {
-    addr.ipVersion = 4, addr.localHost = 0, addr.ipType.ipv4 = ip->sin.sin_addr.s_addr;
+    addr.ipVersion = 4, addr.ipType.ipv4 = ip->sin.sin_addr.s_addr;
+  } else {
+    memcpy(&addr.ipType.ipv6, &ip->sin6.sin6_addr, sizeof(struct ndpi_in6_addr));
+    addr.ipVersion = 6;
   }
-#if defined(USE_IPV6)
-  else {
-    memcpy(&addr.ipType.ipv6, &ip->sin6.sin6_addr.s6_addr, sizeof(struct ndpi_in6_addr));
-    addr.ipVersion = 6, addr.localHost = 0;
-  }
-#endif
+
+  compute_key();
 }
 
 /* ******************************************* */
 
-bool IpAddress::isEmpty() {
+void IpAddress::reloadBlacklist(ndpi_detection_module_struct* ndpi_struct) {
+  char ipbuf[64];
+  char *ip_str = print(ipbuf, sizeof(ipbuf));
+  ndpi_protocol_category_t category;
+
+  if((ndpi_get_custom_category_match(ndpi_struct, ip_str, strlen(ip_str), &category) == 0)
+     && category == CUSTOM_CATEGORY_MALWARE)
+    addr.blacklistedIP = true;
+}
+
+/* ******************************************* */
+
+bool IpAddress::isEmpty() const {
   if((addr.ipVersion == 0)
      || ((addr.ipVersion == 4) && (addr.ipType.ipv4 == 0))) {
     return true;
-  }else if(addr.ipVersion == 6) {
+  } else if(addr.ipVersion == 6) {
     struct ndpi_in6_addr empty_ipv6;
     memset(&empty_ipv6, 0, sizeof(empty_ipv6));
     return memcmp((void*)&empty_ipv6, (void*)&addr.ipType.ipv6, sizeof(empty_ipv6)) == 0 ? true : false;
   }
+
   return false;
 }
 
@@ -82,9 +104,16 @@ void IpAddress::checkIP() {
   u_int32_t nmask;
   u_int8_t nmask_bits;
 
-  addr.privateIP = false; /* Default */
+  /* Default */
+  addr.loopbackIP = false;
+  addr.privateIP = false;
+  addr.multicastIP = false;
+  addr.broadcastIP = false; 
+  addr.localIP = false; 
 
   if(addr.ipVersion == 4) {
+    addr.loopbackIP = addr.ipType.ipv4 == 0x0100007F /* 127.0.0.1 */ ? true : false;
+
     /*
       RFC 1918 - Private Address Space
 
@@ -102,21 +131,35 @@ void IpAddress::checkIP() {
        || ((a & 0xFFF00000) == 0xAC100000 /* 172.16.0.0/12 */)
        || ((a & 0xFFFF0000) == 0xC0A80000 /* 192.168.0.0/16 */)
        || ((a & 0xFF000000) == 0x7F000000 /* 127.0.0.0/8 */)
+       || ((a & 0xFFFF0000) == 0xA9FE0000 /* 169.254.0.0/16 Link-Local communication rfc3927 */)
+       || (a == 0xFFFFFFFF /* 255.255.255.255 */)
+       || (a == 0x0        /* 0.0.0.0 */)
        )
       addr.privateIP = true;
-    else if((a & 0xF0000000) == 0xE0000000 /* 224.0.0.0/4 */)
+
+    if((a & 0xF0000000) == 0xE0000000 /* 224.0.0.0/4 */)
       addr.multicastIP = true;
     else if((a == 0xFFFFFFFF) || (a == 0))
       addr.broadcastIP = true;
-
-    if(ntop->isLocalAddress(AF_INET, &addr.ipType.ipv4, &local_network_id, &nmask_bits)) {
-      if(nmask_bits < 31) { /* /32 is just an host, /31 is a point-to-point */
+    else if(ntop->isLocalAddress(AF_INET, &addr.ipType.ipv4, &local_network_id, &nmask_bits)) {
+      addr.localIP = true;
+      if(nmask_bits > 0 && nmask_bits < 31) { /* /0 no mask, /32 is just an host, /31 is a point-to-point */
         nmask = ~((1 << (32 - nmask_bits)) - 1);
-        if(a == (a | ~nmask) || a == (a & nmask))
-    addr.broadcastIP = true;
+        if(a == (a | ~nmask)   /* e.g., 10.0.0.0/8 -> matches 10.255.255.255.255 */
+	   || a == (a & nmask) /* e.g., 10.0.0.0/8 -> matches 10.0.0.0 */)
+	  addr.broadcastIP = true;
       }
     }
   } else if (addr.ipVersion == 6) {
+    addr.loopbackIP = false; /* TODO: compute loopback for IPv6 */
+
+    /*
+      Link-local addresses have the prefix fe80::/10, we mark them as private
+    */
+    if((addr.ipType.ipv6.u6_addr.u6_addr8[0] & 0xFF) == 0xFE
+       && (addr.ipType.ipv6.u6_addr.u6_addr8[1] & 0xC0) == 0x80)
+      addr.privateIP = true;
+
     /*
      * https://tools.ietf.org/html/rfc2373#section-2.7
      *
@@ -124,6 +167,9 @@ void IpAddress::checkIP() {
      */
     if(addr.ipType.ipv6.u6_addr.u6_addr8[0] == 0xFF)
       addr.multicastIP = true;
+    
+    if(ntop->isLocalAddress(AF_INET6, (void*)&addr.ipType.ipv6, &local_network_id))
+      addr.localIP = true; 
   }
 }
 
@@ -153,17 +199,8 @@ bool IpAddress::isLocalInterfaceAddress() {
   bool systemHost;
 
   if(addr.ipVersion == 4) {
-    ip_key = ntohl(addr.ipType.ipv4);
-
     systemHost = ntop->isLocalInterfaceAddress(AF_INET, &addr.ipType.ipv4);
   } else if(addr.ipVersion == 6) {
-    u_int32_t key = 0;
-
-    for(u_int32_t i=0; i<4; i++)
-      key += addr.ipType.ipv6.u6_addr.u6_addr32[i];
-
-    ip_key = key;
-
     systemHost = ntop->isLocalInterfaceAddress(AF_INET6, &addr.ipType.ipv6);
   } else
     systemHost = false;
@@ -174,8 +211,6 @@ bool IpAddress::isLocalInterfaceAddress() {
 /* ******************************************* */
 
 void IpAddress::compute_key() {
-  if(ip_key != 0) return; /* Already computed */
-
   checkIP();
 
   if(addr.ipVersion == 4) {
@@ -184,19 +219,37 @@ void IpAddress::compute_key() {
     ip_key = 0;
 
     for(u_int32_t i=0; i<4; i++)
-      ip_key += addr.ipType.ipv6.u6_addr.u6_addr32[i];
+      ip_key += addr.ipType.ipv6.u6_addr.u6_addr32[i] << (2*i);
   }
 }
 
 /* ******************************************* */
 
-char* IpAddress::print(char *str, u_int str_len, u_int8_t bitmask) {
+char* IpAddress::print(char *str, u_int str_len, u_int8_t bitmask) const {
   str[0] = '\0';
   return(intoa(str, str_len, bitmask));
 }
 
 /* ******************************************* */
 
+char* IpAddress::get_ip_hex(char *buf, u_int buf_len) {
+  
+  if(addr.ipVersion == 4)
+    snprintf(buf, buf_len, "%08X", ntohl(addr.ipType.ipv4));
+  else if(addr.ipVersion == 6)
+    snprintf(buf, buf_len, "%08X%08X%08X%08X",
+	     ntohl(addr.ipType.ipv6.u6_addr.u6_addr32[0]),
+	     ntohl(addr.ipType.ipv6.u6_addr.u6_addr32[1]),
+	     ntohl(addr.ipType.ipv6.u6_addr.u6_addr32[2]),
+	     ntohl(addr.ipType.ipv6.u6_addr.u6_addr32[3]));
+  else
+    buf[0] = '\0';
+
+  return(buf);
+}
+
+/* ******************************************* */
+
 char* IpAddress::printMask(char *str, u_int str_len, bool isLocalIP) {
   if(Utils::maskHost(isLocalIP)) {
     snprintf(str, str_len, (addr.ipVersion == 4) ? "0.0.0.0" : "::");
@@ -207,14 +260,33 @@ char* IpAddress::printMask(char *str, u_int str_len, bool isLocalIP) {
 
 /* ******************************************* */
 
-bool IpAddress::isLocalHost(int16_t *network_id) {
+bool IpAddress::isLocalHost() const {
+  return (addr.localIP || addr.multicastIP || addr.broadcastIP);
+}
+
+/* ******************************************* */
+
+bool IpAddress::isLocalHost(int16_t *network_id) const {
+  bool local = false;
+
+  if(addr.multicastIP || addr.broadcastIP)
+    return(true);
+  
   if(addr.ipVersion == 4) {
     u_int32_t v = /* htonl */(addr.ipType.ipv4);
-
-    return(ntop->isLocalAddress(AF_INET, (void*)&v, network_id));
+    local = ntop->isLocalAddress(AF_INET, (void*)&v, network_id);
   } else {
-    return(ntop->isLocalAddress(AF_INET6, (void*)&addr.ipType.ipv6, network_id));
+    local = ntop->isLocalAddress(AF_INET6, (void*)&addr.ipType.ipv6, network_id);
   }
+
+#if 0 /* Debug */
+  char buff[64];
+  ntop->getTrace()->traceEvent(TRACE_WARNING, "IP %s is %sLocal [broadcast=%u][multicast=%u]",
+    print(buff, sizeof(buff), 128), local ? "" : "NOT ",
+    addr.broadcastIP, addr.multicastIP);
+#endif
+
+  return local;
 }
 
 /* ******************************************* */
@@ -284,7 +356,6 @@ json_object* IpAddress::getJSONObject() {
   my_object = json_object_new_object();
 
   json_object_object_add(my_object, "ipVersion", json_object_new_int(addr.ipVersion));
-  json_object_object_add(my_object, "localHost", json_object_new_boolean(addr.localHost));
   json_object_object_add(my_object, "ip", json_object_new_string(print(buf, sizeof(buf))));
 
   return(my_object);
@@ -293,17 +364,17 @@ json_object* IpAddress::getJSONObject() {
 /* ******************************************* */
 
 /**
- * @brief Check if the host matches the specified host tree
- *
+ * @brief Check if the host matches the specified host tree.
+
  * @param ptree     The hosts allowed to be accessed.
- * @return true if the host matches the ptre, false otherwise.
+ * @return true if the host matches the tree, false otherwise.
  */
-bool IpAddress::match(AddressTree *tree) {
+bool IpAddress::match(const AddressTree * const tree) const {
   if(tree == NULL)
     return(true);
   else {
-    patricia_tree_t *ptree = tree->getTree((addr.ipVersion == 4) ? true : false);
-    patricia_node_t *node;
+    ndpi_patricia_tree_t *ptree = tree->getTree((addr.ipVersion == 4) ? true : false);
+    ndpi_patricia_node_t *node;
 
     if(ptree == NULL) return(true);
 
@@ -318,14 +389,14 @@ bool IpAddress::match(AddressTree *tree) {
 
 /* ****************************** */
 
-char* IpAddress::intoa(char* buf, u_short bufLen, u_int8_t bitmask) {
+char* IpAddress::intoa(char* buf, u_short bufLen, u_int8_t bitmask) const {
   if((addr.ipVersion == 4) || (addr.ipVersion == 0 /* Misconfigured */)) {
     bitmask = bitmask <= 32 ? bitmask : 32;
     u_int32_t a = ntohl(addr.ipType.ipv4);
 
     if(bitmask > 0) {
       /* bitmask 0 here causes integer overflow */
-      u_int32_t netmask = ~((1 << (32 - bitmask)) - 1);
+      u_int32_t netmask = ~((((u_int32_t)1) << (32 - bitmask)) - 1);
       a &= netmask;
     } else {
       /* bitmask is 0 */
@@ -338,15 +409,24 @@ char* IpAddress::intoa(char* buf, u_short bufLen, u_int8_t bitmask) {
     return(Utils::intoaV6(addr.ipType.ipv6, bitmask, buf, bufLen));
   }
 }
+
+/* ****************************** */
+
+void IpAddress::incCardinality(Cardinality *c) {
+  if(isIPv4())
+    c->addElement(get_ipv4());
+  else
+    c->addElement((const char *)get_ipv6(), sizeof(struct ndpi_in6_addr));
+}
+
 /* ****************************** */
 
 void IpAddress::dump() {
-  int16_t network_id;
   char buf[48];
   const char *local, *system;
 
   ntop->getTrace()->traceEvent(TRACE_NORMAL, "-------------------- [ Local ]");
-  local  =  isLocalHost(&network_id)  ? "Yes" : "No";
+  local  =  isLocalHost()  ? "Yes" : "No";
   ntop->getTrace()->traceEvent(TRACE_NORMAL, "-------------------- [ System ]");
   system =  isLocalInterfaceAddress() ? "Yes" : "No";
   ntop->getTrace()->traceEvent(TRACE_NORMAL, "--------------------");
