@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,18 +23,24 @@
 
 /* *************************************** */
 
-Mac::Mac(NetworkInterface *_iface, u_int8_t _mac[6])
-  : GenericHashEntry(_iface), GenericTrafficElement() {
+Mac::Mac(NetworkInterface *_iface, u_int8_t _mac[6]) : GenericHashEntry(_iface) {
   memcpy(mac, _mac, 6);
-  memset(&arp_stats, 0, sizeof(arp_stats));
+
+  broadcast_mac = Utils::isBroadcastMac(mac);
   special_mac = Utils::isSpecialMac(mac);
-  source_mac = false, fingerprint = NULL, dhcpHost = false;
+  source_mac = false, fingerprint = NULL;
   bridge_seen_iface_id = 0, lockDeviceTypeChanges = false;
-  device_type = device_unknown, os = os_unknown;
+  memset(&names, 0, sizeof(names));
+  device_type = device_unknown;
+  host_pool_id = NO_HOST_POOL_ID;
 #ifdef NTOPNG_PRO
   captive_portal_notified = 0;
 #endif
-  ndpiStats = NULL, model = NULL, ssid = NULL;
+  model = NULL, ssid = NULL;
+  stats_reset_requested = data_delete_requested = false;
+  stats = new (std::nothrow) MacStats(_iface);
+  stats_shadow = NULL;
+  last_stats_reset = ntop->getLastStatsReset(); /* assume fresh stats, may be changed by deserialize */
 
   char redis_key[64], buf1[64], rsp[8];
   char *mac_ptr = Utils::formatMac(mac, buf1, sizeof(buf1));
@@ -60,53 +66,55 @@ Mac::Mac(NetworkInterface *_iface, u_int8_t _mac[6])
 			       iface->getNumL2Devices());
 #endif
 
-  /*
-   * Note: We do not load MAC data from redis right now.
-   * We only need redis MAC data to show Unassigned Devices in host pools view.
-   */
-  if(!special_mac) {
-    char *json = NULL;
-    u_int json_len = 0;
+  if(!special_mac && ntop->getPrefs()->is_idle_local_host_cache_enabled()) {
+    deserializeFromRedis();
 
-    snprintf(redis_key, sizeof(redis_key), MAC_SERIALIZED_KEY, iface->get_id(), mac_ptr);
+    // Load the user defined device type, if available
+    snprintf(redis_key, sizeof(redis_key), MAC_CUSTOM_DEVICE_TYPE, mac_ptr);
+    if((ntop->getRedis()->get(redis_key, rsp, sizeof(rsp)) == 0) && rsp[0])
+      forceDeviceType((DeviceType)atoi(rsp));
 
-    if((json_len = ntop->getRedis()->len(redis_key)) > 0
-       && ++json_len <= HOST_MAX_SERIALIZED_LEN) {
-      if((json = (char*)malloc(json_len * sizeof(char))) == NULL) {
-	ntop->getTrace()->traceEvent(TRACE_ERROR,
-				     "Unable to allocate memory to deserialize %s", redis_key);
-      } else if(!ntop->getRedis()->get(redis_key, json, json_len)) {
-	/* Found saved copy of the host so let's start from the previous state */
-	// ntop->getTrace()->traceEvent(TRACE_NORMAL, "%s => %s", redis_key, json);
-	ntop->getTrace()->traceEvent(TRACE_INFO, "Deserializing %s", redis_key);
-	deserialize(redis_key, json);
-      }
-
-      if(json) free(json);
-    }
+    readDHCPCache();
   }
 
-  // Load the user defined device type, if available
-  snprintf(redis_key, sizeof(redis_key), MAC_CUSTOM_DEVICE_TYPE, mac_ptr);
-  if((ntop->getRedis()->get(redis_key, rsp, sizeof(rsp)) == 0) && rsp[0])
-    forceDeviceType((DeviceType)atoi(rsp));
-
-  updateHostPool(true /* Inline */);
+  updateHostPool(true /* inline with packet processing */, true /* first inc */);
 }
 
 /* *************************************** */
 
 Mac::~Mac() {
+  if(!special_mac &&  ntop->getRedis() && ntop->getPrefs()->is_pro_edition()) {
+    char mac_addr[64], mac_disconnection_key[128];
+    snprintf(mac_disconnection_key, sizeof(mac_disconnection_key), (char*) MACS_DISCONNECTION, iface->get_id());
+    ntop->getRedis()->lpush(mac_disconnection_key, print(mac_addr, sizeof(mac_addr)), 0 /* No Trim */);
+  }
+
+  if(model) free(model);
+  if(ssid) free(ssid);
+  if(fingerprint) free(fingerprint);
+  freeMacData();
+  if(stats) delete(stats);
+  if(stats_shadow) delete(stats_shadow);
+  
+#ifdef DEBUG
+  ntop->getTrace()->traceEvent(TRACE_NORMAL, "Deleted %s [total %u][%s]",
+			       Utils::formatMac(mac, buf, sizeof(buf)),
+			       iface->getNumL2Devices(),
+			       source_mac ? "Host" : "Special");
+#endif
+}
+
+/* *************************************** */
+
+void Mac::set_hash_entry_state_idle() {
   if(source_mac)
     iface->decNumL2Devices();
 
   if(!special_mac) {
-    char key[64], buf1[64];
-    char *json = serialize();
-
-    snprintf(key, sizeof(key), MAC_SERIALIZED_KEY, iface->get_id(), Utils::formatMac(mac, buf1, sizeof(buf1)));
-    ntop->getRedis()->set(key, json, ntop->getPrefs()->get_local_host_cache_duration());
-    free(json);
+    if(data_delete_requested)
+      deleteRedisSerialization();
+    else if(ntop->getPrefs()->is_idle_local_host_cache_enabled())
+      serializeToRedis();
   }
 
   /* Pool counters are updated both in and outside the datapath.
@@ -130,52 +138,12 @@ Mac::~Mac() {
 			       iface->getHostPools()->getNumPoolL2Devices(get_host_pool()));
 #endif
 
-  if(model) free((void*)model);
-  if(ssid) free((void*)ssid);
-  
-  if(ndpiStats) {
-    delete ndpiStats;
-    ndpiStats = NULL;
-  }
-
-  if(fingerprint)
-    free(fingerprint);
-
-#ifdef DEBUG
-  ntop->getTrace()->traceEvent(TRACE_NORMAL, "Deleted %s [total %u][%s]",
-			       Utils::formatMac(mac, buf, sizeof(buf)),
-			       iface->getNumL2Devices(),
-			       source_mac ? "Host" : "Special");
-#endif
-}
-
-/* *************************************** */
-
-bool Mac::idle() {
-  bool rc;
-
-  if((num_uses > 0) || (!iface->is_purge_idle_interface()))
-    return(false);
-
-  rc = isIdle(MAX_LOCAL_HOST_IDLE);
-
-#ifdef DEBUG
-  if(true) {
-    char buf[32];
-
-    ntop->getTrace()->traceEvent(TRACE_NORMAL, "Is idle %s [uses %u][%s][last: %u][diff: %d]",
-				 Utils::formatMac(mac, buf, sizeof(buf)),
-				 num_uses,
-				 rc ? "Idle" : "Not Idle",
-				 last_seen, iface->getTimeLastPktRcvd() - (last_seen+MAX_LOCAL_HOST_IDLE));
-  }
-#endif
-
-  return(rc);
+  GenericHashEntry::set_hash_entry_state_idle();
 }
 
 /* *************************************** */
 
+#ifdef HAVE_NEDGE
 static const char* location2str(MacLocation location) {
   switch(location) {
     case located_on_lan_interface: return "lan";
@@ -183,6 +151,7 @@ static const char* location2str(MacLocation location) {
     default: return "unknown";
   }
 }
+#endif
 
 /* *************************************** */
 
@@ -198,31 +167,23 @@ void Mac::lua(lua_State* vm, bool show_details, bool asListElement) {
     if(manuf)
       lua_push_str_table_entry(vm, "manufacturer", (char*)manuf);
 
-    lua_push_uint64_table_entry(vm, "arp_requests.sent", arp_stats.sent_requests);
-    lua_push_uint64_table_entry(vm, "arp_requests.rcvd", arp_stats.rcvd_requests);
-    lua_push_uint64_table_entry(vm, "arp_replies.sent", arp_stats.sent_replies);
-    lua_push_uint64_table_entry(vm, "arp_replies.rcvd", arp_stats.rcvd_replies);
-
     lua_push_bool_table_entry(vm, "source_mac", source_mac);
     lua_push_bool_table_entry(vm, "special_mac", special_mac);
+#ifdef HAVE_NEDGE
     lua_push_str_table_entry(vm, "location", (char *) location2str(locate()));
+#endif
     lua_push_uint64_table_entry(vm, "devtype", device_type);
     if(model) lua_push_str_table_entry(vm, "model", (char*)model);
     if(ssid) lua_push_str_table_entry(vm, "ssid", (char*)ssid);
-    if(ndpiStats) ndpiStats->lua(iface, vm, true);
   }
 
-  ((GenericTrafficElement*)this)->lua(vm, true);
+  stats->lua(vm, show_details);
 
-  lua_push_bool_table_entry(vm, "dhcpHost", dhcpHost);
-  lua_push_str_table_entry(vm, "fingerprint", fingerprint ? fingerprint : (char*)"");
-  lua_push_uint64_table_entry(vm, "operatingSystem", os);
+  lua_push_str_table_entry(vm,    "fingerprint", fingerprint ? fingerprint : (char*)"");
   lua_push_uint64_table_entry(vm, "seen.first", first_seen);
   lua_push_uint64_table_entry(vm, "seen.last", last_seen);
   lua_push_uint64_table_entry(vm, "duration", get_duration());
-
   lua_push_uint64_table_entry(vm, "num_hosts", getNumHosts());
-
   lua_push_uint64_table_entry(vm, "pool", get_host_pool());
 
   if(asListElement) {
@@ -234,9 +195,14 @@ void Mac::lua(lua_State* vm, bool show_details, bool asListElement) {
 /* *************************************** */
 
 bool Mac::isNull() const {
-  u_int8_t zero_mac[6] = {0};
+  u_int8_t zero_mac[3] = { 0 };
 
-  return !memcmp(mac, zero_mac, sizeof(mac));
+  /*
+    We need to compare only the manufacturer (3 byets instead of 6)
+    as there might be variations of 00:00:00:00:00:00
+    such as 00:00:00:00:01:01 that are basically alike
+  */
+  return(!memcmp(mac, zero_mac, sizeof(zero_mac)));
 }
 
 /* *************************************** */
@@ -252,78 +218,65 @@ bool Mac::equal(const u_int8_t _mac[6]) {
 
 /* *************************************** */
 
-char* Mac::serialize() {
-  json_object *my_object = getJSONObject();
-  char *rsp = strdup(json_object_to_json_string(my_object));
+char* Mac::getSerializationKey(char *buf, uint bufsize) {
+  char buf1[32];
+  char *mac_ptr = Utils::formatMac(mac, buf1, sizeof(buf1));
 
-  ntop->getTrace()->traceEvent(TRACE_INFO, "%s", rsp);
-  
-  /* Free memory */
-  json_object_put(my_object);
-
-  return(rsp);
+  snprintf(buf, bufsize, MAC_SERIALIZED_KEY, iface->get_id(), mac_ptr);
+  return(buf);
 }
 
 /* *************************************** */
 
-void Mac::deserialize(char *key, char *json_str) {
-  json_object *o, *obj;
-  enum json_tokener_error jerr = json_tokener_success;
-
-  ntop->getTrace()->traceEvent(TRACE_INFO, "%s", json_str);
-  
-  if((o = json_tokener_parse_verbose(json_str, &jerr)) == NULL) {
-    ntop->getTrace()->traceEvent(TRACE_WARNING, "JSON Parse error [%s] key: %s: %s",
-				 json_tokener_error_desc(jerr),
-				 key,
-				 json_str);
-    return;
-  }
+void Mac::deserialize(json_object *o) {
+  json_object *obj;
 
   if(json_object_object_get_ex(o, "seen.first", &obj))  first_seen = json_object_get_int64(obj);
   if(json_object_object_get_ex(o, "seen.last", &obj))   last_seen = json_object_get_int64(obj);
+  if(json_object_object_get_ex(o, "last_stats_reset", &obj)) last_stats_reset = json_object_get_int64(obj);
   if(json_object_object_get_ex(o, "devtype", &obj))     device_type = (DeviceType)json_object_get_int(obj);
-  if(json_object_object_get_ex(o, "model", &obj))       setModel((char*)json_object_get_string(obj));
-  if(json_object_object_get_ex(o, "ssid", &obj))        setSSID((char*)json_object_get_string(obj));
-  if(json_object_object_get_ex(o, "fingerprint", &obj)) setFingerprint((char*)json_object_get_string(obj));
-  if(json_object_object_get_ex(o, "operatingSystem", &obj)) setOperatingSystem((OperatingSystem)json_object_get_int(obj));
-  if(json_object_object_get_ex(o, "dhcpHost", &obj))    dhcpHost = json_object_get_boolean(obj);
-  if(ndpiStats && json_object_object_get_ex(o, "ndpiStats", &obj)) ndpiStats->deserialize(iface, obj);
-  if(json_object_object_get_ex(o, "flows.dropped", &obj)) total_num_dropped_flows = json_object_get_int(obj);
+  if(json_object_object_get_ex(o, "model", &obj))       inlineSetModel((char*)json_object_get_string(obj));
+  if(json_object_object_get_ex(o, "ssid", &obj))        inlineSetSSID((char*)json_object_get_string(obj));
+  if(json_object_object_get_ex(o, "fingerprint", &obj)) inlineSetFingerprint((char*)json_object_get_string(obj));
 
-  json_object_put(o);
+  stats->deserialize(o);
+
+  checkStatsReset();
 }
 
 /* *************************************** */
 
-json_object* Mac::getJSONObject() {
-  json_object *my_object;
-  char buf[32];
+bool Mac::statsResetRequested() {
+  return(stats_reset_requested || (last_stats_reset < ntop->getLastStatsReset()));
+}
 
-  if((my_object = json_object_new_object()) == NULL) return(NULL);
+/* *************************************** */
+
+void Mac::serialize(json_object *my_object, DetailsLevel details_level) {
+  char buf[32];
 
   json_object_object_add(my_object, "mac", json_object_new_string(Utils::formatMac(get_mac(), buf, sizeof(buf))));
   json_object_object_add(my_object, "seen.first", json_object_new_int64(first_seen));
   json_object_object_add(my_object, "seen.last",  json_object_new_int64(last_seen));
+  json_object_object_add(my_object, "last_stats_reset",  json_object_new_int64(last_stats_reset));
   json_object_object_add(my_object, "devtype", json_object_new_int(device_type));
   if(model) json_object_object_add(my_object, "model", json_object_new_string(model));
   if(ssid) json_object_object_add(my_object, "ssid", json_object_new_string(ssid));
-  json_object_object_add(my_object, "dhcpHost", json_object_new_boolean(dhcpHost));
-  json_object_object_add(my_object, "operatingSystem", json_object_new_int(os));
   if(fingerprint) json_object_object_add(my_object, "fingerprint", json_object_new_string(fingerprint));
-  if(ndpiStats) json_object_object_add(my_object, "ndpiStats", ndpiStats->getJSONObject(iface));
-  if(total_num_dropped_flows) json_object_object_add(my_object, "flows.dropped", json_object_new_int(total_num_dropped_flows));
 
-  return my_object;
+  if(!statsResetRequested())
+    stats->getJSONObject(my_object);
 }
 
 /* *************************************** */
 
+#ifdef HAVE_NEDGE
 MacLocation Mac::locate() {
   if(iface->is_bridge_interface()) {
-    if(bridge_seen_iface_id == iface->getBridgeLanInterfaceId())
+    InterfaceLocation location = iface->getInterfaceLocation(bridge_seen_iface_id);
+    if(location == lan_interface)
       return(located_on_lan_interface);
-    else if(bridge_seen_iface_id == iface->getBridgeWanInterfaceId())
+    else if(location == wan_interface)
       return(located_on_wan_interface);
   } else {
     if(bridge_seen_iface_id == DUMMY_BRIDGE_INTERFACE_ID)
@@ -332,10 +285,11 @@ MacLocation Mac::locate() {
 
   return(located_on_unknown_interface);
 }
+#endif
 
 /* *************************************** */
 
-void Mac::updateHostPool(bool isInlineCall) {
+void Mac::updateHostPool(bool isInlineCall, bool firstUpdate) {
   if(!iface)
     return;
 
@@ -352,7 +306,7 @@ void Mac::updateHostPool(bool isInlineCall) {
 			       iface->getHostPools()->getNumPoolL2Devices(get_host_pool()));
 #endif
 
-  iface->decPoolNumL2Devices(get_host_pool(), isInlineCall);
+  if(!firstUpdate) iface->decPoolNumL2Devices(get_host_pool(), isInlineCall);
   host_pool_id = iface->getHostPool(this);
   iface->incPoolNumL2Devices(get_host_pool(), isInlineCall);
 
@@ -372,54 +326,15 @@ void Mac::updateHostPool(bool isInlineCall) {
 
 /* *************************************** */
 
-void Mac::updateFingerprint() {
-  /*
-    Inefficient with many signatures but ok for the
-    time being that we have little data
-  */
-  if(!fingerprint) return;
-
-  if(!strcmp(fingerprint,      "017903060F77FC"))
-    setOperatingSystem(os_ios);
-  else if((!strcmp(fingerprint, "017903060F77FC5F2C2E"))
-	  || (!strcmp(fingerprint, "0103060F775FFC2C2E2F"))
-	  || (!strcmp(fingerprint, "0103060F775FFC2C2E"))
-	  )
-    setOperatingSystem(os_macos);
-  else if((!strcmp(fingerprint, "0103060F1F212B2C2E2F79F9FC"))
-	  || (!strcmp(fingerprint, "010F03062C2E2F1F2179F92B"))
-	  )
-    setOperatingSystem(os_windows);
-  else if((!strcmp(fingerprint, "0103060C0F1C2A"))
-	  || (!strcmp(fingerprint, "011C02030F06770C2C2F1A792A79F921FC2A"))
-	  )
-    setOperatingSystem(os_linux); /* Android is also linux */
-  else if((!strcmp(fingerprint, "0603010F0C2C51452B1242439607"))
-	  || (!strcmp(fingerprint, "01032C06070C0F16363A3B45122B7751999A"))
-	  )
-    setOperatingSystem(os_laserjet);
-  else if(!strcmp(fingerprint, "0102030F060C2C"))
-    setOperatingSystem(os_apple_airport);
-  else if(!strcmp(fingerprint, "01792103060F1C333A3B77"))
-    setOperatingSystem(os_android);
-
-  /* Below you can find ambiguous signatures */
-  if(manuf) {
-    if(!strcmp(fingerprint, "0103063633")) {
-      if(strstr(manuf, "Apple"))
-	setOperatingSystem(os_macos);
-      else if(device_type == device_unknown) {
-	setOperatingSystem(os_windows);
-      }
-    }
+char * Mac::getDHCPName(char * const buf, ssize_t buf_size) {
+  if(buf && buf_size) {
+    m.lock(__FILE__, __LINE__);
+    snprintf(buf, buf_size, "%s", names.dhcp ? names.dhcp : "");
+    m.unlock(__FILE__, __LINE__);
   }
-}
-/*
-  Missing OS mapping
 
-  011C02030F06770C2C2F1A792A
-  010F03062C2E2F1F2179F92BFC
-*/
+  return Utils::stringtolower(buf);
+}
 
 /* *************************************** */
 
@@ -459,21 +374,117 @@ void Mac::checkDeviceTypeFromManufacturer() {
 
 /* *************************************** */
 
-void Mac::setModel(char* m) {
-  if(model) free((void*)model);
-  model = strdup(m);
+void Mac::inlineSetModel(const char * the_model) {
+  if(!model && the_model && (model = strdup(the_model))) {
+    if(strstr(model, "AppleTV") != NULL) setDeviceType(device_multimedia);
+    else if(strstr(model, "MacBook") != NULL) setDeviceType(device_laptop);
+    else if(strstr(model, "AirPort") != NULL) setDeviceType(device_wifi);
+    else if(strstr(model, "Mac")     != NULL) setDeviceType(device_workstation);
+    else if(strstr(model, "TimeCapsule") != NULL) setDeviceType(device_nas);
+  }
+}
+/* *************************************** */
 
-  if(strstr(model, "AppleTV") != NULL) setDeviceType(device_multimedia);
-  else if(strstr(model, "MacBook") != NULL) setDeviceType(device_laptop);
-  else if(strstr(model, "AirPort") != NULL) setDeviceType(device_wifi);
-  else if(strstr(model, "Mac")     != NULL) setDeviceType(device_workstation);
-  else if(strstr(model, "TimeCapsule") != NULL) setDeviceType(device_nas);
+bool Mac::inlineSetFingerprint(const char * f) {
+  if(!fingerprint && f) {
+    fingerprint = strdup(f);
+    return(true);
+  }
+
+  return(false);
 }
 
 /* *************************************** */
 
-void Mac::setSSID(char* s) {
-  if(ssid) free((void*)ssid);
-  ssid = strdup(s);
-  setDeviceType(device_wifi);
+void Mac::inlineSetSSID(const char * s) {
+  if(!ssid && s && (ssid = strdup(s)))
+    setDeviceType(device_wifi);
+}
+
+/* *************************************** */
+
+void Mac::inlineSetDHCPName(const char * dhcp_name) {
+  if(!names.dhcp && dhcp_name && (names.dhcp = Utils::toLowerResolvedNames(dhcp_name)))
+    ;
+}
+
+/* *************************************** */
+
+void Mac::checkDataReset() {
+  if(data_delete_requested) {
+    deleteMacData();
+    data_delete_requested = false;
+  }
+}
+
+/* *************************************** */
+
+void Mac::checkStatsReset() {
+  if(statsResetRequested()) {
+    MacStats *new_stats = new (std::nothrow) MacStats(iface);
+    stats_shadow = stats;
+    stats = new_stats;
+    last_stats_reset = ntop->getLastStatsReset();
+    stats_reset_requested = false;
+  }
+}
+
+/* *************************************** */
+
+void Mac::periodic_stats_update(const struct timeval *tv) {
+  checkDataReset();
+  checkStatsReset();
+  stats->updateStats(tv);
+}
+
+/* *************************************** */
+
+void Mac::readDHCPCache() {
+  /* Check DHCP cache */
+  char mac_str[24], buf[64], key[CONST_MAX_LEN_REDIS_KEY];
+
+  if(!names.dhcp && !isNull()) {
+    Utils::formatMac(get_mac(), mac_str, sizeof(mac_str));
+
+    snprintf(key, sizeof(key), DHCP_CACHE, iface->get_id(), mac_str);
+
+    if(ntop->getRedis()->get(key, buf, sizeof(buf)) == 0) {
+      names.dhcp = strdup(buf);
+    }
+  }
+}
+
+/* *************************************** */
+
+void Mac::freeMacData() {
+  // TODO: allow fingerprint, ssid, and model to be resettable
+  if(names.dhcp) { free(names.dhcp); names.dhcp = NULL; }
+}
+
+/* *************************************** */
+
+void Mac::deleteMacData() {
+  m.lock(__FILE__, __LINE__);
+  freeMacData();
+  m.unlock(__FILE__, __LINE__);
+  source_mac = false;
+  device_type = device_unknown;
+#ifdef NTOPNG_PRO
+  captive_portal_notified = false;
+#endif
+  first_seen = last_seen;
+}
+
+/* *************************************** */
+
+u_int64_t Mac::to64(u_int8_t mac[6]) {
+  u_int64_t m = 0;
+  memcpy(&m, mac, 6);
+  return(m);
+}
+
+/* *************************************** */
+
+u_int64_t Mac::get_mac64() {
+  return Mac::to64(mac);
 }
