@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,13 +19,36 @@
  *
  */
 
+/*
+   NOTE
+
+   This class is used to connext to a DB using the MySQL protocol
+   hence it is used by both MySQL and ClickHouse
+*/
+
 #include "ntop_includes.h"
 
 #ifdef HAVE_MYSQL
 
+static bool enable_db_traces = false;
+
+/* **************************************************** */
+
+const char *MySQLDB::getEngineName() {
+  return ntop->getPrefs()->do_dump_flows_on_clickhouse() ? "ClickHouse" : "MySQL";
+}
+
+/* **************************************************** */
+
+void MySQLDB::printFailure(const char *query, int status) {
+  ntop->getTrace()->traceEvent(TRACE_ERROR, "SQL query: %s", query);
+  ntop->getTrace()->traceEvent(TRACE_ERROR, "SQL error (%d): %s", status, get_last_db_error(&mysql));
+}
+
 /* **************************************************** */
 
 static void* queryLoop(void* ptr) {
+  Utils::setThreadName("ntopng-MySQLQry");
   return(((MySQLDB*)ptr)->queryLoop());
 }
 
@@ -33,66 +56,57 @@ static void* queryLoop(void* ptr) {
 
 void* MySQLDB::queryLoop() {
   Redis *r = ntop->getRedis();
-  MYSQL mysql_alt;
   char sql[CONST_MAX_SQL_QUERY_LEN];
+  bool queue_not_empty = false;
 
   while(!ntop->getGlobals()->isShutdown()
-	&& !MySQLDB::isDbCreated() /* wait until the db has been created */) {
+	&& !isDbCreated() /* wait until the db has been created */) {
     sleep(1);
   }
 
-  if(ntop->getGlobals()->isShutdown() || !connectToDB(&mysql_alt, true))
+  if(ntop->getGlobals()->isShutdown())
     return(NULL);
 
-  while(isRunning()) {
+  while(isRunning() || queue_not_empty) {
     int rc = r->lpop(CONST_SQL_QUEUE, sql, sizeof(sql));
 
     if(rc == 0) {
-      if (strlen(sql) >= CONST_MAX_SQL_QUERY_LEN - 1){
-	ntop->getTrace()->traceEvent(TRACE_WARNING,
-				     "Tried to execute a query longer than %u. Skipping.",
-				     CONST_MAX_SQL_QUERY_LEN - 2);
-	continue;  // prevents overflown queries to generate mysql errors
-      } else if((rc = exec_sql_query(&mysql_alt, sql, true /* Attempt to reconnect */, true /* Don't print errors */, false)) < 0) {
-	ntop->getTrace()->traceEvent(TRACE_ERROR, "MySQL error: %s [rc=%d]", get_last_db_error(&mysql_alt), rc);
-	ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", sql);
-
-	/* Don't give up, manually re-connect */
-	disconnectFromDB(&mysql_alt);
-	if(!connectToDB(&mysql_alt, true)) _usleep(100);
-      } else {
-	mysqlExportedFlows++;
-      }
-    } else
-      sleep(1);    
+      queue_not_empty = true;
+      try_exec_sql_query(&mysql, sql);
+    } else {
+      queue_not_empty = false;
+      _usleep(10000);
+    }
   }
 
-  disconnectFromDB(&mysql_alt);
   return(NULL);
 }
 
 /* ******************************************* */
-volatile bool MySQLDB::db_created = false;
-bool MySQLDB::createDBSchema(bool set_db_created) {
+
+bool MySQLDB::createDBSchema() {
   char sql[CONST_MAX_SQL_QUERY_LEN];
+  int rc;
 
   if(iface) {
     disconnectFromDB(&mysql);
     if(connectToDB(&mysql, false) == false){
       ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to connect: %s\n", get_last_db_error(&mysql));
-      return false;
+      return(false);
     }
 
     /* 1 - Create database if missing */
     snprintf(sql, sizeof(sql), "CREATE DATABASE IF NOT EXISTS `%s`", ntop->getPrefs()->get_mysql_dbname());
-    if(exec_sql_query(&mysql, sql, true) < 0) {
-      ntop->getTrace()->traceEvent(TRACE_ERROR, "MySQL error: %s\n", get_last_db_error(&mysql));
-      return false;
+    rc = exec_sql_query(&mysql, sql, true);
+    if (rc < 0) {
+      printFailure(sql, rc);
+      return(false);
     }
 
-    if(mysql_select_db(&mysql, ntop->getPrefs()->get_mysql_dbname())) {
-      ntop->getTrace()->traceEvent(TRACE_ERROR, "MySQL error: %s\n", get_last_db_error(&mysql));
-      return false;
+    rc = mysql_select_db(&mysql, ntop->getPrefs()->get_mysql_dbname());
+    if (rc) {
+      ntop->getTrace()->traceEvent(TRACE_ERROR, "DB select error: %s\n", get_last_db_error(&mysql));
+      return(false);
     }
 
     /* 2.1 - Create table if missing [IPv6] */
@@ -105,8 +119,9 @@ bool MySQLDB::createDBSchema(bool set_db_created) {
 	     "`IP_DST_ADDR` varchar(48) DEFAULT NULL,"
 	     "`L4_DST_PORT` smallint(5) unsigned DEFAULT NULL,"
 	     "`PROTOCOL` tinyint(3) unsigned DEFAULT NULL,"
-	     "`BYTES` int(10) unsigned DEFAULT NULL,"
-	     "`PACKETS` int(10) unsigned DEFAULT NULL,"
+	     "`IN_BYTES` bigint unsigned DEFAULT 0,"
+	     "`OUT_BYTES` bigint unsigned DEFAULT 0,"
+	     "`PACKETS` int unsigned DEFAULT 0,"
 	     "`FIRST_SWITCHED` int(10) unsigned DEFAULT NULL,"
 	     "`LAST_SWITCHED` int(10) unsigned DEFAULT NULL,"
 	     "`INFO` varchar(255) DEFAULT NULL,"
@@ -127,9 +142,10 @@ bool MySQLDB::createDBSchema(bool set_db_created) {
 	     "PARTITIONS 32 */",
 	     ntop->getPrefs()->get_mysql_tablename());
 
-    if(exec_sql_query(&mysql, sql, true) < 0) {
-      ntop->getTrace()->traceEvent(TRACE_ERROR, "MySQL error: %s\n", get_last_db_error(&mysql));
-      return false;
+    rc = exec_sql_query(&mysql, sql, true);
+    if (rc < 0) {
+      printFailure(sql, rc);
+      return(false);
     }
 
     /* 2.2 - Create table if missing [IPv4] */
@@ -142,8 +158,9 @@ bool MySQLDB::createDBSchema(bool set_db_created) {
 	     "`IP_DST_ADDR` int(10) unsigned DEFAULT NULL,"
 	     "`L4_DST_PORT` smallint(5) unsigned DEFAULT NULL,"
 	     "`PROTOCOL` tinyint(3) unsigned DEFAULT NULL,"
-	     "`BYTES` int(10) unsigned DEFAULT NULL,"
-	     "`PACKETS` int(10) unsigned DEFAULT NULL,"
+	     "`IN_BYTES` bigint unsigned DEFAULT 0,"
+	     "`OUT_BYTES` bigint unsigned DEFAULT 0,"
+	     "`PACKETS` int unsigned DEFAULT 0,"
 	     "`FIRST_SWITCHED` int(10) unsigned DEFAULT NULL,"
 	     "`LAST_SWITCHED` int(10) unsigned DEFAULT NULL,"
 	     "`INFO` varchar(255) DEFAULT NULL,"
@@ -164,9 +181,10 @@ bool MySQLDB::createDBSchema(bool set_db_created) {
 	     "PARTITIONS 32 */",
 	     ntop->getPrefs()->get_mysql_tablename());
 
-    if(exec_sql_query(&mysql, sql, true) < 0) {
-      ntop->getTrace()->traceEvent(TRACE_ERROR, "MySQL error: %s\n", get_last_db_error(&mysql));
-      return false;
+    rc = exec_sql_query(&mysql, sql, true);
+    if (rc < 0) {
+      printFailure(sql, rc);
+      return(false);
     }
 
     // the remainder of this method has the purpose of MIGRATING old table structures to
@@ -232,9 +250,9 @@ bool MySQLDB::createDBSchema(bool set_db_created) {
 	     ntop->getPrefs()->get_mysql_tablename(),
 	     ntop->getPrefs()->get_mysql_tablename(), iface->get_id());
     exec_sql_query(&mysql, sql, true, true);
-
     // Drop old tables (their contents have been transferred)
   }
+
   snprintf(sql, sizeof(sql), "DROP TABLE IF EXISTS  `%sv4_%u` ",
 	   ntop->getPrefs()->get_mysql_tablename(), iface->get_id());
   exec_sql_query(&mysql, sql, true, true);
@@ -321,8 +339,8 @@ bool MySQLDB::createDBSchema(bool set_db_created) {
 
       snprintf(sql, sizeof(sql),
 	       "ALTER TABLE `%sv%hu` "
-	       "CHANGE BYTES IN_BYTES INT(10) DEFAULT 0, "
-	       "ADD OUT_BYTES INT(10) DEFAULT 0 AFTER IN_BYTES",
+	       "CHANGE BYTES IN_BYTES BIGINT DEFAULT 0, "
+	       "ADD OUT_BYTES BIGINT DEFAULT 0 AFTER IN_BYTES",
 	       ntop->getPrefs()->get_mysql_tablename(), ipvers[i]);
       exec_sql_query(&mysql, sql, true, true);
     }
@@ -391,7 +409,7 @@ bool MySQLDB::createDBSchema(bool set_db_created) {
 	       "WHERE TABLE_SCHEMA='%s' "
 	       "AND TABLE_NAME='%sv%hu' "
 	       "AND COLUMN_NAME='%s' "
-	       "AND COLUMN_TYPE NOT LIKE '%%UNSIGNED' ",
+	       "AND lower(COLUMN_TYPE) NOT LIKE '%%unsigned' ",
 	       ntop->getPrefs()->get_mysql_dbname(),
 	       ntop->getPrefs()->get_mysql_tablename(),
 	       ipvers[i], counter_fields[j]);
@@ -411,58 +429,30 @@ bool MySQLDB::createDBSchema(bool set_db_created) {
       }
     }
   }
+  for (u_int16_t i = 0; i < sizeof(ipvers) / sizeof(u_int16_t); i++){
+    snprintf(sql, sizeof(sql),
+             "SELECT 1 "
+             "FROM information_schema.COLUMNS "
+             "WHERE TABLE_SCHEMA='%s' "
+             "AND TABLE_NAME='%sv%hu' "
+             "AND COLUMN_NAME='idx' "
+             "AND (lower(COLUMN_TYPE) NOT LIKE 'bigint%%' OR EXTRA != 'auto_increment') ",
+             ntop->getPrefs()->get_mysql_dbname(),
+             ntop->getPrefs()->get_mysql_tablename(),
+             ipvers[i]);
+    if(exec_sql_query(&mysql, sql, true, true) > 0){
+      // if here we have to convert the type to unsigned
+      ntop->getTrace()->traceEvent(TRACE_NORMAL,
+                                   "MySQL schema update. Altering table %sv%hu: "
+                                   "changing idx data type to bigint.",
+                                   ntop->getPrefs()->get_mysql_tablename(),
+                                   ipvers[i]);
 
-  if(set_db_created)
-    db_created = true;
-  return true;
-}
-
-/* ******************************************* */
-
-bool MySQLDB::createNprobeDBView() {
-  char sql[CONST_MAX_SQL_QUERY_LEN];
-  const u_int16_t ipvers[2] = {4, 6};
-  u_int16_t i = 0;
-
-  if(mysql_select_db(&mysql, ntop->getPrefs()->get_mysql_dbname())) {
-    goto err;
-    return false;
-  }
-
-  for(; i < sizeof(ipvers) / sizeof(u_int16_t); i++){
-    snprintf(sql, sizeof(sql), MYSQL_DROP_NPROBE_VIEW, ipvers[i]);
-
-    ntop->getTrace()->traceEvent(TRACE_INFO,
-				 "Deleting existing nProbe views for IPV%hu:\n"
-				 "[%s]",
-				 ipvers[i],
-				 sql);
-
-    if(exec_sql_query(&mysql, sql, true) < 0)
-      goto err;
-
-    snprintf(sql, sizeof(sql), MYSQL_CREATE_NPROBE_VIEW,
-	     ipvers[i], ipvers[i], ipvers[i], iface->get_id(),
-	     ntop->getPrefs()->get_mysql_tablename(),
-	     ipvers[i]);
-
-    ntop->getTrace()->traceEvent(TRACE_INFO,
-				 "Creating nProbe view on table %sflows for IPV%hu:\n"
-				 "[%s]",
-				 ntop->getPrefs()->get_mysql_tablename(),
-				 ipvers[i],
-				 sql);
-
-    if(exec_sql_query(&mysql, sql, true) < 0) {
-    err:
-      ntop->getTrace()->traceEvent(TRACE_ERROR, "MySQL error: %s\n", get_last_db_error(&mysql));
-      ntop->getTrace()->traceEvent(TRACE_ERROR, "Before starting ntopng, make sure to start nprobe with option --mysql and template @NTOPNG@.");
-      ntop->getTrace()->traceEvent(TRACE_ERROR, "Example:");
-      ntop->getTrace()->traceEvent(TRACE_ERROR, "./nprobe -i eno1 -T \"@NTOPNG@\" --mysql=\"localhost:ntopng:nf:root:root\" --zmq \"tcp://127.0.0.1:5556\" --zmq-probe-mode");
-      ntop->getTrace()->traceEvent(TRACE_ERROR, "./ntopng  -i \"tcp://*:5556c\" -F \"mysql-nprobe;localhost;ntopng;nf;root;root\"");
-      return false;
+      snprintf(sql, sizeof(sql),
+               "ALTER TABLE `%sv%hu` MODIFY COLUMN `idx` bigint NOT NULL AUTO_INCREMENT",
+               ntop->getPrefs()->get_mysql_tablename(), ipvers[i]);
+      exec_sql_query(&mysql, sql, true, true);
     }
-
   }
 
   db_created = true;
@@ -473,12 +463,11 @@ bool MySQLDB::createNprobeDBView() {
 /* ******************************************* */
 
 MySQLDB::MySQLDB(NetworkInterface *_iface) : DB(_iface) {
-  mysqlDroppedFlows = mysqlConsumerDroppedFlows = 0;
   mysqlEnqueuedFlows = 0;
-  mysqlExportedFlows = 0, mysqlLastExportedFlows = 0;
-  mysqlExportRate = 0;
-  checkpointDroppedFlows = checkpointExportedFlows = 0;
-  lastUpdateTime.tv_sec = 0, lastUpdateTime.tv_usec = 0;
+  log_fd = NULL;
+  open_log();
+  db_created = false;
+
   connectToDB(&mysql, false);
 }
 
@@ -487,13 +476,48 @@ MySQLDB::MySQLDB(NetworkInterface *_iface) : DB(_iface) {
 MySQLDB::~MySQLDB() {
   shutdown();
   disconnectFromDB(&mysql);
+
+  if(log_fd) fclose(log_fd);
 }
 
 /* ******************************************* */
 
-void MySQLDB::startDBLoop() {
-  running = true;
+void MySQLDB::open_log() {
+  char sql_log_path[512];
+
+  if(ntop->getPrefs()->is_sql_log_enabled()) {
+    snprintf(sql_log_path, sizeof(sql_log_path)-1, "%s/%d/ntopng_sql.log",
+	     ntop->get_working_dir(), iface->get_id());
+
+    log_fd = fopen(sql_log_path, "a");
+
+    if(!log_fd)
+      ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to create log %s", sql_log_path);
+    else
+      chmod(sql_log_path, CONST_DEFAULT_FILE_MODE);
+  }
+}
+
+/* ******************************************* */
+
+bool MySQLDB::startQueryLoop() {
+  /*
+    If mysql flows dump is enabled, then it is necessary to create
+    and update the database schema. This must be executed only once.
+   */
+  if(!db_created) {
+    if(ntop->getPrefs()->do_dump_flows_on_mysql()) {
+      if(!createDBSchema()){
+	ntop->getTrace()->traceEvent(TRACE_ERROR,
+				     "Unable to create database schema, quitting.");
+	exit(EXIT_FAILURE);
+      }
+    }
+  }
+
   pthread_create(&queryThreadLoop, NULL, ::queryLoop, (void*)this);
+
+  return(true);
 }
 
 /* ******************************************* */
@@ -503,42 +527,13 @@ void MySQLDB::shutdown() {
     void *res;
 
     DB::shutdown();
+    
     pthread_join(queryThreadLoop, &res);
   }
 }
 
 /* ******************************************* */
 
-void MySQLDB::updateStats(const struct timeval *tv) {
-  if(tv == NULL) return;
-
-  if(lastUpdateTime.tv_sec > 0) {
-    float tdiffMsec = ((float)(tv->tv_sec-lastUpdateTime.tv_sec)*1000)+((tv->tv_usec-lastUpdateTime.tv_usec)/(float)1000);
-    if(tdiffMsec >= 1000) { /* al least one second */
-      u_int64_t diffFlows = mysqlExportedFlows - mysqlLastExportedFlows;
-      mysqlLastExportedFlows = mysqlExportedFlows;
-
-      mysqlExportRate = ((float)(diffFlows * 1000)) / tdiffMsec;
-      if (mysqlExportRate < 0) mysqlExportRate = 0;
-    }
-  }
-
-  memcpy(&lastUpdateTime, tv, sizeof(struct timeval));
-}
-
-/* ******************************************* */
-
-void MySQLDB::lua(lua_State *vm, bool since_last_checkpoint) const {
-  lua_push_uint64_table_entry(vm, "flow_export_count",
-			   mysqlExportedFlows - (since_last_checkpoint ? checkpointExportedFlows : 0));
-  lua_push_int32_table_entry(vm, "flow_export_drops",
-			     (mysqlDroppedFlows + mysqlConsumerDroppedFlows) - (since_last_checkpoint ? checkpointDroppedFlows : 0));
-  lua_push_float_table_entry(vm, "flow_export_rate",
-			     mysqlExportRate >= 0 ? mysqlExportRate : 0);
-}
-
-/* ******************************************* */
-
 char* MySQLDB::escapeAphostrophes(const char *unescaped) {
   char *buf;
   int l, i, j;
@@ -569,17 +564,17 @@ char* MySQLDB::escapeAphostrophes(const char *unescaped) {
 /* ******************************************* */
 
 int MySQLDB::flow2InsertValues(Flow *f, char *json,
-			       char *values_buf, size_t values_buf_len) const {
-  char cli_str[64], srv_str[64], *json_buf, *info_buf;
+			       char *values_buf, size_t values_buf_len) {
+  char cli_str[64], srv_str[64], *json_buf, *info_buf, buf[64];
   u_int32_t packets, first_seen, last_seen;
-  u_int32_t bytes_cli2srv, bytes_srv2cli;
+  u_int64_t bytes_cli2srv, bytes_srv2cli;
   size_t len;
 
   if(!values_buf || !values_buf_len || !f)
     return -1;
 
-  json_buf = escapeAphostrophes(json);
-  info_buf = escapeAphostrophes(f->getFlowInfo());
+  json_buf = escapeAphostrophes((const char*)json);
+  info_buf = escapeAphostrophes((const char*)f->getFlowInfo(buf, sizeof(buf), false));
 
   /* Prevents ERROR 1406 (22001): Data too long for column 'INFO' at row 1 */
   if(info_buf && strlen(info_buf) > 254)
@@ -592,17 +587,17 @@ int MySQLDB::flow2InsertValues(Flow *f, char *json,
   first_seen = f->get_partial_first_seen();
   last_seen = f->get_partial_last_seen();
 
-  if(f->get_cli_host()->get_ip()->isIPv4()) {
+  if(f->get_cli_ip_addr()->isIPv4()) {
     len = snprintf(values_buf, values_buf_len,
 		   MYSQL_INSERT_VALUES_V4,
 		   f->get_vlan_id(),
 		   f->get_detected_protocol().app_protocol,
-		   htonl(f->get_cli_host()->get_ip()->get_ipv4()),
+		   htonl(f->get_cli_ip_addr()->get_ipv4()),
 		   f->get_cli_port(),
-		   htonl(f->get_srv_host()->get_ip()->get_ipv4()),
+		   htonl(f->get_srv_ip_addr()->get_ipv4()),
 		   f->get_srv_port(),
 		   f->get_protocol(),
-		   bytes_cli2srv, bytes_srv2cli,
+		   (uintmax_t)bytes_cli2srv, (uintmax_t)bytes_srv2cli,
 		   packets, first_seen, last_seen,
 		   info_buf ? info_buf : "",
 		   json_buf ? json_buf : "",
@@ -617,12 +612,12 @@ int MySQLDB::flow2InsertValues(Flow *f, char *json,
 		   MYSQL_INSERT_VALUES_V6,
 		   f->get_vlan_id(),
 		   f->get_detected_protocol().app_protocol,
-		   f->get_cli_host()->get_ip()->print(cli_str, sizeof(cli_str)),
+		   f->get_cli_ip_addr()->print(cli_str, sizeof(cli_str)),
 		   f->get_cli_port(),
-		   f->get_srv_host()->get_ip()->print(srv_str, sizeof(srv_str)),
+		   f->get_srv_ip_addr()->print(srv_str, sizeof(srv_str)),
 		   f->get_srv_port(),
 		   f->get_protocol(),
-		   bytes_cli2srv, bytes_srv2cli,
+		   (uintmax_t)bytes_cli2srv, (uintmax_t)bytes_srv2cli,
 		   packets, first_seen, last_seen,
 		   info_buf ? info_buf : "",
 		   json_buf ? json_buf : "",
@@ -644,13 +639,40 @@ int MySQLDB::flow2InsertValues(Flow *f, char *json,
 
 /* ******************************************* */
 
+void MySQLDB::try_exec_sql_query(MYSQL *conn, char *sql) {
+  int rc;
+
+  if (!db_operational) {
+    if(!connectToDB(conn, true)) {
+      _usleep(100);
+      return;
+    }
+  }
+
+  if (strlen(sql) >= CONST_MAX_SQL_QUERY_LEN - 1) {
+    ntop->getTrace()->traceEvent(TRACE_WARNING, "Tried to execute a query longer than %u. Skipping.",
+      CONST_MAX_SQL_QUERY_LEN - 2);
+  } else if((rc = exec_sql_query(conn, sql, true /* Attempt to reconnect */, true /* Don't print errors */, false)) < 0) {
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "MySQL error: %s [rc=%d]", get_last_db_error(conn), rc);
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", sql);
+
+    /* Don't give up, manually re-connect */
+    disconnectFromDB(conn);
+    if(!connectToDB(conn, true)) _usleep(100);
+  } else {
+    incNumExportedFlows();
+  }
+}
+
+/* ******************************************* */
+
 bool MySQLDB::dumpFlow(time_t when, Flow *f, char *json) {
   char sql[CONST_MAX_SQL_QUERY_LEN];
 
-  if((f->get_cli_host() == NULL) || (f->get_srv_host() == NULL) || !MySQLDB::db_created)
+  if((f->get_cli_ip_addr() == NULL) || (f->get_srv_ip_addr() == NULL) || !MySQLDB::db_created)
     return(false);
 
-  if(f->get_cli_host()->get_ip()->isIPv4())
+  if(f->get_cli_ip_addr()->isIPv4())
     snprintf(sql, sizeof(sql), "INSERT INTO `%sv4` " MYSQL_INSERT_FIELDS " VALUES ",
 	     ntop->getPrefs()->get_mysql_tablename());
   else
@@ -660,17 +682,27 @@ bool MySQLDB::dumpFlow(time_t when, Flow *f, char *json) {
   /* do the actual flow insertion as a tuple */
   flow2InsertValues(f, json, &sql[strlen(sql)], sizeof(sql) - strlen(sql) - 1);
 
-  if (ntop->getRedis()->llen(CONST_SQL_QUEUE) < CONST_MAX_MYSQL_QUEUE_LEN) {
-    /* We know that we should have locked before evaluating the condition
+  if (iface->read_from_pcap_dump()) {
+    /*
+     Inserting inline in case of PCAP file as interrupting the datapath
+     is not an issue and also avoids flows drops due to the redis queue
+     maximum length
+    */
+    try_exec_sql_query(&mysql, sql);
+  } else {
+    if (ntop->getRedis()->llen(CONST_SQL_QUEUE) < CONST_MAX_MYSQL_QUEUE_LEN) {
+      /*
+       We know that we should have locked before evaluating the condition
        above as another thread, via the lpush below, can invalidate the condition.
        However, we prefer to avoid an additional lock as the lpush guarantees
        that no more than CONST_MAX_MYSQL_QUEUE_LEN will ever be in the queue.
        The drawback is that the counter mysqlDroppedFlows
        is not guaranteed to be 100% accurate but we can tolerate this.
-    */
-    ntop->getRedis()->lpush(CONST_SQL_QUEUE, sql, CONST_MAX_MYSQL_QUEUE_LEN);
-  } else {
-    mysqlDroppedFlows++;
+      */
+      ntop->getRedis()->lpush(CONST_SQL_QUEUE, sql, CONST_MAX_MYSQL_QUEUE_LEN);
+    } else {
+      incNumDroppedFlows();
+    }
   }
 
   return(true);
@@ -679,69 +711,187 @@ bool MySQLDB::dumpFlow(time_t when, Flow *f, char *json) {
 /* ******************************************* */
 
 void MySQLDB::disconnectFromDB(MYSQL *conn) {
-  mysql_close(conn);
-  ntop->getTrace()->traceEvent(TRACE_NORMAL, "Disconnected from MySQL for interface %s...",
-			       iface->get_name() ? iface->get_name() : "<unknown>");
+  if(conn != NULL) {
+    mysql_close(conn);
+    ntop->getTrace()->traceEvent(TRACE_NORMAL, "Disconnected from MySQL for interface %s...",
+				 iface->get_name() ? iface->get_name() : "<unknown>");
+  }
+}
+
+/* ******************************************* */
+
+MYSQL* MySQLDB::mysql_try_connect(MYSQL *conn, const char *dbname) {
+  MYSQL *rc;
+  unsigned long flags = CLIENT_COMPRESS;
+  char *host = ntop->getPrefs()->get_mysql_host();
+  char *user = ntop->getPrefs()->get_mysql_user();
+  u_int port = ntop->getPrefs()->get_mysql_port();
+
+  if(!host)
+    return(NULL);
+
+  if((!ntop->getPrefs()->do_dump_flows_on_clickhouse()) && (host[0] == '/') /* Use socketD */)
+    rc = mysql_real_connect(conn,
+			    NULL, /* Host */
+			    user,
+			    ntop->getPrefs()->get_mysql_pw(),
+			    dbname,
+			    0, host /* socket */,
+			    flags);
+  else {
+    ntop->getTrace()->traceEvent(TRACE_INFO, "ClickHouse Connecting to %s:%u [user: %s][db: %s]",
+				 host,
+				 port,
+				 user,
+				 dbname ? dbname : "");
+    rc = mysql_real_connect(conn,
+			    host,
+			    user,
+			    ntop->getPrefs()->get_mysql_pw(),
+			    dbname,
+			    port,
+			    NULL /* socket */,
+			    flags);
+  }
+
+  return(rc);
+}
+
+/* ******************************************* */
+
+void MySQLDB::mysql_result_to_lua(lua_State *vm, MYSQL_RES *result,
+				  int num_fields, bool limitRows) {
+  MYSQL_ROW row;
+  char *fields[MYSQL_MAX_NUM_FIELDS] = { NULL };
+  int num = 0;
+
+  num_fields = min_val(num_fields, MYSQL_MAX_NUM_FIELDS);
+
+  lua_newtable(vm);
+
+  while((row = mysql_fetch_row(result))) {
+    lua_newtable(vm);
+
+    if(num == 0) {
+      for(int i = 0; i < num_fields; i++) {
+	MYSQL_FIELD *field = mysql_fetch_field(result);
+
+	fields[i] = field->name;
+      }
+    }
+
+    for(int i = 0; i < num_fields; i++) {
+#ifdef QUERY_TRACE
+      ntop->getTrace()->traceEvent(TRACE_NORMAL, "[row %u/fields %u] %s=%s", num+1, num_fields, fields[i], row[i] ? row[i] : (char*)"");
+#endif
+      lua_push_str_table_entry(vm, (const char*)fields[i], row[i] ? row[i] : (char*)"");
+    }
+
+    lua_pushinteger(vm, ++num);
+    lua_insert(vm, -2);
+    lua_settable(vm, -3);
+
+    if(num > MYSQL_MAX_NUM_ROWS) {
+      static bool warning_shown = false;
+      if(!warning_shown) {
+	ntop->getTrace()->traceEvent(TRACE_WARNING,
+				     "Too many results returned from MySQL, reduce query result set");
+	warning_shown = true;
+      }
+    }
+
+    if(limitRows && num >= MYSQL_MAX_NUM_ROWS)
+      break;
+  }
 }
 
 /* ******************************************* */
 
 bool MySQLDB::connectToDB(MYSQL *conn, bool select_db) {
   MYSQL *rc;
-  unsigned long flags = CLIENT_COMPRESS;
   char *dbname = select_db ? ntop->getPrefs()->get_mysql_dbname() : NULL;
+  char *host = ntop->getPrefs()->get_mysql_host();
+  char *user = ntop->getPrefs()->get_mysql_user();
+  u_int port = ntop->getPrefs()->get_mysql_port();
 
   db_operational = false;
 
-  ntop->getTrace()->traceEvent(TRACE_NORMAL, "Attempting to connect to MySQL for interface %s...",
+  if((host == NULL) || (user == NULL)) {
+    ntop->getTrace()->traceEvent(TRACE_WARNING, "%s not configured: connection failed", getEngineName());
+    return(db_operational);
+  }
+  
+  ntop->getTrace()->traceEvent(TRACE_INFO, "Attempting to connect to %s for interface %s...",getEngineName(),
 			       iface->get_name());
 
-  if(m) m->lock(__FILE__, __LINE__);
+  m.lock(__FILE__, __LINE__);
 
   if(mysql_init(conn) == NULL) {
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "Failed to initialize MySQL connection");
-    if(m) m->unlock(__FILE__, __LINE__);
-    return(false);
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "Failed to initialize %s connection", getEngineName());
+    m.unlock(__FILE__, __LINE__);
+    return(db_operational);
   }
 
-  if(ntop->getPrefs()->get_mysql_host()[0] == '/') /* Use socketD */
-    rc = mysql_real_connect(conn,
-			    NULL, /* Host */
-			    ntop->getPrefs()->get_mysql_user(),
-			    ntop->getPrefs()->get_mysql_pw(),
-			    dbname,
-			    0, ntop->getPrefs()->get_mysql_host() /* socket */,
-			    flags);
-  else
-    rc = mysql_real_connect(conn,
-			    ntop->getPrefs()->get_mysql_host(),
-			    ntop->getPrefs()->get_mysql_user(),
-			    ntop->getPrefs()->get_mysql_pw(),
-			    dbname,
-			    ntop->getPrefs()->get_mysql_port(),
-			    NULL /* socket */, flags);
+  rc = mysql_try_connect(conn, dbname);
 
   if(rc == NULL) {
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "Failed to connect to MySQL: %s [%s@%s:%i]\n",
-				 mysql_error(conn),
-				 ntop->getPrefs()->get_mysql_user(),
-				 ntop->getPrefs()->get_mysql_host(),
-                 ntop->getPrefs()->get_mysql_port());
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "Failed to connect to %s: %s [%s@%s:%i]\n", getEngineName(),
+				 mysql_error(conn), user, host, port);
 
-    if(m) m->unlock(__FILE__, __LINE__);
-    return(false);
+    m.unlock(__FILE__, __LINE__);
+    return(db_operational);
   }
 
   db_operational = true;
 
-  ntop->getTrace()->traceEvent(TRACE_NORMAL, "Successfully connected to MySQL [%s@%s:%i] for interface %s",
-			       ntop->getPrefs()->get_mysql_user(),
-			       ntop->getPrefs()->get_mysql_host(),
-			       ntop->getPrefs()->get_mysql_port(),
-			       iface->get_name());
+  ntop->getTrace()->traceEvent(TRACE_NORMAL, "Successfully connected to %s [%s@%s:%i][dbname: %s] for interface %s",
+			       getEngineName(), user, host, port, dbname ? dbname : "", iface->get_name());
 
-  if(m) m->unlock(__FILE__, __LINE__);
-  return(true);
+  m.unlock(__FILE__, __LINE__);
+  return(db_operational);
+}
+
+/* ******************************************* */
+
+int MySQLDB::exec_single_query(lua_State *vm, char *sql) {
+  MYSQL conn;
+  MYSQL_RES *result;
+  bool result_ok = false;
+  struct timeval begin, end;
+
+  if(enable_db_traces) {
+    ntop->getTrace()->traceEvent(TRACE_NORMAL, "%s", sql);
+    gettimeofday(&begin, NULL);
+  }
+
+  if(mysql_init(&conn) != NULL) {
+    if((mysql_try_connect(&conn, NULL /* no db */) != NULL)
+       && (mysql_query(&conn, sql) == 0)
+       && ((result = mysql_store_result(&conn)) != NULL)) {
+      int num_fields = mysql_field_count(&conn);
+      
+      if(num_fields != 0) {	
+	mysql_result_to_lua(vm, result, num_fields, false);
+	result_ok = true;
+      }
+
+      mysql_free_result(result);
+    }
+
+    mysql_close(&conn);
+  }
+
+  if(enable_db_traces) {
+    gettimeofday(&end, NULL);
+    ntop->getTrace()->traceEvent(TRACE_NORMAL, "Query completed in %.3f sec", Utils::usecTimevalDiff(&end, &begin)/1000000.);
+  }
+
+  if(!result_ok) {
+    lua_pushnil(vm);
+    return(-1);
+  }
+
+  return(0);
 }
 
 /* ******************************************* */
@@ -755,25 +905,34 @@ int MySQLDB::exec_sql_query(MYSQL *conn, const char *sql,
 			    bool doLock) {
   int rc;
   MYSQL_RES *result;
+  struct timeval begin, end;
 
-  /* Don't check db_created here. This method is private
+  if(enable_db_traces) {
+    ntop->getTrace()->traceEvent(TRACE_NORMAL, "%s", sql);
+    gettimeofday(&begin, NULL);
+  }
+
+  /*
+     Don't check db_created here. This method is private
      so hopefully we know what we're doing.
    */
-  if(!db_operational)
-    return(-2);
+  if(!db_operational) {
+    if(!connectToDB(conn, true))
+      return(-3);
+  }
 
-  if(doLock && m) m->lock(__FILE__, __LINE__);
+  if(doLock) m.lock(__FILE__, __LINE__);
+  
   if((rc = mysql_query(conn, sql)) != 0) {
     if(!ignoreErrors)
-      ntop->getTrace()->traceEvent(TRACE_ERROR, "MySQL error: [%s][%s]",
-				   get_last_db_error(conn), sql);
+      printFailure(sql, rc);
 
     switch(mysql_errno(conn)) {
     case CR_SERVER_GONE_ERROR:
     case CR_SERVER_LOST:
       if(doReconnect) {
 	disconnectFromDB(conn);
-	if(doLock && m) m->unlock(__FILE__, __LINE__);
+	if(doLock) m.unlock(__FILE__, __LINE__);
 
 	connectToDB(conn, true);
 
@@ -803,25 +962,43 @@ int MySQLDB::exec_sql_query(MYSQL *conn, const char *sql,
     }
   }
 
-  if(doLock && m) m->unlock(__FILE__, __LINE__);
+  if(enable_db_traces) {
+    gettimeofday(&end, NULL);
+    ntop->getTrace()->traceEvent(TRACE_NORMAL, "Query completed in %.3f sec",
+				 Utils::usecTimevalDiff(&end, &begin)/1000000.);
+  }
+
+  if(doLock) m.unlock(__FILE__, __LINE__);
 
   return(rc);
 }
 
 /* ******************************************* */
 
-int MySQLDB::exec_sql_query(lua_State *vm, char *sql, bool limitRows, bool wait_for_db_created) {
+int MySQLDB::exec_sql_query(lua_State *vm, char *sql, bool limitRows,
+			    bool wait_for_db_created) {
   MYSQL_RES *result;
-  MYSQL_ROW row;
-  char *fields[MYSQL_MAX_NUM_FIELDS] = { NULL };
-  int num_fields, rc, num = 0;
+  int rc, num_fields;
+  struct timeval begin, end;
 
-  if((wait_for_db_created && !MySQLDB::db_created /* Make sure the db exists before doing queries */)
-     || !db_operational)
-    return(-2);
+  if(!ntop->getPrefs()->do_dump_flows_on_clickhouse()) {
+    if(wait_for_db_created && (!db_created /* Make sure the db exists before doing queries */)) {
+      ntop->getTrace()->traceEvent(TRACE_NORMAL, "Unable to perform query: database being created");
+      return(-2);
+    }
+  }
 
-  if(m) m->lock(__FILE__, __LINE__);
+  if(!db_operational) {
+    if(!connectToDB(&mysql, true))
+      return(-3);
+  }
 
+  if(enable_db_traces) {
+    ntop->getTrace()->traceEvent(TRACE_NORMAL, "%s", sql);
+    gettimeofday(&begin, NULL);
+  }
+
+  m.lock(__FILE__, __LINE__);
 
   if(ntop->getPrefs()->is_sql_log_enabled() && log_fd && sql) {
 #ifndef WIN32
@@ -837,17 +1014,122 @@ int MySQLDB::exec_sql_query(lua_State *vm, char *sql, bool limitRows, bool wait_
     fprintf(log_fd, "%s\n", sql);
     fflush(log_fd);
   }
+
+#ifdef QUERY_TRACE
+  ntop->getTrace()->traceEvent(TRACE_NORMAL, "%s", sql);
+#endif
   
   if((rc = mysql_query(&mysql, sql)) != 0) {
     /* retry */
     disconnectFromDB(&mysql);
-    if(m) m->unlock(__FILE__, __LINE__);
+    m.unlock(__FILE__, __LINE__);
     connectToDB(&mysql, true);
 
     if(!db_operational)
       return(-2);
 
-    if(m) m->lock(__FILE__, __LINE__);
+    m.lock(__FILE__, __LINE__);
+    rc = mysql_query(&mysql, sql);
+  }
+
+  if(rc == 0) {
+    if((result = mysql_store_result(&mysql)) == NULL) {
+      if(mysql_field_count(&mysql) == 0) {
+	/* If mysql_field_count() == 0 this was an INSERT so it's normal that tha result is NULL */
+	if(vm) lua_pushnil(vm);
+	m.unlock(__FILE__, __LINE__);
+	return(0);
+      } else
+	rc = -1;
+    } else {
+#ifdef QUERY_TRACE
+      ntop->getTrace()->traceEvent(TRACE_NORMAL, "[%p] %u row(s) found", &mysql, mysql_num_rows(result));
+#endif
+      
+      num_fields = mysql_field_count(&mysql);
+
+      if(num_fields == 0)
+	rc = -2;
+    }
+  }
+
+  if(rc != 0) {
+    if(vm) lua_pushstring(vm, get_last_db_error(&mysql));
+    printFailure(sql, rc); 
+    m.unlock(__FILE__, __LINE__);
+    return(rc);
+  }
+
+#ifdef QUERY_TRACE
+  ntop->getTrace()->traceEvent(TRACE_NORMAL, "num_fields=%d", num_fields);
+#endif
+  
+  if(enable_db_traces) {
+    gettimeofday(&end, NULL);
+    ntop->getTrace()->traceEvent(TRACE_NORMAL, "Query completed in %.3f sec", Utils::usecTimevalDiff(&end, &begin)/1000000.);
+  }
+
+  if((result == NULL) || (num_fields == 0)) {
+    if(vm) lua_pushnil(vm);
+  } else {
+    if(vm) mysql_result_to_lua(vm, result, num_fields, limitRows);
+    mysql_free_result(result);
+  }
+
+  m.unlock(__FILE__, __LINE__);
+
+  return(0);
+}
+
+/* ******************************************* */
+
+int MySQLDB::exec_quick_sql_query(char *sql, char *out, u_int out_len) {
+  MYSQL_RES *result;
+  int rc;
+  struct timeval begin, end;
+
+  out[0] = '\0';
+
+  if(!db_created /* Make sure the db exists before doing queries */)
+    return(-2);
+
+  if(!db_operational) {
+    if(!connectToDB(&mysql, true))
+      return(-3);
+  }
+
+  if(enable_db_traces) {
+    ntop->getTrace()->traceEvent(TRACE_NORMAL, "%s", sql);
+    gettimeofday(&begin, NULL);
+  }
+
+  m.lock(__FILE__, __LINE__);
+
+  if(ntop->getPrefs()->is_sql_log_enabled() && log_fd && sql) {
+#ifndef WIN32
+    char log_date[32];
+    time_t log_time = time(NULL);
+    struct tm result;
+
+    strftime(log_date, sizeof(log_date),
+	     "%d/%b/%Y %H:%M:%S", localtime_r(&log_time, &result));
+    fprintf(log_fd, "%s ", log_date);
+#endif
+
+    fprintf(log_fd, "%s\n", sql);
+    fflush(log_fd);
+  }
+
+  if((rc = mysql_query(&mysql, sql)) != 0) {
+    /* retry */
+    disconnectFromDB(&mysql);
+    m.unlock(__FILE__, __LINE__);
+    connectToDB(&mysql, true);
+
+    if(!db_operational)
+      return(-2);
+
+    m.lock(__FILE__, __LINE__);
     rc = mysql_query(&mysql, sql);
   }
 
@@ -856,60 +1138,49 @@ int MySQLDB::exec_sql_query(lua_State *vm, char *sql, bool limitRows, bool wait_
 	 && mysql_field_count(&mysql) != 0 /* mysql_store_result() returned nothing; should it have? */)) {
     rc = mysql_errno(&mysql);
 
-    if(rc) {
-      ntop->getTrace()->traceEvent(TRACE_ERROR, "MySQL error: [%s][%d][%s]",
-				   get_last_db_error(&mysql), rc, sql);
-      lua_pushstring(vm, get_last_db_error(&mysql));
-    } else {
-      lua_pushnil(vm);
-    }
+    if(rc)
+      printFailure(sql, rc); 
 
-    if(m) m->unlock(__FILE__, __LINE__);
+    m.unlock(__FILE__, __LINE__);
     return(rc);
   }
 
+  if(enable_db_traces) {
+    gettimeofday(&end, NULL);
+    ntop->getTrace()->traceEvent(TRACE_NORMAL, "Query completed in %.3f sec", Utils::usecTimevalDiff(&end, &begin)/1000000.);
+  }
+
   if((result == NULL) || (mysql_field_count(&mysql) == 0)) {
-    lua_pushnil(vm);
+    ;
   } else {
-    num_fields = min_val(mysql_num_fields(result), MYSQL_MAX_NUM_FIELDS);
-    lua_newtable(vm);
+    MYSQL_ROW row = mysql_fetch_row(result);
 
-    num = 0;
-    while((row = mysql_fetch_row(result))) {
-      lua_newtable(vm);
+    if(row != NULL) {
+      u_int num_fields = mysql_num_fields(result);
 
-      if(num == 0) {
-	for(int i = 0; i < num_fields; i++) {
-	  MYSQL_FIELD *field = mysql_fetch_field(result);
-
-	  fields[i] = field->name;
-	}
-      }
-
-      for(int i = 0; i < num_fields; i++)
-	lua_push_str_table_entry(vm, (const char*)fields[i], row[i] ? row[i] : (char*)"");
-
-      lua_pushinteger(vm, ++num);
-      lua_insert(vm, -2);
-      lua_settable(vm, -3);
-
-      if(num > MYSQL_MAX_NUM_ROWS) {
-	static bool warning_shown = false;
-	if(!warning_shown) {
-	  ntop->getTrace()->traceEvent(TRACE_WARNING, "Too many results returned from MySQL, reduce query result set");
-	  warning_shown = true;
-	}
-      }
-
-      if(limitRows && num >= MYSQL_MAX_NUM_ROWS) break;
+      if(num_fields > 0)
+	snprintf(out, out_len, "%s", row[0 /* first field */]);
     }
 
     mysql_free_result(result);
   }
-  
-  if(m) m->unlock(__FILE__, __LINE__);
+
+  m.unlock(__FILE__, __LINE__);
 
   return(0);
 }
 
+/* ******************************************* */
+
+int MySQLDB::select_database(char *dbname) {
+  int rc = mysql_select_db(&mysql, (const char*)dbname);
+
+  if(rc) {
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "DB select error: [%s][%d][%s]",
+				 get_last_db_error(&mysql), rc, dbname);    
+  }
+  
+  return(rc);
+}
+
 #endif
