@@ -23,13 +23,14 @@
 
 /* *************************************** */
 
-NetworkStats::NetworkStats(NetworkInterface *iface, u_int8_t _network_id) : NetworkStatsAlertableEntity(iface), GenericTrafficElement(), Score(iface) {
+NetworkStats::NetworkStats(NetworkInterface *iface, u_int16_t _network_id) : InterfaceMemberAlertableEntity(iface, alert_entity_network), GenericTrafficElement(), Score(iface) {
   const char *netname;
   network_id = _network_id;
-  numHosts = 0;
+  numHosts = 0, alerted_flows_as_client = alerted_flows_as_server = 0;
   syn_recvd_last_min = synack_sent_last_min = 0;
 
 #ifdef NTOPNG_PRO
+  network_matrix = (InOutTraffic *) calloc(ntop->getNumLocalNetworks(), sizeof(InOutTraffic));
   nextMinPeriodicUpdate = 0;
 
   score_behavior = NULL;
@@ -74,6 +75,7 @@ bool NetworkStats::match(const AddressTree * const tree) const {
 
 NetworkStats::~NetworkStats() {
 #ifdef NTOPNG_PRO
+  if(network_matrix) free(network_matrix);
   if(score_behavior) delete(score_behavior);
   if(traffic_tx_behavior) delete(traffic_tx_behavior);
   if(traffic_rx_behavior) delete(traffic_rx_behavior);
@@ -102,7 +104,29 @@ void NetworkStats::lua(lua_State* vm, bool diff) {
   lua_insert(vm, -2);
   lua_settable(vm, -3);
 
+  lua_newtable(vm);
+  lua_push_uint64_table_entry(vm, "as_client", getTotalAlertedNumFlowsAsClient());
+  lua_push_uint64_table_entry(vm, "as_server", getTotalAlertedNumFlowsAsServer());
+  lua_push_uint64_table_entry(vm, "total", getTotalAlertedNumFlowsAsClient() + getTotalAlertedNumFlowsAsServer());
+  lua_pushstring(vm, "alerted_flows");
+  lua_insert(vm, -2);
+  lua_settable(vm, -3);
+
 #ifdef NTOPNG_PRO
+  lua_newtable(vm);
+  
+  for(u_int16_t i = 0; i < ntop->getNumLocalNetworks(); i++) {
+    lua_newtable(vm);
+    lua_push_uint64_table_entry(vm, "bytes_sent", network_matrix[i].bytes_sent);
+    lua_push_uint64_table_entry(vm, "bytes_rcvd", network_matrix[i].bytes_rcvd);
+    lua_pushstring(vm, ntop->getLocalNetworkName(i));
+    lua_insert(vm, -2);
+    lua_settable(vm, -3);
+  }
+  lua_pushstring(vm, "intranet_traffic");
+  lua_insert(vm, -2);
+  lua_settable(vm, -3);
+
   if(traffic_rx_behavior)
     traffic_rx_behavior->luaBehavior(vm, "traffic_rx_behavior", diff ? NETWORK_BEHAVIOR_REFRESH : 0);
   if(traffic_tx_behavior)
@@ -203,6 +227,28 @@ void NetworkStats::updateStats(const struct timeval *tv)  {
 
 /* ***************************************** */
 
+void NetworkStats::incTrafficBetweenNets(u_int16_t net_id, u_int32_t bytes_sent, u_int32_t bytes_rcvd) { 
+#ifdef NTOPNG_PRO
+  if(net_id < ntop->getNumLocalNetworks() && net_id != (u_int16_t) -1) {
+    network_matrix[net_id].bytes_sent += bytes_sent; 
+    network_matrix[net_id].bytes_rcvd += bytes_rcvd; 
+  }
+#endif
+}
+
+/* ***************************************** */
+
+void NetworkStats::resetTrafficBetweenNets() { 
+#ifdef NTOPNG_PRO
+  for(u_int16_t i = 0; i < ntop->getNumLocalNetworks(); i++) {
+    network_matrix[i].bytes_sent = 0; 
+    network_matrix[i].bytes_rcvd = 0;
+  }
+#endif
+}
+
+/* ***************************************** */
+
 void NetworkStats::updateBehaviorStats(const struct timeval *tv) {
   /* 5 Min Update */
   if(tv->tv_sec >= nextMinPeriodicUpdate) {
