@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,9 +25,8 @@
 
 NtopGlobals::NtopGlobals() {
   start_time = time(NULL);
-  ifMTU = snaplen = 1514;
   file_id = 0;
-  trace = new Trace();  
+  trace = new (std::nothrow) Trace();  
   is_shutdown = shutdown_requested = false, do_decode_tunnels = true;
 };
 
