@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -34,6 +34,7 @@ pfring *PF_RINGInterface::pfringSocketInit(const char *name) {
   pfring *handle;
   packet_direction direction;
   u_int32_t version;
+  int rc;
 
   flags |= PF_RING_LONG_HEADER;
   flags |= PF_RING_DNA_SYMMETRIC_RSS;  /* Note that symmetric RSS is ignored by non-DNA drivers */
@@ -46,27 +47,48 @@ pfring *PF_RINGInterface::pfringSocketInit(const char *name) {
   else if(ntop->getPrefs()->are_vss_apcon_timestamps_enabled())
     flags |= PF_RING_VSS_APCON_TIMESTAMP;
 
-  if((handle = pfring_open(name, ntop->getGlobals()->getSnaplen(), flags)) == NULL)
+  if((handle = pfring_open(name, ntop->getGlobals()->getSnaplen(name), flags)) == NULL)
     return NULL;
 
-  pfring_version(handle, &version);
+  rc = pfring_version(handle, &version);
+
+  rc = pfring_version(handle, &version);
+
+  if (rc == -1) {
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "Unexpected socket error: unable to read PF_RING version");
+    exit(0);
+  }
+
+  if (version != RING_VERSION_NUM) {
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "PF_RING version mismatch (expected v.%d.%d.%d found v.%d.%d.%d)",
+      (RING_VERSION_NUM & 0xFFFF0000) >> 16, (RING_VERSION_NUM & 0x0000FF00) >> 8, RING_VERSION_NUM & 0x000000FF,
+      (version & 0xFFFF0000) >> 16, (version & 0x0000FF00) >> 8, version & 0x000000FF); 
+    exit(0);
+  }
+
   ntop->getTrace()->traceEvent(TRACE_NORMAL, "Reading packets from PF_RING v.%d.%d.%d interface %s...",
 			       (version & 0xFFFF0000) >> 16, (version & 0x0000FF00) >> 8, version & 0x000000FF,
 			       name);
 
+  pfring_get_bound_device_address(handle, ifMac);
   pfring_set_poll_watermark(handle, 8);
   pfring_set_application_name(handle, (char*)"ntopng");
-  pfring_enable_rss_rehash(handle);
+
+  if (ntop->getPrefs()->hasPF_RINGClusterID())
+    pfring_set_cluster(handle, ntop->getPrefs()->getPF_RINGClusterID(), cluster_per_flow_5_tuple);
+  else
+    pfring_enable_rss_rehash(handle);
   
   switch(ntop->getPrefs()->getCaptureDirection()) {
   case PCAP_D_INOUT: direction = rx_and_tx_direction; break;
   case PCAP_D_IN:    direction = rx_only_direction;   break;
   case PCAP_D_OUT:   direction = tx_only_direction;   break;
+  default:           direction = rx_and_tx_direction; break;
   }
 
   if(pfring_set_direction(handle, direction) != 0) {
     ntop->getTrace()->traceEvent(TRACE_WARNING, "Unable to set packet capture direction on %s", name);
-    if (strstr(name, "zc:") && direction != rx_only_direction)
+    if(strstr(name, "zc:") && direction != rx_only_direction)
       ntop->getTrace()->traceEvent(TRACE_WARNING, "ZC supports RX capture only, please use --capture-direction 1"); 
   }
 
@@ -86,39 +108,50 @@ pfring *PF_RINGInterface::pfringSocketInit(const char *name) {
 
 /* **************************************************** */
 
-PF_RINGInterface::PF_RINGInterface(const char *name) : NetworkInterface(name) {
+PF_RINGInterface::PF_RINGInterface(const char *_name) : NetworkInterface(_name) {
+  char name[MAX_INTERFACE_NAME_LEN];
+  int err;
 
   num_pfring_handles = 0;
   pcap_datalink_type = DLT_EN10MB;
   dropped_packets = 0;
 
-  if (strchr(ifname, ':') && strchr(ifname, ',')) { 
-    char name_list[MAX_INTERFACE_NAME_LEN];
-    char *name, *tmp;
+  strncpy(name, ifname, sizeof(name));
+  name[sizeof(name) - 1] = '\0';
+
+  if(strchr(name, ':') && strchr(name, ',')) { 
+    char *item_name, *tmp;
 
     /* This looks like a list of ZC interfaces, aggregation need to be done here */
+    item_name = strtok_r(name, ",", &tmp);
+    while (item_name != NULL && num_pfring_handles < PF_RING_MAX_SOCKETS) {
 
-    strncpy(name_list, ifname, sizeof(name_list));
-    name_list[sizeof(name_list) - 1] = '\0';
-    name = strtok_r(name_list, ",", &tmp);
-    while (name != NULL && num_pfring_handles < PF_RING_MAX_SOCKETS) {
+      pfring_handle[num_pfring_handles] = pfringSocketInit(item_name);
 
-      pfring_handle[num_pfring_handles] = pfringSocketInit(name);
-
-      if (pfring_handle[num_pfring_handles] == NULL)
-        throw errno; 
+      if(pfring_handle[num_pfring_handles] == NULL) {
+        err = errno;
+        throw err; 
+      }
      
       num_pfring_handles++;
 
-      name = strtok_r(NULL, ",", &tmp);
+      item_name = strtok_r(NULL, ",", &tmp);
     }
 
   } else {
 
-    pfring_handle[0] = pfringSocketInit(ifname);
+    if (ntop->getPrefs()->hasPF_RINGClusterID() && !strchr(name, ':')) {
+      /* PF_RING cluster configured, cleaning up queue from the interface name */
+      char *at = strchr(name, '@');
+      if (at) *at = '\0';
+    }
 
-    if (pfring_handle[0] == NULL)
-      throw errno; 
+    pfring_handle[0] = pfringSocketInit(name);
+
+    if(pfring_handle[0] == NULL) {
+      err = errno;
+      throw err; 
+    }
 
     num_pfring_handles = 1;
   }
@@ -132,7 +165,7 @@ PF_RINGInterface::~PF_RINGInterface() {
   shutdown();
 
   for (i = 0; i < num_pfring_handles; i++) {
-    if (pfring_handle[i])
+    if(pfring_handle[i])
       pfring_close(pfring_handle[i]);
   }
 }
@@ -171,7 +204,7 @@ void PF_RINGInterface::singlePacketPollLoop() {
       }
     } else {
       if(sleep_time < max_sleep) sleep_time += step_sleep;
-      usleep(sleep_time);
+      _usleep(sleep_time);
       purgeIdle(time(NULL));
     }
   }
@@ -203,8 +236,7 @@ void PF_RINGInterface::multiPacketPollLoop() {
 
         if(hdr.ts.tv_sec == 0) gettimeofday(&hdr.ts, NULL);
 	dissectPacket(DUMMY_BRIDGE_INTERFACE_ID,
-		      (hdr.extended_hdr.rx_direction == 1) ? 
-		      true /* ingress */ : false /* egress */,
+		      (idx == 0) /* Assuming 0 ingress, 1 egress (TAP) */,
 		      NULL, (const struct pcap_pkthdr *) &hdr, buffer,
 		      &p, &srcHost, &dstHost, &flow);
 	sleep_time = step_sleep;
@@ -218,7 +250,7 @@ void PF_RINGInterface::multiPacketPollLoop() {
       }
     } else {
       if(sleep_time < max_sleep) sleep_time += step_sleep;
-      usleep(sleep_time);
+      _usleep(sleep_time);
       purgeIdle(time(NULL));
     }
 
@@ -232,15 +264,19 @@ static void* packetPollLoop(void* ptr) {
   PF_RINGInterface *iface = (PF_RINGInterface *) ptr;
   
   /* Wait until the initialization completes */
-  while(!iface->isRunning()) sleep(1);
+  while(iface->isStartingUp()) sleep(1);
 
-  while(iface->idle()) { iface->purgeIdle(time(NULL)); sleep(1); }
-
-  if (iface->get_num_pfring_handles() == 1)
+  while(iface->idle()) {
+    if(ntop->getGlobals()->isShutdown()) return(NULL);
+    iface->purgeIdle(time(NULL));
+    sleep(1);
+  }
+  
+  if(iface->get_num_pfring_handles() == 1)
     iface->singlePacketPollLoop();
   else
     iface->multiPacketPollLoop();  
-
+  
   ntop->getTrace()->traceEvent(TRACE_NORMAL, "Terminated packet polling for %s", 
 			       iface->get_name());
   return(NULL);
@@ -257,16 +293,11 @@ void PF_RINGInterface::startPacketPolling() {
 /* **************************************************** */
 
 void PF_RINGInterface::shutdown() {
-  void *res;
-  int i;
-
-  if(running) {
-    NetworkInterface::shutdown();
-    for (i = 0; i < num_pfring_handles; i++) {
-      if (pfring_handle[i]) pfring_breakloop(pfring_handle[i]);
-    }
-    pthread_join(pollLoop, &res);
+  for (int i = 0; i < num_pfring_handles; i++) {
+    if(pfring_handle[i]) pfring_breakloop(pfring_handle[i]);
   }
+
+  NetworkInterface::shutdown();
 }
 
 /* **************************************************** */
@@ -281,7 +312,7 @@ void PF_RINGInterface::updatePacketsStats() {
   int i;
 
   for (i = 0; i < num_pfring_handles; i++) {
-    if (pfring_stats(pfring_handle[i], &stats) >= 0) {
+    if(pfring_stats(pfring_handle[i], &stats) >= 0) {
 #if 0
       ntop->getTrace()->traceEvent(TRACE_NORMAL, "[%s][Rcvd: %llu][Drops: %llu][DroppedByFilter: %u]",
 				   ifname, stats.recv, stats.drop, stats.droppedbyfilter);
@@ -305,7 +336,7 @@ bool PF_RINGInterface::set_packet_filter(char *filter) {
   int i;
 
   for (i = 0; i < num_pfring_handles; i++) {
-    if (pfring_set_bpf_filter(pfring_handle[i], filter) != 0) {
+    if(pfring_set_bpf_filter(pfring_handle[i], filter) != 0) {
       ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to set filter %s.\n", filter);
       return(false);
     }
