@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,6 +24,12 @@
 /* *************************************** */
 
 PacketStats::PacketStats() {
+  resetStats();
+}
+
+/* *************************************** */
+
+void PacketStats::resetStats() {
   upTo64 = 0, upTo128 = 0, upTo256 = 0,
     upTo512 = 0, upTo1024 = 0, upTo1518 = 0,
     upTo2500 = 0, upTo6500 = 0, upTo9000 = 0,
@@ -33,26 +39,48 @@ PacketStats::PacketStats() {
 
 /* *************************************** */
 
-void PacketStats::incStats(u_int pkt_len) { 
-  if(pkt_len <= 64)        upTo64 += 1;
-  else if(pkt_len <= 128)  upTo128 += 1;
-  else if(pkt_len <= 256)  upTo256 += 1;
-  else if(pkt_len <= 512)  upTo512 += 1;
-  else if(pkt_len <= 1024) upTo1024 += 1;
-  else if(pkt_len <= 1518) upTo1518 += 1;
-  else if(pkt_len <= 2500) upTo2500 += 1;
-  else if(pkt_len <= 6500) upTo6500 += 1;
-  else if(pkt_len <= 9000) upTo9000 += 1;
-  else above9000 += 1;
+void PacketStats::incStats(u_int num_pkts, u_int pkt_len) { 
+  if(pkt_len <= 64)        upTo64   += num_pkts;
+  else if(pkt_len <= 128)  upTo128  += num_pkts;
+  else if(pkt_len <= 256)  upTo256  += num_pkts;
+  else if(pkt_len <= 512)  upTo512  += num_pkts;
+  else if(pkt_len <= 1024) upTo1024 += num_pkts;
+  else if(pkt_len <= 1518) upTo1518 += num_pkts;
+  else if(pkt_len <= 2500) upTo2500 += num_pkts;
+  else if(pkt_len <= 6500) upTo6500 += num_pkts;
+  else if(pkt_len <= 9000) upTo9000 += num_pkts;
+  else above9000 += num_pkts;
 };  
 
 /* *************************************** */
 
-void PacketStats::incFlagStats(u_int8_t flags) { 
-  if(flags == TH_SYN)                 syn++;
-  else if(flags == (TH_SYN|TH_ACK))   synack++;
-  else if(flags == (TH_FIN|TH_ACK))   finack++;
-  else if((flags & TH_RST) == TH_RST) rst++;
+void PacketStats::incFlagStatsSingleSegment(u_int8_t flags) {
+  switch(flags) {
+  case TH_SYN:        syn++;    break;
+  case TH_SYN|TH_ACK: synack++; break;
+  case TH_FIN|TH_ACK: finack++; break;
+  default:                      break;
+  }
+
+  if((flags & TH_RST) == TH_RST) rst++;
+}
+
+/* *************************************** */
+
+void PacketStats::incFlagStatsCumulative(u_int8_t flags) {
+  if((flags & TH_SYN) == TH_SYN)                   syn++;
+  if((flags & (TH_SYN|TH_ACK)) == (TH_SYN|TH_ACK)) synack++;
+  if((flags & (TH_FIN|TH_ACK)) == (TH_FIN|TH_ACK)) finack++;
+  if((flags & TH_RST) == TH_RST)                   rst++;
+}
+
+/* *************************************** */
+
+void PacketStats::incFlagStats(u_int8_t flags, bool cumulative_flags) {
+  if(cumulative_flags)
+    incFlagStatsCumulative(flags);
+  else
+    incFlagStatsSingleSegment(flags);
 }
 
 /* *************************************** */
@@ -119,7 +147,8 @@ json_object* PacketStats::getJSONObject() {
 
 void PacketStats::lua(lua_State* vm, const char *label) {
   lua_newtable(vm);
-  
+
+  lua_newtable(vm);
   lua_push_uint64_table_entry(vm, "upTo64", upTo64);
   lua_push_uint64_table_entry(vm, "upTo128", upTo128);
   lua_push_uint64_table_entry(vm, "upTo256", upTo256);
@@ -130,11 +159,18 @@ void PacketStats::lua(lua_State* vm, const char *label) {
   lua_push_uint64_table_entry(vm, "upTo6500", upTo6500);
   lua_push_uint64_table_entry(vm, "upTo9000", upTo9000);
   lua_push_uint64_table_entry(vm, "above9000", above9000);
+  lua_pushstring(vm, "size");
+  lua_insert(vm, -2);
+  lua_settable(vm, -3);
 
+  lua_newtable(vm);
   lua_push_uint64_table_entry(vm, "syn", syn);
   lua_push_uint64_table_entry(vm, "synack", synack);
   lua_push_uint64_table_entry(vm, "finack", finack);
   lua_push_uint64_table_entry(vm, "rst", rst);
+  lua_pushstring(vm, "tcp_flags");
+  lua_insert(vm, -2);
+  lua_settable(vm, -3);
   
   lua_pushstring(vm, label);
   lua_insert(vm, -2);
