@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -107,7 +107,7 @@ void Trace::set_trace_level(u_int8_t id) {
 void Trace::initRedis(const char *redis_host, const char *redis_password,
 		      u_int16_t redis_port, u_int8_t _redis_db_id) {
   Utils::initRedis(&traceRedis, redis_host, redis_password,
-		   redis_port, _redis_db_id);
+		   redis_port, _redis_db_id, false);
 }
 
 /* ******************************* */
@@ -120,7 +120,7 @@ void Trace::traceEvent(int eventTraceLevel, const char* _file,
 #endif
 
   if((eventTraceLevel <= traceLevel) && (traceLevel > 0)) {
-    char buf[8192], out_buf[8192];
+    char buf[8100], out_buf[9000];
     char theDate[32], *file = (char*)_file;
     const char *extra_msg = "";
     time_t theTime = time(NULL);
@@ -161,7 +161,7 @@ void Trace::traceEvent(int eventTraceLevel, const char* _file,
 
     while(buf[strlen(buf)-1] == '\n') buf[strlen(buf)-1] = '\0';
 
-    snprintf(out_buf, sizeof(out_buf), "%s [%s:%d] %s%s", theDate, file, line, extra_msg, buf);
+    snprintf(out_buf, sizeof(out_buf)-1, "%s [%s:%d] %s%s", theDate, file, line, extra_msg, buf);
 
     if(logFd) {
       rotate_mutex.lock(__FILE__, __LINE__); /* Need to lock as a rotation may be in progress */
@@ -184,11 +184,11 @@ void Trace::traceEvent(int eventTraceLevel, const char* _file,
 
     printf("%s\n", out_buf);
     fflush(stdout);
-    
-    if(traceRedis)
+
+    if(traceRedis && traceRedis->isOperational() && ntop->getRedis()->isOperational())
       traceRedis->lpush(NTOPNG_TRACE, out_buf, MAX_NUM_NTOPNG_TRACES,
-			false /* Do not re-trace errors, re-tracing would yield a deadlock */);
-    
+    			false /* Do not re-trace errors, re-tracing would yield a deadlock */);
+
     va_end(va_ap);
   }
 }
