@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,7 @@
 /* *************************************** */
 
 TrafficStats::TrafficStats() {
-  numPkts = 0, numBytes = 0;
+  numPkts.reset(), numBytes.reset();
 }
 
 /* *************************************** */
@@ -56,14 +56,14 @@ void TrafficStats::deserialize(json_object *o) {
   if(!o) return;
 
   if(json_object_object_get_ex(o, "packets", &obj))
-    numPkts = json_object_get_int64(obj);
+    numPkts.setInitialValue(json_object_get_int64(obj));
   else
-    numPkts = 0;
+    numPkts.reset();
   
   if(json_object_object_get_ex(o, "bytes", &obj))
-    numBytes = json_object_get_int64(obj);
+    numBytes.setInitialValue(json_object_get_int64(obj));
   else
-    numBytes = 0;
+    numBytes.reset();
 }
 
 /* ******************************************* */
@@ -72,8 +72,8 @@ json_object* TrafficStats::getJSONObject() {
   json_object *my_object = json_object_new_object();
   
   if(my_object) {
-    json_object_object_add(my_object, "packets", json_object_new_int64(numPkts));
-    json_object_object_add(my_object, "bytes", json_object_new_int64(numBytes));
+    json_object_object_add(my_object, "packets", json_object_new_int64(numPkts.get()));
+    json_object_object_add(my_object, "bytes", json_object_new_int64(numBytes.get()));
   }
 
   return(my_object);
