@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -35,35 +35,12 @@ void sighup(int sig) {
 
 /* ******************************** */
 
-#ifdef __linux__
-static void nohup_after_shutdown_command(const char * const after_shutdown_command) {
-  char command_buf[256];
-  int res;
-
-  if(!after_shutdown_command || after_shutdown_command[0] == '\0')
-    return;
-
-  /* Self-restarting service does not restart with systemd:
-     This is a hard limitation imposed by systemd.
-     The best suggestions so far are to use at, cron, or systemd timer units.
-
-     https://unix.stackexchange.com/questions/202048/self-restarting-service-does-not-restart-with-systemd
-   */
-  if((res = snprintf(command_buf, sizeof(command_buf),
-		     "echo \"sleep 1 && %s\" | at now",
-		     after_shutdown_command)) < 0
-     || res >= (int)sizeof(command_buf))
-    return;
-
-  printf("%s\n", command_buf);
-  fflush(stdout);
-  system(command_buf);
-}
+#ifdef WIN32
+BOOL WINAPI sigproc(DWORD sig)
+#else
+void sigproc(int sig)
 #endif
-
-/* ******************************** */
-
-void sigproc(int sig) {
+ {
   static int called = 0;
   
   if(called) {
@@ -110,12 +87,27 @@ int main(int argc, char *argv[])
   int indexAffinity = 0;
   char *core_id_s = NULL;
   int core_id;
-  char path[MAX_PATH];
+  char path[2*MAX_PATH];
+  bool has_view_all = false;
   FILE *fd;
-  ThreadedActivity *boot_activity;
-    
+  ThreadedActivity *boot_activity;  
+
+  tzset(); /* Init time and timezone */
+  
 #ifdef WIN32
   initWinsock32();
+#else
+  sqlite3_shutdown();
+  /* Multi-thread.
+     In this mode, SQLite can be safely used by multiple threads
+     provided that no single database connection is used simultaneously
+     in two or more threads.
+  */
+  if(sqlite3_config(SQLITE_CONFIG_MULTITHREAD) != SQLITE_OK) {
+    ntop->getTrace()->traceEvent(TRACE_WARNING, "Unable to set SQLITE_CONFIG_MULTITHREAD for sqlite, exiting.");
+    exit(1);
+  }
+  sqlite3_initialize();
 #endif
 
   if((ntop = new(std::nothrow)  Ntop(argv[0])) == NULL) _exit(0);
@@ -131,16 +123,26 @@ int main(int argc, char *argv[])
     rc = prefs->loadFromCLI(argc, argv);
 
   if(rc < 0) return(-1);
+ 
+  /* Create the working dir before dropping the provileges, otherwise the
+   * user might not be able to create it. Utils::dropPrivileges will move
+   * the directory ownership to the user. */
+  Utils::mkdir_tree(ntop->get_working_dir());
+
+#ifndef WIN32
+  ntop->lockNtopInstance();
+#endif
 
   ntop->registerPrefs(prefs, false);
+  prefs->reloadPrefsFromRedis();
 
-  if((boot_activity = new ThreadedActivity(BOOT_SCRIPT_PATH))) {
+  if((boot_activity = new (std::nothrow) ThreadedActivity(BOOT_SCRIPT_PATH))) {
     /* Don't call run() as by the time the script will be run the delete below will free the memory */
     /* NOTE: preferences restore from file is handled here */
-    boot_activity->runScript();
+    boot_activity->runSystemScript(time(NULL));
     delete boot_activity;
   }
-  
+
   prefs->registerNetworkInterfaces();
 
   if(prefs->get_num_user_specified_interfaces() == 0) {
@@ -148,16 +150,29 @@ int main(int argc, char *argv[])
     prefs->add_default_interfaces();
   }
 
-#ifdef NTOPNG_PRO
-  ntop->registerNagios();
+  prefs->validate();
+
+#ifndef HAVE_NEDGE 
+  if(prefs->is_appliance()) {
+    /* Disabling user change for the time being as system configuration
+     * scripts currently require permissions. */
+    ntop->getTrace()->traceEvent(TRACE_WARNING, "Disabling user change in appliance mode");
+    prefs->dont_change_user();
+  }
+#endif
+ 
+#ifndef WIN32
+  ntop->initPing();
 #endif
 
-  prefs->reloadPrefsFromRedis();
-  prefs->validate();
-  
   if(prefs->daemonize_ntopng())
     ntop->daemonize();
-
+  
+#ifndef HAVE_NEDGE
+  /* Force ZMQ interface creation */
+  ntop->broadcastIPSMessage(NULL);
+#endif
+  
 #ifdef __linux__
   /* Store number of CPUs before dropping privileges */
   ntop->setNumCPUs(sysconf(_SC_NPROCESSORS_ONLN));
@@ -177,8 +192,10 @@ int main(int argc, char *argv[])
       /* [ zmq-collector.lua@tcp://127.0.0.1:5556 ] */
 #ifndef HAVE_NEDGE
       if(!strcmp(ifName, "dummy")) {
-	iface = new DummyInterface();
-      } else if((strstr(ifName, "tcp://") || strstr(ifName, "ipc://"))) {
+	iface = new (std::nothrow) DummyInterface();
+      } else if(strstr(ifName, "zmq://")
+		|| strstr(ifName, "tcp://")
+		|| strstr(ifName, "ipc://")) {
 	char *at = strchr(ifName, '@');
 	char *endpoint;
 
@@ -187,16 +204,24 @@ int main(int argc, char *argv[])
 	else
 	  endpoint = ifName;
 
-	iface = new CollectorInterface(endpoint);
+	iface = new (std::nothrow) ZMQCollectorInterface(endpoint);
+#if defined(HAVE_KAFKA) && defined(NTOPNG_PRO)
+      } else if(strstr(ifName, "kafka://")) {
+	iface = new (std::nothrow) KafkaCollectorInterface(&ifName[8], false);
+      } else if(strstr(ifName, "kafka-ssl://")) {
+	iface = new (std::nothrow) KafkaCollectorInterface(&ifName[12], true);
+#endif
+      } else if(strstr(ifName, "syslog://")) {
+	iface = new (std::nothrow) SyslogCollectorInterface(ifName);
 #if defined(HAVE_PF_RING) && (!defined(NTOPNG_EMBEDDED_EDITION)) && (!defined(__i686__)) && (!defined(__ARM_ARCH))
       } else if(strstr(ifName, "zcflow:")) {
-	iface = new ZCCollectorInterface(ifName);
+	iface = new (std::nothrow) ZCCollectorInterface(ifName);
 #endif
       } else
 #endif
-	{
-	iface = NULL;
-
+      {
+        iface = NULL;
+	  
 #if defined(NTOPNG_PRO) && !defined(WIN32)
 	if(strncmp(ifName, "bridge:", 7) == 0) {
 	  ntop->getTrace()->traceEvent(TRACE_WARNING, "\n");
@@ -209,23 +234,30 @@ int main(int argc, char *argv[])
 
 #if defined(HAVE_NEDGE)
         if(iface == NULL && strncmp(ifName, "nf:", 3) == 0)
-          iface = new NetfilterInterface(ifName);
+          iface = new (std::nothrow) NetfilterInterface(ifName);
+#else
+#if defined(NTOPNG_PRO)
+        if((iface == NULL)
+	   && (strncmp(ifName, "ntap:", 3) == 0)
+	   && ntop->getPro()->has_valid_enterprise_l_license())
+          iface = new (std::nothrow) nTapInterface(ifName);
 #endif
-	
+#endif
+
 #ifdef HAVE_PF_RING
 	if((iface == NULL) && (!strstr(ifName, ".pcap"))) {
 	  errno = 0;
-	  iface = new PF_RINGInterface(ifName);
+	  iface = new (std::nothrow) PF_RINGInterface(ifName);
 	}
 #endif
       }
     } catch(int err) {
-      ntop->getTrace()->traceEvent(TRACE_INFO, "An exception occurred during %s interface creation[%d]: %s. Falling back to pcap",
-				   ifName, err, strerror(err));
+      ntop->getTrace()->traceEvent(TRACE_NORMAL, "Unable to open interface %s [%d]: %s. Falling back to pcap.",
+				   ifName, err, err == EAFNOSUPPORT ? "PF_RING not loaded" : strerror(err));
       if(iface) delete iface;
       iface = NULL;
     } catch(...) {
-      ntop->getTrace()->traceEvent(TRACE_INFO, "An exception occurred during %s interface creation. Falling back to pcap", ifName);
+      ntop->getTrace()->traceEvent(TRACE_NORMAL, "Unable to open interface %s. Falling back to pcap.", ifName);
       if(iface) delete iface;
       iface = NULL;
     }
@@ -234,9 +266,9 @@ int main(int argc, char *argv[])
     if(iface == NULL) {
       try {
 	errno = 0;
-	iface = new PcapInterface(ifName);
+	iface = new PcapInterface(ifName, i, false);
       } catch(int err) {
-	ntop->getTrace()->traceEvent(TRACE_ERROR, "An exception occurred during %s interface creation[%d]: %s",
+	ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to open interface %s with pcap [%d]: %s",
 				     ifName, err, strerror(err));
 	if(iface) delete iface;
 	iface = NULL;
@@ -276,12 +308,30 @@ int main(int argc, char *argv[])
     if((ifName = ntop->get_if_name(i)) == NULL || strncmp(ifName, "view:", 5))
       continue;
 
-    if((iface = new ViewInterface(ifName)))
+    if(!strcmp(ifName, "view:all")) {
+      /* Defer view:all interface after the other view interfaces */
+      has_view_all = true;
+      continue;
+    }
+
+    if((iface = new (std::nothrow) ViewInterface(ifName)))
       ntop->registerInterface(iface);
   }
 
+  if(has_view_all) {
+    NetworkInterface *iface = NULL;
+
+    if((iface = new (std::nothrow) ViewInterface("view:all")))
+      ntop->registerInterface(iface);
+  }
+  
   if(ntop->getFirstInterface() == NULL) {
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "Startup error: missing super-user privileges ?");
+#ifdef WIN32
+      ntop->getTrace()->traceEvent(TRACE_ERROR, "Please make sure you have installed the https://nmap.org/npcap/ driver");
+#else
+      ntop->getTrace()->traceEvent(TRACE_ERROR, "Startup error: missing super-user privileges ?");
+#endif
+
     exit(0);
   }
 
@@ -310,63 +360,36 @@ int main(int argc, char *argv[])
   }
 #endif
 
-  /*
-    It's safe to drop privileges now if http and https ports
-    are non-privileged. Otherwise it is necessary to delay
-    the privilege drop after the web server bind()
-   */
-  if(prefs->do_change_user()
-     && (prefs->get_http_port()  >= 1024)
-     && (prefs->get_https_port() >= 1024)) {
-    Utils::dropPrivileges();
-  }
-
 #ifndef HAVE_NEDGE
   ntop->createExportInterface();
-  ntop->getElasticSearch()->startFlowDump();
-  ntop->getLogstash()->startFlowDump();
 #endif
 
-  ntop->loadGeolocation(prefs->get_docs_dir());
+  ntop->loadGeolocation();
   ntop->loadMacManufacturers(prefs->get_docs_dir());
   ntop->loadTrackers();
-  ntop->registerHTTPserver(new HTTPserver(prefs->get_docs_dir(), prefs->get_scripts_dir()));
 
-  /* initInterface writes DB data on disk, keep it after changing user
-   * Note: privileges can be dropped by mongoose (creating HTTPserver) */
+  /* Register the HTTP server before dropping the privileges. This is required
+   * in order to possibly bind the HTTP server to privileged ports (< 1024) */
+  ntop->registerHTTPserver(new (std::nothrow) HTTPserver(prefs->get_docs_dir(), prefs->get_scripts_dir()));
+
+  /* Drop the privileges before initializing the network interfaces. This
+   * is necessary as the initialization may create files/directories, which
+   * should not be created as root. */
+  if(prefs->do_change_user())
+    Utils::dropPrivileges();
+
+  /* initInterface writes DB data on disk, keep it after changing user */
   for(int i = 0; i < MAX_NUM_INTERFACE_IDS; i++) {
     NetworkInterface *iface;
-	
+
     if((iface = ntop->getInterface(i)) != NULL)
       ntop->initInterface(iface);
   }
 
-  /*
-    If mysql flows dump is enabled, then it is necessary to create
-    and update the database schema
-   */
-  if(prefs->do_dump_flows_on_mysql()) {
-    /* create the schema only one time, no need to call it for every interface */
-    if(!ntop->getFirstInterface()->createDBSchema()){
-      ntop->getTrace()->traceEvent(TRACE_ERROR,
-				   "Unable to create database schema, quitting.");
-      exit(EXIT_FAILURE);
-    }
-  } else if(prefs->do_read_flows_from_nprobe_mysql()) {
-    /* Create a view only one time for the first interface */
-    if(!ntop->getFirstInterface()->createNprobeDBView()){
-      ntop->getTrace()->traceEvent(TRACE_ERROR,
-				   "Unable to create a view on the nProbe database.");
-      exit(EXIT_FAILURE);
-    }
-  }
-
   /*
     We have created the network interface and thus changed user. Let's now check
     if we can write on the data directory
   */
-  Utils::mkdir_tree(ntop->get_working_dir());
-
   snprintf(path, sizeof(path), "%s/.test", ntop->get_working_dir());
   ntop->fixPath(path);
   
@@ -386,7 +409,7 @@ int main(int argc, char *argv[])
      || prefs->daemonize_ntopng()
 #endif
      ) {
-      char path[MAX_PATH];
+      char path[2*MAX_PATH];
 
       Utils::mkdir_tree(ntop->get_data_dir());
       Utils::mkdir_tree(ntop->get_working_dir());
@@ -401,7 +424,9 @@ int main(int argc, char *argv[])
   ntop->getTrace()->traceEvent(TRACE_NORMAL, "Scripts/HTML pages directory: %s",
 			       ntop->get_install_dir());
 
-#ifndef WIN32
+#ifdef WIN32
+  SetConsoleCtrlHandler(sigproc, TRUE);
+#else
   signal(SIGHUP,  sighup);
   signal(SIGINT,  sigproc);
   signal(SIGTERM, sigproc);
@@ -414,28 +439,34 @@ int main(int argc, char *argv[])
   ntop->getTrace()->traceEvent(TRACE_NORMAL, "-----------------------------------------------------------");
 #endif
 
-  /* this returns after a shutdown has been requested */
+  /* This method returns after a shutdown has been requested.
+     runHousekeepingTasks() is performed within this method until it has returned */
   ntop->start();
 
-  /* perform all the necessary cleanup and wait for other threads termination */
+  ntop->getTrace()->traceEvent(TRACE_NORMAL, "Terminating...");
+
+  /* Perform all the necessary cleanup (purge all flows, hosts, etc)
+   * and wait for other threads termination */
   ntop->shutdownAll();
 
+  ntop->runShutdownTasks();
+
   delete ntop;
   ntop = NULL;
 
 #ifdef __linux__
   switch(afterShutdownAction) {
   case after_shutdown_reboot:
-    nohup_after_shutdown_command("systemctl start systemd-reboot");
+    Utils::deferredExec("systemctl start systemd-reboot");
     break;
   case after_shutdown_poweroff:
-    nohup_after_shutdown_command("systemctl start systemd-poweroff");
+    Utils::deferredExec("systemctl start systemd-poweroff");
     break;
   case after_shutdown_restart_self:
-    /* Necessary to check if the service has been activated via systemd before actually
-       restarting it using systemd. When the service hasn't been started with systemd (e.g., during the development)
-       it is desirable to skip the use of systemd */
-    nohup_after_shutdown_command("systemctl restart ntopng");
+    /* Returning 1 to force systemd to restart the service due 
+     * to 'Unclean exit code'. A systemctl restart does not work here.
+     * Note: this requires 'Restart=on-failure' to work */
+    return(1);
     break;
   case after_shutdown_nop:
   default:
