@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-23 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,38 +24,88 @@
 
 /* *************************************** */
 
-nDPIStats::nDPIStats() {
+nDPIStats::nDPIStats(bool enable_throughput_stats, bool enable_behavior_stats) {
   memset(counters, 0, sizeof(counters));
   memset(cat_counters, 0, sizeof(cat_counters));
+
+#ifdef NTOPNG_PRO
+  nextMinPeriodicUpdate = 0;
+
+  behavior_bytes_traffic = NULL;
+
+  if(enable_behavior_stats)
+    behavior_bytes_traffic = new (std::nothrow)BehaviorAnalysis*[MAX_NDPI_PROTOS]();
+#endif
+
+  if(enable_throughput_stats)
+    bytes_thpt = new (std::nothrow)ThroughputStats*[MAX_NDPI_PROTOS]();
+  else
+    bytes_thpt = NULL;
 }
 
 /* *************************************** */
 
 nDPIStats::nDPIStats(const nDPIStats &stats) {
-  for(int i=0; i<MAX_NDPI_PROTOS; i++) {
-    if(stats.counters[i] != NULL) {
-      counters[i] = (ProtoCounter*) malloc(sizeof(ProtoCounter));
-      memcpy(counters[i], stats.counters[i], sizeof(ProtoCounter));
-    } else {
-      counters[i] = NULL;
-    }
-  }
+  memset(counters, 0, sizeof(counters));
+  memset(cat_counters, 0, sizeof(cat_counters));
 
-  memcpy(cat_counters, stats.cat_counters, sizeof(cat_counters));
+#ifdef NTOPNG_PRO
+  nextMinPeriodicUpdate = 0;
+  
+  behavior_bytes_traffic = NULL;
+
+  if(stats.behavior_bytes_traffic) {
+    behavior_bytes_traffic = new (std::nothrow)BehaviorAnalysis*[MAX_NDPI_PROTOS]();
+  }
+#endif
+
+  if(stats.bytes_thpt)
+    bytes_thpt = new (std::nothrow)ThroughputStats*[MAX_NDPI_PROTOS]();
+  else
+    bytes_thpt = NULL;
+
+  for(int i = 0; i < MAX_NDPI_PROTOS; i++) {      
+    if(bytes_thpt && stats.bytes_thpt && stats.bytes_thpt[i])
+      bytes_thpt[i] = new (std::nothrow)ThroughputStats(*stats.bytes_thpt[i]);
+
+    if(stats.counters[i]
+       && (counters[i] = (ProtoCounter*)malloc(sizeof(*counters[i]))))
+      memcpy(counters[i], stats.counters[i], sizeof(*counters[i]));
+    else
+      counters[i] = NULL;
+  }
 }
 
 /* *************************************** */
 
 nDPIStats::~nDPIStats() {
   for(int i=0; i<MAX_NDPI_PROTOS; i++) {
+#ifdef NTOPNG_PRO
+    if(behavior_bytes_traffic && behavior_bytes_traffic[i])
+      delete behavior_bytes_traffic[i];
+#endif
+
     if(counters[i] != NULL)
       free(counters[i]);
+
+    if(bytes_thpt && bytes_thpt[i])
+      delete bytes_thpt[i];
   }
+
+  if(bytes_thpt)
+    delete []bytes_thpt;
+#ifdef NTOPNG_PRO
+  if(behavior_bytes_traffic)
+    delete []behavior_bytes_traffic;
+#endif
 }
 
 /* *************************************** */
 
-void nDPIStats::sum(nDPIStats *stats) {
+void nDPIStats::sum(nDPIStats *stats) const {
+  if(bytes_thpt && !stats->bytes_thpt)
+    stats->bytes_thpt = new (std::nothrow)ThroughputStats*[MAX_NDPI_PROTOS]();
+
   for(int i = 0; i < MAX_NDPI_PROTOS; i++) {
     if(counters[i] != NULL) {
       if(stats->counters[i] == NULL) {
@@ -66,7 +116,7 @@ void nDPIStats::sum(nDPIStats *stats) {
 	    ntop->getTrace()->traceEvent(TRACE_WARNING, "Not enough memory");
 	    oom_warning_sent = true;
 	  }
-	 
+
 	  return;
 	}
       }
@@ -76,6 +126,14 @@ void nDPIStats::sum(nDPIStats *stats) {
       stats->counters[i]->bytes.sent    += counters[i]->bytes.sent;
       stats->counters[i]->bytes.rcvd    += counters[i]->bytes.rcvd;
       stats->counters[i]->duration      += counters[i]->duration;
+      stats->counters[i]->total_flows   += counters[i]->total_flows;
+
+      if(bytes_thpt && bytes_thpt[i] && stats->bytes_thpt) {
+	if(!stats->bytes_thpt[i])
+	  stats->bytes_thpt[i] = new (std::nothrow)ThroughputStats(*bytes_thpt[i]);
+	else
+	  bytes_thpt[i]->sum(stats->bytes_thpt[i]);
+      }
     }
   }
 
@@ -92,81 +150,110 @@ void nDPIStats::sum(nDPIStats *stats) {
 /* *************************************** */
 
 void nDPIStats::print(NetworkInterface *iface) {
-  for(int i=0; i<MAX_NDPI_PROTOS; i++) {
+  for(int i = 0; i < MAX_NDPI_PROTOS; i++) {
     if(counters[i] != NULL) {
-      if(counters[i]->packets.sent || counters[i]->packets.rcvd)
-	printf("[%s] [pkts: %llu/%llu][bytes: %llu/%llu][duration: %u sec]\n",
+      if(counters[i]->bytes.sent || counters[i]->bytes.rcvd)
+	printf("[%s] [pkts: %llu/%llu][bytes: %llu/%llu][duration: %u sec][thpt: %.2f]\n",
 	       iface->get_ndpi_proto_name(i),
 	       (long long unsigned) counters[i]->packets.sent, (long long unsigned) counters[i]->packets.rcvd,
 	       (long long unsigned) counters[i]->bytes.sent,   (long long unsigned)counters[i]->bytes.rcvd,
-	       counters[i]->duration);
+	       counters[i]->duration,
+	       bytes_thpt && bytes_thpt[i] ? bytes_thpt[i]->getThpt() : 0);
     }
   }
 }
 
 /* *************************************** */
 
-void nDPIStats::lua(NetworkInterface *iface, lua_State* vm, bool with_categories, bool tsLua) {
+void nDPIStats::lua(NetworkInterface *iface, lua_State* vm, bool with_categories, bool tsLua, bool diff) {
   lua_newtable(vm);
 
-  for(int i=0; i<MAX_NDPI_PROTOS; i++)
+  for(int i = 0; i < MAX_NDPI_PROTOS; i++) {
     if(unlikely(counters[i] != NULL)) {
       char *name = iface->get_ndpi_proto_name(i);
 
       if(name != NULL) {
-	if(counters[i]->packets.sent || counters[i]->packets.rcvd) {
-          if(!tsLua) {
+	if(counters[i]->bytes.sent || counters[i]->bytes.rcvd
+	    || iface->hasSeenEBPFEvents() /* eBPF flows can have 0 traffic */) {
+	  if(!tsLua) {
 	    lua_newtable(vm);
 
-	    lua_push_str_table_entry(vm, "breed", iface->get_ndpi_proto_breed_name(i));
+	    lua_push_str_table_entry(vm,    "breed", iface->get_ndpi_proto_breed_name(i));
 	    lua_push_uint64_table_entry(vm, "packets.sent", counters[i]->packets.sent);
 	    lua_push_uint64_table_entry(vm, "packets.rcvd", counters[i]->packets.rcvd);
 	    lua_push_uint64_table_entry(vm, "bytes.sent", counters[i]->bytes.sent);
 	    lua_push_uint64_table_entry(vm, "bytes.rcvd", counters[i]->bytes.rcvd);
 	    lua_push_uint64_table_entry(vm, "duration", counters[i]->duration);
+	    lua_push_uint64_table_entry(vm, "num_flows", counters[i]->total_flows);
+
+    #ifdef NTOPNG_PRO
+	    if(behavior_bytes_traffic && behavior_bytes_traffic[i])
+	      behavior_bytes_traffic[i]->luaBehavior(vm, "l7_traffic_behavior", (diff ? NDPI_TRAFFIC_BEHAVIOR_REFRESH : 0 ));
+    #endif
+
+	    if(bytes_thpt && bytes_thpt[i]) {
+	      lua_newtable(vm);
+
+	      lua_push_float_table_entry(vm, "bps", bytes_thpt[i]->getThpt());
+	      lua_push_uint64_table_entry(vm, "trend_bps", bytes_thpt[i]->getTrend());
+
+	      lua_pushstring(vm, "throughput"); lua_insert(vm, -2); lua_rawset(vm, -3);
+	    }
 
 	    lua_pushstring(vm, name);
 	    lua_insert(vm, -2);
 	    lua_rawset(vm, -3);
-          } else {
-            char buf[64];
-	    
-            snprintf(buf, sizeof(buf), "%llu|%llu",
-		     (unsigned long long)counters[i]->bytes.sent,
-		     (unsigned long long)counters[i]->bytes.rcvd);
+	  } else {
+	    char buf[64];
 
-            lua_push_str_table_entry(vm, name, buf);
-          }
+	    snprintf(buf, sizeof(buf), "%llu|%llu|%u",
+		     (unsigned long long)counters[i]->bytes.sent,
+		     (unsigned long long)counters[i]->bytes.rcvd,
+		     counters[i]->total_flows);
+
+	    lua_push_str_table_entry(vm, name, buf);
+	  }
 	}
       }
     }
-
+  }
+  
   lua_pushstring(vm, "ndpi");
   lua_insert(vm, -2);
   lua_settable(vm, -3);
 
-  if (with_categories) {
+  /* *********************************************** */
+  
+  if(with_categories) {
     lua_newtable(vm);
 
-    for (int i=0; i<NDPI_PROTOCOL_NUM_CATEGORIES; i++) {
-      if(unlikely(cat_counters[i].bytes.sent + cat_counters[i].bytes.rcvd > 0)) {
-        const char *name = iface->get_ndpi_category_name((ndpi_protocol_category_t)i);
+    for (int i = 0;  i < NDPI_PROTOCOL_NUM_CATEGORIES; i++) {
+      if(cat_counters[i].bytes.sent + cat_counters[i].bytes.rcvd) {
+	const char *name = iface->get_ndpi_category_name((ndpi_protocol_category_t)i);
 
-        if(!tsLua) {
-          lua_newtable(vm);
 
-          lua_push_uint64_table_entry(vm, "category", i);
-          lua_push_uint64_table_entry(vm, "bytes", cat_counters[i].bytes.sent + cat_counters[i].bytes.rcvd);
-          lua_push_uint64_table_entry(vm, "bytes.sent", cat_counters[i].bytes.sent);
-          lua_push_uint64_table_entry(vm, "bytes.rcvd", cat_counters[i].bytes.rcvd);
-          lua_push_uint64_table_entry(vm, "duration", cat_counters[i].duration);
+	if(!tsLua) {
+	  lua_newtable(vm);
 
-          lua_pushstring(vm, name);
-          lua_insert(vm, -2);
-          lua_rawset(vm, -3);
-        } else {
-          lua_push_uint64_table_entry(vm, name, cat_counters[i].bytes.sent + cat_counters[i].bytes.rcvd);
-        }
+	  lua_push_uint64_table_entry(vm, "category", i);
+	  lua_push_uint64_table_entry(vm, "bytes", cat_counters[i].bytes.sent + cat_counters[i].bytes.rcvd);
+	  lua_push_uint64_table_entry(vm, "bytes.sent", cat_counters[i].bytes.sent);
+	  lua_push_uint64_table_entry(vm, "bytes.rcvd", cat_counters[i].bytes.rcvd);
+	  lua_push_uint64_table_entry(vm, "duration", cat_counters[i].duration);
+
+	  lua_pushstring(vm, name);
+	  lua_insert(vm, -2);
+	  lua_rawset(vm, -3);
+	} else {
+
+	  char buf[64];
+
+	  snprintf(buf, sizeof(buf), "%llu|%llu",
+	    (unsigned long long)cat_counters[i].bytes.sent,
+	    (unsigned long long)cat_counters[i].bytes.rcvd);
+
+	  lua_push_str_table_entry(vm, name, buf);
+	}
       }
     }
 
@@ -178,10 +265,42 @@ void nDPIStats::lua(NetworkInterface *iface, lua_State* vm, bool with_categories
 
 /* *************************************** */
 
+void nDPIStats::updateStats(const struct timeval *tv) {
+  if(!bytes_thpt)
+    return;
+
+  for(int i = 0; i < MAX_NDPI_PROTOS; i++) {
+    if(!counters[i])
+      continue;
+
+    if(!bytes_thpt[i])
+      bytes_thpt[i] = new (std::nothrow)ThroughputStats();
+
+    if(bytes_thpt[i])
+      bytes_thpt[i]->updateStats(tv, counters[i]->bytes.sent + counters[i]->bytes.rcvd);
+
+#ifdef NTOPNG_PRO
+    if(tv->tv_sec >= nextMinPeriodicUpdate) {
+      if(!behavior_bytes_traffic)
+        continue;
+
+      if(!behavior_bytes_traffic[i])
+        behavior_bytes_traffic[i] = new (std::nothrow)BehaviorAnalysis(0.9 /* Alpha parameter */, 0.1 /* Beta parameter */, 0.05 /* Significance */, true /* Counter */);
+
+      if(behavior_bytes_traffic[i])
+        behavior_bytes_traffic[i]->updateBehavior(NULL, counters[i]->bytes.sent + counters[i]->bytes.rcvd, NULL, false);
+
+      nextMinPeriodicUpdate = tv->tv_sec + NDPI_TRAFFIC_BEHAVIOR_REFRESH;
+    }
+#endif
+  }
+}
+
+/* *************************************** */
+
 void nDPIStats::incStats(u_int32_t when, u_int16_t proto_id,
 			 u_int64_t sent_packets, u_int64_t sent_bytes,
 			 u_int64_t rcvd_packets, u_int64_t rcvd_bytes) {
-
   if(proto_id < (MAX_NDPI_PROTOS)) {
     if(counters[proto_id] == NULL) {
       if((counters[proto_id] = (ProtoCounter*)calloc(1, sizeof(ProtoCounter))) == NULL) {
@@ -191,7 +310,7 @@ void nDPIStats::incStats(u_int32_t when, u_int16_t proto_id,
 	  ntop->getTrace()->traceEvent(TRACE_WARNING, "Not enough memory");
 	  oom_warning_sent = true;
 	}
-	
+
 	return;
       }
     }
@@ -210,7 +329,7 @@ void nDPIStats::incStats(u_int32_t when, u_int16_t proto_id,
 /* *************************************** */
 
 void nDPIStats::incCategoryStats(u_int32_t when, ndpi_protocol_category_t category_id,
-          u_int64_t sent_bytes, u_int64_t rcvd_bytes) {
+	  u_int64_t sent_bytes, u_int64_t rcvd_bytes) {
   if(category_id < NDPI_PROTOCOL_NUM_CATEGORIES) {
     cat_counters[category_id].bytes.sent += sent_bytes;
     cat_counters[category_id].bytes.rcvd += rcvd_bytes;
@@ -225,8 +344,17 @@ void nDPIStats::incCategoryStats(u_int32_t when, ndpi_protocol_category_t catego
 
 /* *************************************** */
 
+void nDPIStats::incFlowsStats(u_int16_t proto_id) {
+  if(proto_id < (MAX_NDPI_PROTOS)) {
+    if(counters[proto_id] != NULL)
+      counters[proto_id]->total_flows++;
+  }
+}
+
+/* *************************************** */
+
 char* nDPIStats::serialize(NetworkInterface *iface) {
-  json_object *my_object = getJSONObject(iface);  
+  json_object *my_object = getJSONObject(iface);
   char *rsp = strdup(json_object_to_json_string(my_object));
 
   /* Free memory */
@@ -257,7 +385,7 @@ void nDPIStats::deserialize(NetworkInterface *iface, json_object *o) {
 
 	if((counters[proto_id] = (ProtoCounter*)calloc(1, sizeof(ProtoCounter))) != NULL) {
 	  json_object *duration;
-	  
+
 	  if(json_object_object_get_ex(obj, "bytes", &bytes)) {
 	    json_object *sent, *rcvd;
 
@@ -279,7 +407,7 @@ void nDPIStats::deserialize(NetworkInterface *iface, json_object *o) {
 	  }
 
 	  if(json_object_object_get_ex(obj, "duration", &duration))
-	    counters[proto_id]->duration = json_object_get_int(duration);	  
+	    counters[proto_id]->duration = json_object_get_int(duration);
 	}
       }
     }
@@ -317,7 +445,7 @@ static void addProtoJson(json_object *my_object, ProtoCounter *counter, const ch
   if(! inner) return;
 
   json_object_object_add(inner, "duration", json_object_new_int64(counter->duration));
-  
+
   inner1 = json_object_new_object();
   if(! inner1) { json_object_put(inner); return; }
   json_object_object_add(inner1, "sent", json_object_new_int64(counter->bytes.sent));
@@ -331,23 +459,25 @@ static void addProtoJson(json_object *my_object, ProtoCounter *counter, const ch
   json_object_object_add(inner, "packets", inner1);
 
   json_object_object_add(my_object, name, inner);
-} 
+}
+
+/* *************************************** */
 
 json_object* nDPIStats::getJSONObject(NetworkInterface *iface) {
   char *unknown = iface->get_ndpi_proto_name(NDPI_PROTOCOL_UNKNOWN);
   json_object *my_object;
-  json_object *inner, *inner1;  
+  json_object *inner, *inner1;
 
   my_object = json_object_new_object();
 
   for(int proto_id = 0; proto_id < MAX_NDPI_PROTOS; proto_id++) {
     if(counters[proto_id] != NULL) {
       char *name = iface->get_ndpi_proto_name(proto_id);
-      
+
       if((proto_id > 0) && (name == unknown)) break;
 
       if(name != NULL)
-        addProtoJson(my_object, counters[proto_id], name);
+	addProtoJson(my_object, counters[proto_id], name);
     }
   }
 
@@ -371,43 +501,15 @@ json_object* nDPIStats::getJSONObject(NetworkInterface *iface) {
 
 /* *************************************** */
 
-json_object* nDPIStats::getJSONObjectForCheckpoint(NetworkInterface *iface) {
-  json_object *my_object;
-  char *name;
-
-  my_object = json_object_new_object();
-  if(my_object == NULL) return NULL;
-
-  if (counters[NDPI_PROTOCOL_DNS] != NULL) {
-    name = iface->get_ndpi_proto_name(NDPI_PROTOCOL_DNS);
-    if(name) addProtoJson(my_object, counters[NDPI_PROTOCOL_DNS], name);
-  }
-
-  if (counters[NDPI_PROTOCOL_EDONKEY] != NULL) {
-    name = iface->get_ndpi_proto_name(NDPI_PROTOCOL_EDONKEY);
-    if(name) addProtoJson(my_object, counters[NDPI_PROTOCOL_EDONKEY], name);
-  }
-
-  if (counters[NDPI_PROTOCOL_BITTORRENT] != NULL) {
-    name = iface->get_ndpi_proto_name(NDPI_PROTOCOL_BITTORRENT);
-    if(name) addProtoJson(my_object, counters[NDPI_PROTOCOL_BITTORRENT], name);
-  }
-
-  if (counters[NDPI_PROTOCOL_SKYPE] != NULL) {
-    name = iface->get_ndpi_proto_name(NDPI_PROTOCOL_SKYPE);
-    if(name) addProtoJson(my_object, counters[NDPI_PROTOCOL_SKYPE], name);
-  }
-
-  return my_object;
-}
-
-/* *************************************** */
-
 void nDPIStats::resetStats() {
-  for(int i=0; i<MAX_NDPI_PROTOS; i++) {
+  for(int i = 0; i < MAX_NDPI_PROTOS; i++) {
     /* NOTE: do not deallocate counters since they can be in use by other threads */
-    if(counters[i] != NULL)
+    if(counters[i] != NULL) {
       memset(&counters[i], 0, sizeof(counters[i]));
+
+      if(bytes_thpt && bytes_thpt[i])
+	bytes_thpt[i]->resetStats();
+    }
   }
 
   memset(cat_counters, 0, sizeof(cat_counters));
