@@ -10,21 +10,19 @@
  * ./speedtest
  */
 
-#ifdef TEST_SPEEDTEST
 #include <stdio.h>
 #include <string.h>
+#ifndef WIN32
 #include <unistd.h>
 #include <pthread.h>
+#endif
 #include <math.h>
 #include <curl/curl.h>
 #include "json.h"
-#define _usleep usleep
-#define DEBUG_SPEEDTEST
-#endif
 
-//#ifdef DEBUG_SPEEDTEST
+#ifdef DEBUG_SPEEDTEST
 #define INFO_SPEEDTEST
-//#endif
+#endif
 
 #ifdef HAVE_EXPAT
 
@@ -102,16 +100,20 @@ struct server_info
   double distance;
 };
 
-int depth;
-struct client_info client;
-struct server_info servers[MAX_CLOSEST_SERVER_NUM];
-int num_servers = 0;
+static int depth;
+static struct client_info client;
+static struct server_info servers[MAX_CLOSEST_SERVER_NUM];
+static int num_servers = 0;
+
+/* ***************************************** */
 
 static int calc_past_time(struct timeval* start, struct timeval* end)
 {
   return (end->tv_sec - start->tv_sec) * 1000 + (end->tv_usec - start->tv_usec)/1000;
 }
 
+/* ***************************************** */
+
 static size_t write_data(void* ptr, size_t size, size_t nmemb, void *stream)
 {
   struct thread_para *p_thread = (struct thread_para*)stream;
@@ -128,6 +130,7 @@ static size_t write_data(void* ptr, size_t size, size_t nmemb, void *stream)
   return size * nmemb;
 }
 
+/* ***************************************** */
 
 size_t
 write_web_buf(void *ptr, size_t size, size_t nmemb, void *data)
@@ -144,11 +147,14 @@ write_web_buf(void *ptr, size_t size, size_t nmemb, void *data)
   return realsize;
 }
 
-double radian(double d)
-{
+/* ***************************************** */
+
+double radian(double d) {
   return d * PI / 180.0;
 }
 
+/* ***************************************** */
+
 double get_distance(double lat1, double lng1, double lat2, double lng2)
 {
   double radLat1 = radian(lat1);
@@ -163,6 +169,8 @@ double get_distance(double lat1, double lng1, double lat2, double lng2)
   return dst;
 }
 
+/* ***************************************** */
+
 static void XMLCALL start_element(void *userData, const char *el, const char **atts)
 {
   int i;
@@ -206,6 +214,8 @@ static void XMLCALL start_element(void *userData, const char *el, const char **a
   depth++;
 }
 
+/* ***************************************** */
+
 static void XMLCALL end_element(void *userData, const char *name)
 {
   depth--;
@@ -247,6 +257,8 @@ static void XMLCALL end_element(void *userData, const char *name)
   }
 }
 
+/* ***************************************** */
+
 static int do_latency(char *p_url)
 {
   char latency_url[URL_LENGTH_MAX + sizeof(LATENCY_TXT_URL)] = {0};
@@ -258,15 +270,23 @@ static int do_latency(char *p_url)
 
   curl = curl_easy_init();
 
-  sprintf(latency_url, "%s%s", p_url, LATENCY_TXT_URL);
-  sprintf(useragent, "curl/%.02f.0", v);
+  snprintf(latency_url, sizeof(latency_url), "%s%s", p_url, LATENCY_TXT_URL);
+  snprintf(useragent, sizeof(useragent), "curl/%.02f.0", v);
 
+#ifdef DEBUG_SPEEDTEST
+  printf("Calling %s\n", latency_url);  
+#endif
+  
   curl_easy_setopt(curl, CURLOPT_URL, latency_url);
   curl_easy_setopt(curl, CURLOPT_USERAGENT, useragent);
   curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
   curl_easy_setopt(curl, CURLOPT_WRITEDATA, NULL);
   curl_easy_setopt(curl, CURLOPT_TIMEOUT, 3L);
 
+#ifdef DEBUG_SPEEDTEST
+  curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
+#endif
+
   res = curl_easy_perform(curl);
 
   curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &response_code);
@@ -281,6 +301,8 @@ static int do_latency(char *p_url)
   return OK;
 }
 
+/* ***************************************** */
+
 static double test_latency(char *p_url)
 {
   struct timeval s_time, e_time;
@@ -295,37 +317,57 @@ static double test_latency(char *p_url)
   return latency;
 }
 
+/* ***************************************** */
+
 static void* do_download(void* data)
 {
   CURL *curl;
   CURLcode res;
   struct thread_para* p_para = (struct thread_para*)data;
-  double length = 0;
+  double size = 0;
   double time = 0, time1 = 0, time2;
-
+  char useragent[16];
+  double v = (double) (rand() % 1000) / 100;
+  
   curl = curl_easy_init();
-
-  //printf("image url = %s\n", p_para->url);
+  snprintf(useragent, sizeof(useragent), "curl/%.02f.0", v);
+  
+#ifdef DEBUG_SPEEDTEST
+  printf("image url = %s\n", p_para->url);
+#endif
   curl_easy_setopt(curl, CURLOPT_URL, p_para->url);
   curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
   curl_easy_setopt(curl, CURLOPT_WRITEDATA, p_para);
+  curl_easy_setopt(curl, CURLOPT_USERAGENT, useragent);
+  
+#ifdef DEBUG_SPEEDTEST
+  curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
+#endif
+  
   res = curl_easy_perform(curl);
+
   if (res != CURLE_OK) {
-    // printf("curl_easy_perform() failed: %s\n", curl_easy_strerror(res));
-    curl_easy_cleanup(curl);
-    return NULL;
+#ifdef DEBUG_SPEEDTEST
+    printf("curl_easy_perform() failed: %s\n", curl_easy_strerror(res));
+#endif
+  } else {
+    curl_easy_getinfo(curl, CURLINFO_SIZE_DOWNLOAD, &size);
+    curl_easy_getinfo(curl, CURLINFO_TOTAL_TIME, &time);
+    curl_easy_getinfo(curl, CURLINFO_CONNECT_TIME, &time1);
+    curl_easy_getinfo(curl, CURLINFO_STARTTRANSFER_TIME, &time2);
+#ifdef DEBUG_SPEEDTEST
+    printf("Completed: [Size: %lf][TotalTime: %lf][ConnectTime: %lf][TransferTime: %lf]\n", size, time, time1, time2);
+#endif
+    p_para->result = size;
+    p_para->finish = 1;
   }
-  curl_easy_getinfo(curl, CURLINFO_SIZE_DOWNLOAD, &length);
-  curl_easy_getinfo(curl, CURLINFO_TOTAL_TIME, &time);
-  curl_easy_getinfo(curl, CURLINFO_CONNECT_TIME, &time1);
-  curl_easy_getinfo(curl, CURLINFO_STARTTRANSFER_TIME, &time2);
-  //printf("Length is %lf %lf %lf %lf\n", length, time, time1, time2);
-  p_para->result = length;
-  p_para->finish = 1;
+  
   curl_easy_cleanup(curl);
   return NULL;
 }
 
+/* ***************************************** */
+
 static void loop_threads(struct thread_para *p_thread,
 			 int num_thread, double *speed, int *p_num_speed)
 {
@@ -352,7 +394,7 @@ static void loop_threads(struct thread_para *p_thread,
       //printf("p_thread[i].now = %lf\n", p_thread[i].now);
     }
 
-    _usleep(RECORED_EVERY_SEC*1000*1000); //0.2s
+    usleep(RECORED_EVERY_SEC*1000*1000); //0.2s
     if (sum == 0 && num_speed == 0)
       continue;
 
@@ -374,6 +416,8 @@ static void loop_threads(struct thread_para *p_thread,
   return;
 }
 
+/* ***************************************** */
+
 static double calculate_average_speed(double *p_speed, int num_speed)
 {
 
@@ -398,7 +442,7 @@ static double calculate_average_speed(double *p_speed, int num_speed)
     }
 
   }
-#if 0
+#ifdef DEBUG_SPEEDTEST
   for (i = 0; i < num_speed; i++) {
     printf("%0.2lf ", p_speed[i]*8/(1024*1024));
     if (i%10 == 0)
@@ -415,9 +459,13 @@ static double calculate_average_speed(double *p_speed, int num_speed)
   }
   //printf("speed = %0.2lf\n", (sum*8/(end - start))/(1024*1024));
 
+  if(end == start) end++;
+  
   return sum/(end - start);
 }
 
+/* ***************************************** */
+
 static int init_instant_speed(double **p_speed, int *p_speed_num)
 {
   *p_speed_num    = SPEEDTEST_TIME_MAX/RECORED_EVERY_SEC + 1;
@@ -430,6 +478,9 @@ static int init_instant_speed(double **p_speed, int *p_speed_num)
   memset(*p_speed, 0, (*p_speed_num)*sizeof(double));
   return 0;
 }
+
+/* ***************************************** */
+
 static double test_download(char *p_url, int num_thread, int dsize, char init)
 {
   struct timeval s_time;
@@ -491,6 +542,8 @@ static double test_download(char *p_url, int num_thread, int dsize, char init)
   return speed;
 }
 
+/* ***************************************** */
+
 #ifdef FULL_REPORT
 static size_t read_data(void* ptr, size_t size, size_t nmemb, void *userp)
 {
@@ -527,27 +580,36 @@ static size_t read_data(void* ptr, size_t size, size_t nmemb, void *userp)
   return  length;
 }
 
+/* ***************************************** */
+
 static void* do_upload(void *p) {
   struct thread_para* para = (struct thread_para*)p;
   CURL *curl;
   CURLcode res;
   long size = para->upload_size;
   int loop = 1;
-
+  char useragent[16];
+  double v = (double) (rand() % 1000) / 100;
+  
   if (size > UPLOAD_CHRUNK_SIZE_MAX)
     loop = (size / UPLOAD_CHRUNK_SIZE_MAX) + 1;
 
   curl = curl_easy_init();
-
+  sprintf(useragent, "curl/%.02f.0", v);
+  
   curl_easy_setopt(curl, CURLOPT_URL, para->url);
   curl_easy_setopt(curl, CURLOPT_READFUNCTION, read_data);
   curl_easy_setopt(curl, CURLOPT_READDATA, para);
   curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
   curl_easy_setopt(curl, CURLOPT_WRITEDATA, NULL);
   curl_easy_setopt(curl, CURLOPT_POSTFIELDS, NULL);
+  curl_easy_setopt(curl, CURLOPT_USERAGENT, useragent);
+
+#ifdef DEBUG_SPEEDTEST
+  curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
+#endif
 
   while (loop) {
-
     double size_upload;
 
     para->chunk_size = size - para->result> UPLOAD_CHRUNK_SIZE_MAX ?
@@ -568,11 +630,14 @@ static void* do_upload(void *p) {
 
   curl_easy_cleanup(curl);
   para->finish = 1;
-  //printf("size upload = %lf\n", size_upload);
-
+#ifdef DEBUG_SPEEDTEST
+  printf("size upload = %lu\n", para->result);
+#endif
+  
   return(NULL);
 }
 
+/* ***************************************** */
 
 static double test_upload(char *p_url, int num_thread, long size,
 		       char *p_ext, char init)
@@ -593,11 +658,16 @@ static double test_upload(char *p_url, int num_thread, long size,
     paras[i].result = 0;
     paras[i].finish = 0;
     paras[i].upload_size = size/num_thread;
+#ifdef DEBUG_SPEEDTEST
+    printf("[thread %u] Upload size: %lu\n", i, paras[i].upload_size);
+#endif
+    
     //printf("szeleft = %ld\n", paras[i].upload_size);
     //int error = 
     pthread_create(&paras[i].tid, NULL, do_upload, (void*)&paras[i]);
     // if ( error != 0) printf("Can't Run thread num %d, error %d\n", i, error);
   }
+  
   if (init != 0) {
     loop_threads(paras, num_thread, instant_speed, &speed_num);
   }
@@ -607,6 +677,10 @@ static double test_upload(char *p_url, int num_thread, long size,
     sum += paras[i].result;
   }
 
+#ifdef DEBUG_SPEEDTEST
+  printf("Total size upload = %lf\n", sum);
+#endif
+
   if (init != 0) {
     speed = calculate_average_speed(instant_speed, speed_num);
 
@@ -631,12 +705,16 @@ static double test_upload(char *p_url, int num_thread, long size,
 }
 #endif
 
+/* ***************************************** */
+
 static int get_download_filename(double speed, int num_thread)
 {
   int i;
   int filelist[] = {350, 500, 750, 1000, 1500, 2000, 3000, 3500, 4000};
   int num_file = ARRAY_SIZE(filelist);
 
+  if(speed == 0) speed = 1;
+  
   for (i = 1; i < num_file; i++) {
     long long int time;
     float times = (float)filelist[i]/350;
@@ -647,11 +725,15 @@ static int get_download_filename(double speed, int num_thread)
     if (time > SPEEDTEST_TIME_MAX)
       break;
   }
+
   if (i < num_file)
     return filelist[i - 1];
+
   return filelist[num_file - 1];
 }
 
+/* ***************************************** */
+
 static int get_upload_extension(char *server, char *p_ext)
 {
   CURL *curl;
@@ -690,6 +772,8 @@ cleanup:
   return rv;
 }
 
+/* ***************************************** */
+
 static int get_client_info(struct client_info *p_client)
 {
   CURL *curl;
@@ -706,9 +790,14 @@ static int get_client_info(struct client_info *p_client)
   curl_easy_setopt(curl, CURLOPT_WRITEDATA, &web);
   curl_easy_setopt(curl, CURLOPT_USERAGENT, "haibbo speedtest-cli");
   curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1L);
-  //curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
+
+#ifdef DEBUG_SPEEDTEST
+  curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
+#endif
+
   res = curl_easy_perform(curl);
   curl_easy_cleanup(curl);
+  
   if (res != CURLE_OK) {
 #ifdef DEBUG_SPEEDTEST
     printf("curl_easy_perform() failed: %s\n", curl_easy_strerror(res));
@@ -734,6 +823,8 @@ cleanup:
   return rv;
 }
 
+/* ***************************************** */
+
 static int get_closest_server()
 {
   CURL *curl;
@@ -742,19 +833,32 @@ static int get_closest_server()
   struct web_buffer web;
   struct server_info server;
   int rv = NOK;
-
+  char useragent[16];
+  const char *url = "https://www.speedtest.net/speedtest-servers.php";
+  double v = (double) (rand() % 1000) / 100;
+  
   memset(&web, 0, sizeof(web));
-
+  sprintf(useragent, "curl/%.02f.0", v);
+  
   curl = curl_easy_init();
 
-  curl_easy_setopt(curl, CURLOPT_URL, "http://www.speedtest.net/speedtest-servers.php");
+#ifdef DEBUG_SPEEDTEST
+  printf("Retrieving servers list %s\n", url);
+#endif
+
+  curl_easy_setopt(curl, CURLOPT_URL, url);
   curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);
   curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_web_buf);
   curl_easy_setopt(curl, CURLOPT_WRITEDATA, &web);
-  curl_easy_setopt(curl, CURLOPT_USERAGENT, "ntopng");
-  //curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
+  curl_easy_setopt(curl, CURLOPT_USERAGENT, useragent);
+
+#ifdef DEBUG_SPEEDTEST
+  curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
+#endif
+  
   res = curl_easy_perform(curl);
   curl_easy_cleanup(curl);
+  
   if (res != CURLE_OK) {
 #ifdef DEBUG_SPEEDTEST
     printf("curl_easy_perform() failed: %s\n", curl_easy_strerror(res));
@@ -781,6 +885,8 @@ cleanup:
   return rv;
 }
 
+/* ***************************************** */
+
 static int get_best_server(int *p_index)
 {
   int     i;
@@ -793,24 +899,30 @@ static int get_best_server(int *p_index)
     if (minimum != DBL_MAX && i >= MIN_SERVERS_TO_CHECK)
       break; /* MIN_SERVERS_TO_CHECK evaluated and at least one found */
 
-    if (strlen(servers[i].url) == 0)
+    if((strlen(servers[i].url) == 0))
       continue;
 
     sscanf(servers[i].url, "http://%[^/]speedtest/upload.%*s", server);
+    if(server[0] == '\0')
+      sscanf(servers[i].url, "https://%[^/]speedtest/upload.%*s", server);
+    
+    if(server[0] == '\0')
+      continue;
+    
     latency = test_latency(server);
 
 #ifdef DEBUG_SPEEDTEST
-    //printf("Measured latency for %s is %0.3fms\n", server, latency);
+    printf("Measured latency for %s is %0.3fms\n", server, latency);
 #endif
-
+    
     if (minimum > latency ) {
       minimum = latency;
       *p_index = i;
 #ifdef DEBUG_SPEEDTEST
-      //printf("Best server set to %u (%s)\n", i, server);
+      printf("Best server set to %u (%s)\n", i, server);
 #endif
     }
-  }
+  } /* for */
 
   if (minimum == DBL_MAX)
     return NOK;
@@ -818,6 +930,8 @@ static int get_best_server(int *p_index)
   return OK;
 }
 
+/* ***************************************** */
+
 json_object* speedtest() {
   int     num_thread;
   char    server_url[URL_LENGTH_MAX], ext[UPLOAD_EXT_LENGTH_MAX];
@@ -872,8 +986,13 @@ json_object* speedtest() {
   }
   
   sscanf(servers[sindex].url, "http://%[^/]/speedtest/upload.%4s", server_url, ext);
+
+  if(server_url[0] == '\0')
+    sscanf(servers[sindex].url, "https://%[^/]/speedtest/upload.%4s", server_url, ext);
+  
 #ifdef DEBUG_SPEEDTEST
-  printf("Best server: %s(%0.2fKM)\n", server_url, servers[sindex].distance);
+  printf("Best server: %s (%0.2fKM) [index: %u][%s]\n",
+	 server_url, servers[sindex].distance, sindex, servers[sindex].url);
 #endif
   json_object_object_add(rc, "server.url", json_object_new_string(server_url));
   json_object_object_add(rc, "server.distance", json_object_new_double(servers[sindex].distance));
